@@ -22,7 +22,10 @@
 /*----------------------------------------------------------------*/
 /**** Include these here for potential optimization for OpenMP ****/
 /*----------------------------------------------------------------*/
-/*! Pearson correlation of x[] and y[] (x and y are NOT modified. */
+/*! Pearson correlation of x[] and y[] (x and y are NOT modified.
+    And we know ahead of time that the time series have 0 mean
+    and L2 norm 1.
+*//*--------------------------------------------------------------*/
 
 float zm_THD_pearson_corr( int n, float *x , float *y ) /* inputs are */
 {                                                       /* zero mean  */
@@ -37,6 +40,9 @@ float zm_THD_pearson_corr( int n, float *x , float *y ) /* inputs are */
    return xy ;
 }
 
+/*----------------------------------------------------------------*/
+/* General correlation calculation. */
+
 #if 0
 float my_THD_pearson_corr( int n, float *x , float *y )
 {
@@ -239,6 +245,9 @@ int main( int argc , char *argv[] )
              "                 RAM on your system, this program will be very slow\n"
              "                 with or without '-mmap'.\n"
              "              ** This option won't work with NIfTI-1 (.nii) output!\n"
+#else
+             "\n"
+             "  -mmap is disabled at this time :-(\n"
 #endif
              "\n"
              "Notes:\n"
@@ -252,11 +261,15 @@ int main( int argc , char *argv[] )
              "        *** malloc error for dataset sub-brick\n"
              "      this means that the program ran out of memory when making\n"
              "      the output dataset.\n"
+#ifdef ALLOW_MMAP
+             "   ++ If this happens, you can try to use the '-mmap' option,\n"
+             "      and if you are lucky, the program may actually run.\n"
+#endif
              " * The program prints out an estimate of its memory usage\n"
              "    when it starts.  It also prints out a progress 'meter'\n"
              "    to keep you pacified.\n"
              " * This is a quick hack for Peter Bandettini. Now pay up.\n"
-             " * OpenMP-ized for Hang Joon Jo.  Where's my soju?\n"
+             " * OpenMP-ized for Hang Joon Jo.  Where's my baem-sul?\n"
              "\n"
              "-- RWCox - 31 Jan 2002 and 16 Jul 2010\n"
             ) ;
@@ -324,7 +337,7 @@ int main( int argc , char *argv[] )
       }
 
       if( strcmp(argv[nopt],"-polort") == 0 ){
-         int val = strtod(argv[++nopt],&cpt) ;
+         int val = (int)strtod(argv[++nopt],&cpt) ;
          if( *cpt != '\0' || val < -1 || val > 3 ){
             ERROR_exit("Illegal value after -polort!") ;
          }
@@ -348,7 +361,7 @@ int main( int argc , char *argv[] )
 
    xset = THD_open_dataset(argv[nopt]); CHECK_OPEN_ERROR(xset,argv[nopt]);
    if( DSET_NVALS(xset) < 3 )
-      ERROR_exit("Input dataset %s does not have 3 or more sub-bricks!",argv[nopt]) ;
+     ERROR_exit("Input dataset %s does not have 3 or more sub-bricks!",argv[nopt]) ;
    DSET_load(xset) ; CHECK_LOAD_ERROR(xset) ;
 
    /*-- compute mask array, if desired --*/
@@ -439,6 +452,10 @@ int main( int argc , char *argv[] )
        "Memory required = %.1f Mbytes for %d output sub-bricks",nb,nmask);
      if( nb > 2000.0 && (sizeof(void *) < 8 || sizeof(size_t) < 8) )
        ERROR_exit("Can't run on a 32-bit system!") ;
+#ifdef ALLOW_MMAP
+     if( nb > 1000.0 && !do_mmap )
+       INFO_message("If you run out of memory, try using the -mmap option.") ;
+#endif
    }
 
    tross_Make_History( "3dAutoTcorrelate" , argc,argv , cset ) ;
@@ -505,7 +522,7 @@ AFNI_OMP_START ;
 #pragma omp parallel if( nmask > 999 )
 {
    int ii,jj,kout , ithr,nthr , vstep,vii ;
-   float *xsar , *ysar ; short *car ;
+   float *xsar , *ysar ; short *car=NULL ;
 
 #ifdef USE_OMP
    ithr = omp_get_thread_num() ;
@@ -527,7 +544,8 @@ AFNI_OMP_START ;
 
       ii = imap[kout] ;  /* ii= source voxel (we know that ii is in the mask) */
 
-      if( ithr == 0 ){ vii++ ; if( vii%vstep == vstep/2 ) vstep_print(); }
+      if( ithr == 0 && vstep > 2 ) /* allow small dsets 16 Jun 2011 [rickr] */
+      { vii++ ; if( vii%vstep == vstep/2 ) vstep_print(); }
 
       /* get ref time series from this voxel */
 
@@ -550,13 +568,12 @@ AFNI_OMP_START ;
 
          car[jj] = (short)(10000.49f*corfun(nvals,xsar,ysar)) ;
 
-      } /* end of innter loop over voxels */
+      } /* end of inner loop over voxels */
 
 #ifdef ALLOW_MMAP
       if( do_mmap ){  /* copy results to disk mmap now */
         short *cout = ((short *)(cbrik)) + (int64_t)(nvox)*(int64_t)(kout) ;
-#pragma omp critical
-        { memcpy( cout , car , sizeof(short)*nvox ) ; }
+        AAmemcpy( cout , car , sizeof(short)*nvox ) ;
       }
 #endif
 
