@@ -56,7 +56,14 @@
 
 #define METH_CENTROMEAN   31  /* RWC 01 Nov 2010 */
 
-#define MAX_NUM_OF_METHS  32
+#define METH_CVARINV      32  /* RWC 09 Aug 2011 */
+#define METH_CVARINVNOD   33
+
+#define METH_ZCOUNT       34
+#define METH_NZMEDIAN     35   /* RCR 27 Jun 2012 */
+#define METH_SIGNED_ABSMAX 36  /* RCR 31 Aug 2012 */
+
+#define MAX_NUM_OF_METHS  37
 
 /* allow single inputs for some methods (test as we care to add) */
 #define NUM_1_INPUT_METHODS 4
@@ -82,7 +89,9 @@ static char *meth_names[] = {
    "ArgAbsMax"     , "Sum"          , "Duration"      , "Centroid"    ,
    "CentDuration"  , "Absolute Sum" , "Non-zero Mean" , "Onset"       ,
    "Offset"        , "Accumulate"   , "SS"            , "BiwtMidV"    ,
-   "ArgMin+1"      , "ArgMax+1"     , "ArgAbsMax+1"   , "CentroMean"
+   "ArgMin+1"      , "ArgMax+1"     , "ArgAbsMax+1"   , "CentroMean"  ,
+   "CVarInv"       , "CvarInv (NOD)", "ZeroCount"     , "NZ Median"   ,
+   "Signed Absmax"
 };
 
 static void STATS_tsfunc( double tzero , double tdelta ,
@@ -95,25 +104,10 @@ static int Calc_duration(float *ts, int npts, float vmax, int max_index,
    int *onset, int *offset);
 static float Calc_centroid(float *ts, int npts);
 
-
-int main( int argc , char *argv[] )
+void usage_3dTstat(int detail) 
 {
-   THD_3dim_dataset *old_dset , *new_dset ;  /* input and output datasets */
-   THD_3dim_dataset *mask_dset=NULL  ;
-   float mask_bot=666.0 , mask_top=-666.0 ;
-   byte *cmask=NULL ; int ncmask=0 ;
-   byte *mmm   = NULL ;
-   int mcount=0, verb=0;
-   int nopt, nbriks, ii ;
-   int addBriks = 0 ;   /* n-1 sub-bricks out */
-   int fullBriks = 0 ;  /* n   sub-bricks out */
-   int tsout = 0 ;      /* flag to output a time series (not a stat bucket) */
-   int numMultBriks,methIndex,brikIndex;
 
-   /*----- Help the pitiful user? -----*/
-
-   if( argc < 2 || strcasecmp(argv[1],"-help") == 0 ){
-      printf(
+     printf(
 "Usage: 3dTstat [options] dataset\n"
  "Computes one or more voxel-wise statistics for a 3D+time dataset\n"
  "and stores them in a bucket dataset.  If no statistic option is\n"
@@ -132,22 +126,25 @@ int main( int argc , char *argv[] )
  "             [      the slope has been removed]\n"
  " -cvar   = compute coefficient of variation of input\n"
  "             voxels = stdev/fabs(mean)\n"
- "   **N.B.: You can add NOD to the end of the above 2\n"
- "           options only, to turn off detrending, as in\n"
- "             -stdevNOD  and/or  -cvarNOD\n"
+ " -cvarinv= 1.0/cvar = 'signal to noise ratio' [for Vinai]\n"
+ "           **N.B.: You can add NOD to the end of the above 3\n"
+ "                   options only, to turn off detrending, as in\n"
+ "                     -stdevNOD  and/or  -cvarNOD  and/or  -cvarinvNOD\n"
  "\n"
- " -MAD    = compute MAD (median absolute deviation) of\n"
- "             input voxels = median(|voxel-median(voxel)|)\n"
- "             [N.B.: the trend is NOT removed for this]\n"
- " -DW    = compute Durbin-Watson Statistic of input voxels\n"
- "             [N.B.: the trend IS removed for this]\n"
- " -median = compute median of input voxels  [undetrended]\n"
- " -bmv    = compute biweight midvariance of input voxels [undetrended]\n"
- "             [actually is 0.989*sqrt(biweight midvariance), to make]\n"
- "             [the value comparable to the standard deviation output]\n"
- " -min    = compute minimum of input voxels [undetrended]\n"
- " -max    = compute maximum of input voxels [undetrended]\n"
+ " -MAD       = compute MAD (median absolute deviation) of\n"
+ "                input voxels = median(|voxel-median(voxel)|)\n"
+ "                [N.B.: the trend is NOT removed for this]\n"
+ " -DW        = compute Durbin-Watson Statistic of input voxels\n"
+ "                [N.B.: the trend IS removed for this]\n"
+ " -median    = compute median of input voxels  [undetrended]\n"
+ " -nzmedian  = compute median of non-zero input voxels [undetrended]\n"
+ " -bmv       = compute biweight midvariance of input voxels [undetrended]\n"
+ "                [actually is 0.989*sqrt(biweight midvariance), to make]\n"
+ "                [the value comparable to the standard deviation output]\n"
+ " -min       = compute minimum of input voxels [undetrended]\n"
+ " -max       = compute maximum of input voxels [undetrended]\n"
  " -absmax    = compute absolute maximum of input voxels [undetrended]\n"
+ " -signed_absmax = (signed) value with absolute maximum [undetrended]\n"
  " -argmin    = index of minimum of input voxels [undetrended]\n"
  " -argmin1   = index + 1 of minimum of input voxels [undetrended]\n"
  " -argmax    = index of maximum of input voxels [undetrended]\n"
@@ -164,6 +161,7 @@ int main( int argc , char *argv[] )
  "              (sum(i*f(i)) / sum(f(i)))\n"
  " -centduration = compute duration using centroid's index as center\n"
  " -nzmean    = compute mean of non-zero voxels\n"
+ " -zcount    = count number of zero values at each voxel\n"
  "\n"
  " -autocorr n = compute autocorrelation function and return\n"
  "               first n coefficients\n"
@@ -252,22 +250,46 @@ int main( int argc , char *argv[] )
   "option, then the output will be written into a NIML-formatted 1D\n"
   "dataset, which you might find slightly confusing (but still usable).\n"
  ) ;
-      PRINT_COMPILE_DATE ; exit(0) ;
-   }
+   
+   PRINT_COMPILE_DATE ;  
+   
+   return;
+}
+
+int main( int argc , char *argv[] )
+{
+   THD_3dim_dataset *old_dset , *new_dset ;  /* input and output datasets */
+   THD_3dim_dataset *mask_dset=NULL  ;
+   float mask_bot=666.0 , mask_top=-666.0 ;
+   byte *cmask=NULL ; int ncmask=0 ;
+   byte *mmm   = NULL ;
+   int mcount=0, verb=0;
+   int nopt, nbriks, ii ;
+   int addBriks = 0 ;   /* n-1 sub-bricks out */
+   int fullBriks = 0 ;  /* n   sub-bricks out */
+   int tsout = 0 ;      /* flag to output a time series (not a stat bucket) */
+   int numMultBriks,methIndex,brikIndex;
+
+   /*----- Help the pitiful user? -----*/
+
 
    /* bureaucracy */
-
    mainENTRY("3dTstat main"); machdep(); AFNI_logger("3dTstat",argc,argv);
    PRINT_VERSION("3dTstat"); AUTHOR("KR Hammett & RW Cox");
 
    /*--- scan command line for options ---*/
 
+   if (argc == 1) { usage_3dTstat(1); exit(0); } /* Bob's help shortcut */
    nopt = 1 ;
    nbriks = 0 ;
    nmeths = 0 ;
    verb = 0;
    while( nopt < argc && argv[nopt][0] == '-' ){
-
+      if (strcmp(argv[nopt], "-h") == 0 || strcmp(argv[nopt], "-help") == 0) {
+         usage_3dTstat(strlen(argv[nopt]) > 3 ? 2:1);
+         exit(0);
+      }
+      
       /*-- methods --*/
 
       if( strcasecmp(argv[nopt],"-centromean") == 0 ){ /* 01 Nov 2010 */
@@ -288,12 +310,24 @@ int main( int argc , char *argv[] )
          nopt++ ; continue ;
       }
 
+      if( strcasecmp(argv[nopt],"-nzmedian") == 0 ){
+         meth[nmeths++] = METH_NZMEDIAN ;
+         nbriks++ ;
+         nopt++ ; continue ;
+      }
+
       if( strcasecmp(argv[nopt],"-DW") == 0 ){
          meth[nmeths++] = METH_DW ;
          nbriks++ ;
          nopt++ ; continue ;
       }
 
+      if( strcasecmp(argv[nopt],"-zcount") == 0 ){
+         meth[nmeths++] = METH_ZCOUNT ;
+         nbriks++ ;
+         nopt++ ; continue ;
+      }
+
       if( strcasecmp(argv[nopt],"-MAD") == 0 ){
          meth[nmeths++] = METH_MAD ;
          nbriks++ ;
@@ -343,6 +377,12 @@ int main( int argc , char *argv[] )
          nopt++ ; continue ;
       }
 
+      if( strcasecmp(argv[nopt],"-cvarinv") == 0 ){
+         meth[nmeths++] = METH_CVARINV ;
+         nbriks++ ;
+         nopt++ ; continue ;
+      }
+
       if( strcasecmp(argv[nopt],"-stdevNOD") == 0 ||
           strcasecmp(argv[nopt],"-sigmaNOD") == 0   ){  /* 07 Dec 2001 */
 
@@ -357,6 +397,12 @@ int main( int argc , char *argv[] )
          nopt++ ; continue ;
       }
 
+      if( strcasecmp(argv[nopt],"-cvarinvNOD") == 0 ){
+         meth[nmeths++] = METH_CVARINVNOD ;
+         nbriks++ ;
+         nopt++ ; continue ;
+      }
+
       if( strcasecmp(argv[nopt],"-min") == 0 ){
          meth[nmeths++] = METH_MIN ;
          nbriks++ ;
@@ -375,6 +421,12 @@ int main( int argc , char *argv[] )
          nopt++ ; continue ;
       }
 
+      if( strcasecmp(argv[nopt],"-signed_absmax") == 0 ){
+         meth[nmeths++] = METH_SIGNED_ABSMAX ;
+         nbriks++ ;
+         nopt++ ; continue ;
+      }
+
       if( strcasecmp(argv[nopt],"-argmin") == 0 ){
          meth[nmeths++] = METH_ARGMIN ;
          nbriks++ ;
@@ -552,9 +604,16 @@ int main( int argc , char *argv[] )
 
       /*-- Quien sabe'? --*/
 
-      ERROR_exit("Unknown option: %s\n",argv[nopt]) ;
+      ERROR_message("Unknown option: %s\n",argv[nopt]) ;
+      suggest_best_prog_option(argv[0], argv[nopt]);
+      exit(1);
    }
 
+    if (argc < 2) {
+      ERROR_message("Too few options, use -help for details");
+      exit(1);
+    }
+
    /*--- If no options selected, default to single stat MEAN -- KRH ---*/
 
    if (nmeths == 0) nmeths = 1;
@@ -803,29 +862,36 @@ static void STATS_tsfunc( double tzero, double tdelta ,
 
       case METH_SLOPE: val[out_index] = ts_slope ; break ;
 
-      case METH_CVAR_NOD:
+      case METH_CVAR_NOD:     /* methods that depend on the mean and stdev */
       case METH_SIGMA_NOD:
+      case METH_CVARINVNOD:
       case METH_CVAR:
+      case METH_CVARINV:
       case METH_SIGMA:{
         register int ii ;
         register double sum ;
 
+        int mm = meth[meth_index] ;
+        int nod = (mm == METH_CVAR_NOD)  ||   /* no detrend flag */
+                  (mm == METH_SIGMA_NOD) ||
+                  (mm == METH_CVARINVNOD)  ;
+
         sum = 0.0 ;
-        if((meth[meth_index] == METH_CVAR) || (meth[meth_index] == METH_SIGMA )){
+        if( !nod ){   /* not no detrend ==> use detrended data */
           for( ii=0 ; ii < npts ; ii++ ) sum += ts_det[ii] * ts_det[ii] ;
-        } else {
+        } else {      /* use data as God gave it to us */
           for( ii=0 ; ii < npts ; ii++ ) sum += (ts[ii]-ts_mean)
                                                *(ts[ii]-ts_mean) ;
         }
 
-        sum = sqrt( sum/(npts-1) ) ;
+        sum = sqrt( sum/(npts-1.0) ) ;  /* stdev */
 
-        if((meth[meth_index] == METH_SIGMA) || (meth[meth_index] == METH_SIGMA_NOD))
+        if( mm == METH_SIGMA ||  mm == METH_SIGMA_NOD )
           val[out_index] = sum ;
-        else if( ts_mean != 0.0 )
-          val[out_index] = sum / fabs(ts_mean) ;
+        else if( mm == METH_CVAR || mm == METH_CVAR_NOD )
+          val[out_index] = (ts_mean != 0.0) ? sum/fabs(ts_mean) : 0.0 ;
         else
-          val[out_index] = 0.0 ;
+          val[out_index] = (sum     != 0.0) ? fabs(ts_mean)/sum : 0.0 ;
       }
       break ;
 
@@ -841,6 +907,21 @@ static void STATS_tsfunc( double tzero, double tdelta ,
       }
       break ;
 
+      case METH_NZMEDIAN:{      /* 27 Jun 2012 [rickr] */
+        float* ts_copy;
+        int    lind, lnzcount;
+        ts_copy = (float*)calloc(npts, sizeof(float));
+        /* replace memcpy with non-zero copy */
+        lnzcount=0;
+        for (lind=0; lind < npts; lind++)
+           if( ts[lind] ) ts_copy[lnzcount++] = ts[lind];
+        /* and get the result from the possibly shortened array */
+        if( lnzcount > 0 ) val[out_index] = qmed_float( lnzcount , ts_copy ) ;
+        else               val[out_index] = 0.0 ;
+        free(ts_copy);
+      }
+      break ;
+
       case METH_CENTROMEAN:{  /* 01 Nov 2010 */
         float* ts_copy;
         ts_copy = (float*)calloc(npts, sizeof(float));
@@ -889,6 +970,13 @@ static void STATS_tsfunc( double tzero, double tdelta ,
       }
       break ;
 
+      case METH_ZCOUNT:{  /* 05 Apr 2012 */
+        int ii , zc ;
+        for( ii=zc=0 ; ii < npts ; ii++ ) if( ts[ii] == 0.0f ) zc++ ;
+        val[out_index] = zc ;
+      }
+      break ;
+
       case METH_DW:{
          register int ii ;
          register float den=ts[0]*ts[0] ;
@@ -910,6 +998,7 @@ static void STATS_tsfunc( double tzero, double tdelta ,
       case METH_OFFSET:
       case METH_DURATION:
       case METH_ABSMAX:
+      case METH_SIGNED_ABSMAX:
       case METH_ARGMAX:
       case METH_ARGMAX1:
       case METH_ARGABSMAX:
@@ -918,7 +1007,7 @@ static void STATS_tsfunc( double tzero, double tdelta ,
       case METH_CENTDURATION:
       case METH_CENTROID:{
          register int ii, outdex=0 ;
-         register float vm=ts[0] ;
+         register float vm=ts[0];
          if ( (meth[meth_index] == METH_ABSMAX) ||
                (meth[meth_index] == METH_ARGABSMAX) ||
                (meth[meth_index] == METH_ARGABSMAX1)  ) {
@@ -929,6 +1018,16 @@ static void STATS_tsfunc( double tzero, double tdelta ,
                outdex = ii ;
              }
            }
+         } else if ( meth[meth_index] == METH_SIGNED_ABSMAX ) {
+           /* for P Hamilton    31 Aug 2012 [rickr] */
+           register float avm=fabs(vm);
+           for( ii=1 ; ii < npts ; ii++ ) {
+             if( fabs(ts[ii]) > avm ) {
+               vm = ts[ii] ;
+               avm = fabs(vm) ;
+               outdex = ii ;
+             }
+           }
          } else {
            for( ii=1 ; ii < npts ; ii++ ) {
              if( ts[ii] > vm ) {
@@ -940,6 +1039,7 @@ static void STATS_tsfunc( double tzero, double tdelta ,
        switch( meth[meth_index] ){
             default:
             case METH_ABSMAX:
+            case METH_SIGNED_ABSMAX:
             case METH_MAX:
                val[out_index] = vm ;
             break;
