@@ -11,7 +11,15 @@
 /*-- 18 Oct 1999: modified to implement -preserve option --*/
 
 THD_fvec3 AFNI_backward_warp_vector( THD_warp * warp , THD_fvec3 old_fv ) ;
-
+/*== Note any changes here should be reflected in both ================
+     plug_drawdset.c:DRAW_ttatlas_CB()
+     3dfractionize.c:main()
+  or updated to use the same function instead in the future....
+  Unless voting, this function doesn't really need to work this way;
+  it can instead use the 3dresample or future warp-on-demand way.
+  Resulting ROIs are different between this method and "Show Atlas Colors"
+==========================================================================
+*/
 int main( int argc , char * argv[] )
 {
    char *prefix = "fractionize" ;
@@ -35,7 +43,7 @@ int main( int argc , char * argv[] )
 
    int do_vote=0 ;          /* 18 Oct 1999 */
    int *vote_val = NULL ;
-   int  nvote_val , ivote , voter , vote_print=0 ;
+   int  nvote_val=0 , ivote , voter=0 , vote_print=0 ;
    byte  *vote_bout = NULL ;
    short *vote_sout = NULL ;
    float *vote_best = NULL ;
@@ -193,6 +201,7 @@ int main( int argc , char * argv[] )
             exit(1) ;
          }
          iarg++ ; continue ;
+         THD_make_cardinal(tset);    /* deoblique    21 Oct, 2011 [rickr] */
       }
 
       if( strcmp(argv[iarg],"-input") == 0 || strcmp(argv[iarg],"-iset") == 0 ){
@@ -205,6 +214,7 @@ int main( int argc , char * argv[] )
             fprintf(stderr,"** Can't open input %s\n",argv[iarg]) ;
             exit(1) ;
          }
+         THD_make_cardinal(iset);    /* deoblique    21 Oct, 2011 [rickr] */
          iarg++ ; continue ;
       }
 
@@ -218,6 +228,7 @@ int main( int argc , char * argv[] )
             fprintf(stderr,"** Can't open warp %s\n",argv[iarg]) ;
             exit(1) ;
          }
+         THD_make_cardinal(wset);    /* deoblique    21 Oct, 2011 [rickr] */
          iarg++ ; continue ;
       }
 
