@@ -27,15 +27,19 @@
 #include <math.h>
 
 #include "mrilib.h"
-#include "thd_atlas.h"
 
 static EDIT_options HI_edopt ;
 
 #define NBIN_SPECIAL 65536
+#define NBIN_DEFAULT 100
 #define BIG_NUMBER 9.999e+37
 
+#define HI_UNKNOWN 0
+#define HI_INTOUT 1
+#define HI_FLOATOUT 2
+
 static int   HI_nopt ;
-static int   HI_nbin = 100 ;
+static int   HI_nbin = -1 ;
 static int   HI_log  = 0 ;
 
 static int     HI_dind  = -1 ;   /* 23 Sep 1998 */
@@ -54,13 +58,23 @@ static double  HI_max = -BIG_NUMBER;
 static char *  HI_unq = NULL;
 static char *  HI_ni = NULL;
 
+static int     HI_datatype = HI_UNKNOWN; /* DRG Aug 2011 */
+
 static short * HI_roi_unq = NULL; /* ZSS March 2011 */
-static int     HI_N_roi_unq = 0; 
+static int     HI_N_roi_unq = 0;
 static THD_3dim_dataset *HI_roi=NULL;
 
+static int     HI_pdf = 0;        /* ZSS Sept 2012 */
+
+static int integral_dset(THD_3dim_dataset *dset, int iv_bot, int iv_top);
+static int minmax_dset(THD_3dim_dataset *dset, float *dmin, float *dmax,
+                       int iv_bot, int iv_top);
+
 #define KEEP(x) ( (HI_nomit==0) ? 1 :  \
                   (HI_nomit==1) ? ((x) != HI_omit[0]) : HI_keep(x) )
 
+#define CEIL_CHECK(x) ( use_ceil ? ceil(x) : (x) )
+
 void HI_read_opts( int , char ** ) ;
 #define HI_syntax(str) \
   do{ fprintf(stderr,"\n** ERROR: %s\a\n",str) ; exit(1) ; } while(0)
@@ -76,89 +90,113 @@ int HI_keep(float x)  /* check if value x is in the omitted list */
 
 /*---------------------------------------------------------------------------------*/
 
+void usage_3dhistog(int detail) {
+         printf(
+"Compute histogram of 3D Dataset\n"
+"Usage: 3dhistog [editing options] [histogram options] dataset\n"
+"\n"
+"The editing options are the same as in 3dmerge\n"
+" (i.e., the options starting with '-1').\n"
+"\n"
+"The histogram options are:\n"
+"  -nbin #   Means to use '#' bins [default=100]\n"
+"            Special Case: for short or byte dataset bricks,\n"
+"                          set '#' to zero to have the number\n"
+"                          of bins set by the brick range.\n"
+"  -dind i   Means to take data from sub-brick #i, rather than #0\n"
+"  -omit x   Means to omit the value 'x' from the count;\n"
+"              -omit can be used more than once to skip multiple values.\n"
+"  -mask m   Means to use dataset 'm' to determine which voxels to use\n"
+"  -roi_mask r Means to create a histogram for each non-zero value in \n"
+"              dataset 'r'. If -mask option is also used, dataset 'r' is \n"
+"              masked by 'm' before creating the histograms.\n"
+"  -doall    Means to include all sub-bricks in the calculation;\n"
+"              otherwise, only sub-brick #0 (or that from -dind) is used.\n"
+"  -notitle  Means to leave the title line off the output.\n"
+"  -log10    Output log10() of the counts, instead of the count values.\n"
+"            This option cannot be used with -pdf or with -prefix\n"
+"  -pdf      Output the counts divided by the number of samples.\n"
+"            This option is only valid with -prefix\n"
+"  -min x    Means specify minimum (inclusive) of histogram.\n"
+"  -max x    Means specify maximum (inclusive) of histogram.\n"
+"  Output options for integer and floating point data\n"
+"  By default, the program will determine if the data is integer or float\n"
+"   even if the data is stored as shorts with a scale factor.\n"
+"   Integer data will be binned by default to be 100 or the maximum number of\n"
+"   integers in the range, whichever is less. For example, data with the range\n"
+"   (0..20) gives 21 bins for each integer, and non-integral bin boundaries\n"
+"   will be raised to the next integer (2.3 will be changed to 3, for instance).\n"
+"   If the number of bins is higher than the number of integers in the range,\n"
+"   the bins will be labeled with floating point values, and multiple bins\n"
+"   may be zero between the integer values\n"
+"   Float data will be binned by default to 100 bins with absolute limits for\n"
+"   the min and max if these are specified as inclusive. For example,\n"
+"   float data ranging from (0.0 to 20.0) will be binned into bins that\n"
+"   are 0.2 large  (0..0.199999, 0.2..0.399999,...,19.8..20.0)\n"
+"   To have bins divided at 1.0 instead, specify the number of bins as 20\n"
+"   Bin 0 is 0..0.9999, Bin 1 is 1.0 to 1.9999, ..., Bin 20 is 19 to 20.0000\n"
+"   giving a slight bias to the last bin\n"
+"  -int      Treat data and output as integers\n"
+"  -float    Treat data and output as floats\n"
+"  -unq U.1D Writes out the sorted unique values to file U.1D.\n"
+"            This option is not allowed for float data\n"
+"            If you have a problem with this, write\n"
+"            Ziad S. Saad (saadz@mail.nih.gov)\n"
+"  -prefix HOUT: Write a copy of the histogram into file HOUT.1D\n"
+"                you can plot the file with:\n"
+"             1dplot -sepscl -x HOUT.1D'[0]' HOUT.1D'[1,2]' \n"
+"        or   \n"
+"             1dRplot -input HOUT.1D\n"
+"\n"
+"Without -prefix, the histogram is written to stdout.  \n"
+"Use redirection '>' if you want to save it to a file.\n"
+"The format is a title line, then three numbers printed per line:\n"
+"  bottom-of-interval  count-in-interval  cumulative-count\n"
+"\n"
+"-- by RW Cox (V Roopchansingh added the -mask option)\n"
+         ) ;
+
+      if (detail) printf("\n" MASTER_SHORTHELP_STRING ) ;
+
+      PRINT_COMPILE_DATE ; 
+
+   return;
+}
+
 int main( int argc , char * argv[] )
 {
    int iarg, i_roi_unq;
    THD_3dim_dataset *dset ;
 
    int nx,ny,nz , nxyz , ii , kk , nopt , nbin ;
-   float fbot , ftop ;
-   int   ibot , itop , has_fac; /* to deal with multiple short sub-bricks */
-   int *fbin=NULL , *tbin=NULL ;
+   float fbot , ftop, temp_fbot, temp_ftop ;
+   /* removed ibot, itop              10 Jun 2011 [rcr,drg] */
+   int   use_ceil;
+   void *vfim = NULL;
+   int64_t *fbin=NULL ;
    float df , dfi ;
    float fval ;
 
    float fimfac;
    int iv_fim, fim_type;
-   byte    *bfim = NULL ;
-   short   *sfim = NULL ;
-   float   *ffim = NULL ;
-   void    *vfim = NULL ;
-   long cumfbin, cumtbin;
+   int64_t cumfbin;
    int iv_bot , iv_top ;
-   float vbot , vtop ;
    FILE *fout = NULL;
    int n_unq=0;
 
-   if( argc < 2 || strncmp(argv[1],"-help",4) == 0 ){
-      printf(
-   "Compute histogram of 3D Dataset\n"
-   "Usage: 3dhistog [editing options] [histogram options] dataset\n"
-   "\n"
-   "The editing options are the same as in 3dmerge\n"
-   " (i.e., the options starting with '-1').\n"
-   "\n"
-   "The histogram options are:\n"
-   "  -nbin #   Means to use '#' bins [default=100]\n"
-   "            Special Case: for short or byte dataset bricks,\n"
-   "                          set '#' to zero to have the number\n"
-   "                          of bins set by the brick range.\n"
-   "  -dind i   Means to take data from sub-brick #i, rather than #0\n"
-   "  -omit x   Means to omit the value 'x' from the count;\n"
-   "              -omit can be used more than once to skip multiple values.\n"
-   "  -mask m   Means to use dataset 'm' to determine which voxels to use\n"
-   "  -roi_mask r Means to create a histogram for each non-zero value in \n"
-   "              dataset 'r'. If -mask option is also used, dataset 'r' is \n"
-   "              masked by 'm' before creating the histograms.\n"
-   "  -doall    Means to include all sub-bricks in the calculation;\n"
-   "              otherwise, only sub-brick #0 (or that from -dind) is used.\n"
-   "  -notit    Means to leave the title line off the output.\n"
-   "  -log10    Output log10() of the counts, instead of the count values.\n"
-   "  -min x    Means specify minimum of histogram.\n"
-   "  -max x    Means specify maximum of histogram.\n"
-   "  -unq U.1D Writes out the sorted unique values to file U.1D.\n"
-   "            This option is not allowed for float data\n"
-   "            If you have a problem with this, write\n"
-   "            Ziad S. Saad (saadz@mail.nih.gov)\n"
-   "  -prefix HOUT: Write a copy of the histogram into file HOUT.1D\n"
-   "                you can plot the file with:\n"
-   "             1dplot -sepscl -x HOUT.1D'[0]' HOUT.1D'[1,2]' \n"
-   "        or   \n"
-   "             1dRplot -input HOUT.1D\n"
-   "\n"
-   "Without -prefix, the histogram is written to stdout.  \n"
-   "Use redirection '>' if you want to save it to a file.\n"
-   "The format is a title line, then three numbers printed per line:\n"
-   "  bottom-of-interval  count-in-interval  cumulative-count\n"
-   "\n"
-   "-- by RW Cox (V Roopchansingh added the -mask option)\n"
-         ) ;
-
-      printf("\n" MASTER_SHORTHELP_STRING ) ;
-
-      PRINT_COMPILE_DATE ; exit(0) ;
-   }
 
    mainENTRY("3dhistog") ; machdep() ; AFNI_logger("3dhistog",argc,argv) ;
    PRINT_VERSION("3dhistog") ;
 
+   if(argc == 1){ usage_3dhistog(1); exit(0); } /* Bob's help shortcut */
+
    HI_read_opts( argc , argv ) ;
    nopt = HI_nopt ;
 
    if( nopt >=  argc ) HI_syntax("no dset argument?") ;
 
-   fbin = (int *) calloc( sizeof(int) , HI_nbin ) ;
-   if( fbin == NULL ) HI_syntax("can't allocate histogram array!") ;
+   /* check if user has requested number of bins */
+   nbin = (HI_nbin < 0) ? NBIN_DEFAULT : HI_nbin ;
 
    iarg = nopt ;
    dset = THD_open_dataset( argv[iarg] ) ;
@@ -186,46 +224,71 @@ int main( int argc , char * argv[] )
    } else {
      iv_bot = (HI_dind >= 0) ? HI_dind
                              : DSET_IS_MASTERED(dset) ? 0
-                                                      : DSET_PRINCIPAL_VALUE(dset) ;
+                                        : DSET_PRINCIPAL_VALUE(dset) ;
      iv_top = iv_bot ;
      if( iv_bot < 0 || iv_bot >= DSET_NVALS(dset) )
        ERROR_exit("Sub-brick index %d out of range for dataset %s",
                   iv_bot , argv[iarg] ) ;
    }
+
+   /* assume all sub-bricks have the same type */
    fim_type = DSET_BRICK_TYPE(dset,iv_bot) ;
 
    /* find global min and max of data in all used bricks */
 
-   fbot = BIG_NUMBER ; ftop = -fbot ;
-   itop = -32768 ; ibot = 32767 ;
-   has_fac = 0 ;
-   for( iv_fim=iv_bot ; iv_fim <= iv_top ; iv_fim++ ){
-     vbot = mri_min( DSET_BRICK(dset,iv_fim) ) ;
-     vtop = mri_max( DSET_BRICK(dset,iv_fim) ) ;
-     fimfac = DSET_BRICK_FACTOR(dset,iv_fim) ; if (fimfac == 0.0)  fimfac = 1.0;
+   fbot = BIG_NUMBER ;
+   ftop = -fbot ;
 
-     /* if short, get range before applying factor */
-     if( fim_type == MRI_short || fim_type == MRI_byte ){
-        if( fimfac != 1.0 ) has_fac = 1 ;
-        if( vbot < ibot ) ibot = vbot ;
-        if( vtop > itop ) itop = vtop ;
-     }
-
-     vbot *= fimfac ; vtop *= fimfac ;
-     if( vbot < fbot ) fbot = vbot;
-     if( vtop > ftop ) ftop = vtop;
+   /* assume rounding of bin bottom limit for integer data only*/
+   if(HI_datatype == HI_UNKNOWN){
+      if(integral_dset(dset, iv_bot, iv_top)) {
+         HI_datatype = HI_INTOUT;
+      }
+      else
+      HI_datatype = HI_FLOATOUT;
    }
 
-   if(HI_min != BIG_NUMBER)
-     fbot = HI_min;
+   if(HI_datatype == HI_INTOUT) {
+     use_ceil = 1 ;
+   }
+   else {
+      use_ceil = 0 ;
+   }
 
-   if(HI_max != -BIG_NUMBER)
+   minmax_dset(dset, &temp_fbot, &temp_ftop, iv_bot, iv_top);
+
+   if(HI_min != BIG_NUMBER) {
+     fbot = HI_min;
+   }
+   else {
+     fbot = temp_fbot;
+   }
+
+   if(HI_max != -BIG_NUMBER) {
      ftop = HI_max;
+   }
+   else {
+     ftop = temp_ftop;
+   }
 
    if( fbot >= ftop ){
      fprintf(stderr,"** ERROR: all values in dataset are = %f!\n",fbot) ;
      exit(1) ;
    }
+
+   if( fbot > temp_ftop){
+     WARNING_message(
+        "minimum value to display is higher than maximum value in dataset!");
+   }
+
+   if( ftop < temp_fbot){
+     WARNING_message(
+        "maximum value to display is less than minimum value in dataset!");
+   }
+
+   if((HI_nbin<0) && (HI_datatype == HI_INTOUT))
+      nbin = ftop-fbot+1;
+
    switch( fim_type ){
       default:
         fprintf(stderr,"** ERROR: can't process data of this type!\n") ;
@@ -233,21 +296,27 @@ int main( int argc , char * argv[] )
 
       case MRI_byte:
       case MRI_short:
-        nbin = (int)(itop-ibot+1.0) ;  /* ftop -> stop (for unscaled range) */
-        if( nbin > HI_nbin ) nbin = HI_nbin ;
-        if( nbin < 2       ) nbin = 2 ;
+        /* 10 Jun 2011 */
+/*        if((HI_nbin==NBIN_SPECIAL) && use_ceil && ((ftop-fbot)<100)) {*/
+        /* integral data, calculate the default number of bins differently */
+/*        if((HI_nbin<0) && (use_ceil)) {
+               nbin = ftop-fbot+1;
+        }*/
       break ;
 
       case MRI_float:
-        nbin = (HI_nbin==NBIN_SPECIAL) ? 100 : HI_nbin ;
         if (HI_unq) {
          fprintf(stderr,"** ERROR: Unique operation not allowed for float data.\n") ;
-      exit(1) ;
+         exit(1) ;
         }
       break ;
    }
-   df  = (ftop-fbot) / (nbin-1) ;
-   dfi = 1.0 / df ;
+   df  = (ftop-fbot) / nbin ;
+   if(df==0.0) dfi = 1;
+   else dfi = 1.0 / df ;
+
+   fbin = (int64_t *) calloc( sizeof(int64_t) , nbin ) ;
+   if( fbin == NULL ) HI_syntax("can't allocate histogram array!") ;
 
    if (HI_roi) {
       if (DSET_NVOX(HI_roi) != nxyz) {
@@ -266,7 +335,7 @@ int main( int argc , char * argv[] )
          HI_mask = (byte *)calloc(sizeof(byte), nxyz);
       }
    }
-   
+
    i_roi_unq = 0;
    do {
       if (HI_roi) {
@@ -274,7 +343,7 @@ int main( int argc , char * argv[] )
          if (!HI_roi_unq[i_roi_unq]) {
             ++i_roi_unq;
             continue; /* skipping zero ROI */
-         } 
+         }
          /* prepare mask */
          HI_mask_hits = 0;
          for( ii=0 ; ii < nxyz ; ii++ ){
@@ -285,7 +354,7 @@ int main( int argc , char * argv[] )
             }
          }
          /* flush fbin */
-         memset(fbin, 0, sizeof(int)*HI_nbin);
+         memset(fbin, 0, sizeof(int64_t)*nbin);
          /* fprintf(stderr,"Processing ROI %d\n", HI_roi_unq[i_roi_unq]); */
       }
       /* loop over all bricks and accumulate histogram */
@@ -295,7 +364,7 @@ int main( int argc , char * argv[] )
             fclose(fout);
             if (!THD_ok_overwrite()) {
                fprintf(stderr,
-                        "** ERROR: Output file %s exists, will not overwrite.\n", 
+                        "** ERROR: Output file %s exists, will not overwrite.\n",
                         HI_unq) ;
                exit(1) ;
             }
@@ -314,6 +383,7 @@ int main( int argc , char * argv[] )
      fimfac = DSET_BRICK_FACTOR(dset,iv_fim) ;
      if (fimfac == 0.0)  fimfac = 1.0;
      vfim = DSET_ARRAY(dset,iv_fim) ;
+     fim_type = DSET_BRICK_TYPE(dset,iv_fim) ;
 
         switch( fim_type ){
 
@@ -326,6 +396,7 @@ int main( int argc , char * argv[] )
               if( (fval >= fbot && fval <= ftop) &&
                    KEEP(fval) && (HI_mask == NULL || HI_mask[ii]) ){
                 kk = (int)( (fval-fbot)*dfi ) ; /* use real value */
+                if(kk>=nbin) kk = nbin-1;
                 fbin[kk]++ ;
               }
             }
@@ -336,7 +407,7 @@ int main( int argc , char * argv[] )
                   exit(1) ;
                }
                fprintf(fout,"# %d unique values in %s\n", n_unq, argv[iarg] );
-               for (ii=0; ii<n_unq; ++ii) 
+               for (ii=0; ii<n_unq; ++ii)
                   if (KEEP(funq[ii])) fprintf(fout,"%d\n", funq[ii]);
                fclose(fout); fout = NULL;
                free(funq); funq = NULL;
@@ -352,6 +423,7 @@ int main( int argc , char * argv[] )
               if( (fval >= fbot && fval <= ftop) &&
                    KEEP(fval) && (HI_mask == NULL || HI_mask[ii]) ){
                 kk = (int)( (fval-fbot)*dfi ) ;
+                if(kk>=nbin) kk = nbin-1;
                 fbin[kk]++ ;
               }
             }
@@ -362,7 +434,7 @@ int main( int argc , char * argv[] )
                   exit(1) ;
                }
                fprintf(fout,"# %d unique values in %s\n", n_unq, argv[iarg] );
-               for (ii=0; ii<n_unq; ++ii) 
+               for (ii=0; ii<n_unq; ++ii)
                   if (KEEP(funq[ii])) fprintf(fout,"%d\n", funq[ii]);
                fclose(fout); fout = NULL;
                free(funq); funq = NULL;
@@ -373,10 +445,11 @@ int main( int argc , char * argv[] )
           case MRI_float:{
             float *fim = (float *)vfim ;
             for( ii=0 ; ii < nxyz ; ii++ ){
-	      fval = fim[ii]*fimfac ;  /* thanks to Tom Holroyd for noticing this*/
+              fval = fim[ii]*fimfac ;  /* thanks to Tom Holroyd for noticing this*/
               if( (fval >= fbot && fval <= ftop) &&
                    KEEP(fval) && (HI_mask == NULL || HI_mask[ii]) ){
                 kk = (int)( (fval-fbot)*dfi ) ;
+                if(kk>=nbin) kk = nbin-1;
                 fbin[kk]++ ;
               }
             }
@@ -391,16 +464,20 @@ int main( int argc , char * argv[] )
 
       if (!HI_ni) {
          cumfbin = 0;
-
          if( HI_log ){
            if( ! HI_notit )
-             printf ("%12s %13s %13s\n", 
+             printf ("%12s %13s %13s\n",
                      "#Magnitude", "Log_Freq", "Log_Cum_Freq");
 
            for( kk=0 ; kk < nbin ; kk++ ){
              cumfbin += fbin[kk];
-             printf ("%12.6f %13.6f %13.6f\n",
-                     fbot+kk*df,
+             if((use_ceil) && (nbin <= (ftop-fbot+1)))
+                printf ("%12d %13.6f %13.6f\n",
+                     (int) ceil(fbot+kk*df),
+                     log10((double)fbin[kk]+1.0), log10((double)cumfbin+1.0));
+             else
+                printf ("%12.6f %13.6f %13.6f\n",
+                     (fbot+kk*df),
                      log10((double)fbin[kk]+1.0), log10((double)cumfbin+1.0));
            }
          } else {
@@ -409,32 +486,51 @@ int main( int argc , char * argv[] )
 
            for( kk=0 ; kk < nbin ; kk++ ){
              cumfbin += fbin[kk];
-             printf ("%12.6f %13d %13ld\n",
-                     fbot+kk*df, fbin[kk], cumfbin);
+             if((use_ceil) && (nbin <= (ftop-fbot+1)))
+                printf ("%12d %13lld %13lld\n",
+                     (int) ceil(fbot+kk*df), (long long)fbin[kk], (long long)cumfbin);
+             else
+                printf ("%12.6f %13lld %13lld\n",
+                     (fbot+kk*df), (long long)fbin[kk], (long long)cumfbin);
            }
          }
       } else { /*                ZSS Dec. 2010 */
          NI_stream ns=NULL;
          NI_element *hni=NULL;
          char sstr[strlen(HI_ni)+64];
-         float *bb=(float*)calloc(nbin, sizeof(float));
-         double *cf=(double*)calloc(nbin, sizeof(double));
-         cumfbin = 0.0;
+         float *bb =(float* )calloc(nbin,sizeof(float) );
+         double *cf=(double*)calloc(nbin,sizeof(double));
+         double *bd=(double*)calloc(nbin,sizeof(double));
+         
+         if (HI_log) {
+            ERROR_message("Option -log10 not available with -prefix");
+            exit(1);
+         }
+         
+         cumfbin = 0 ;
          for( kk=0 ; kk < nbin ; kk++ ){
           cumfbin += fbin[kk];
-          cf[kk] = cumfbin;
-          bb[kk] = fbot+kk*df;
+          cf[kk] = (double)cumfbin;
+          bd[kk] = (double)fbin[kk];
+          bb[kk] = CEIL_CHECK(fbot+kk*df);
+         }
+         if (HI_pdf) {
+            for( kk=0 ; kk < nbin ; kk++ ){
+               bd[kk] /= cf[nbin-1];
+               cf[kk] /= cf[nbin-1];
+            }
          }
          hni = NI_new_data_element("3dhistog", nbin);
-         NI_add_column(hni, NI_FLOAT, bb); 
-         NI_add_column(hni, NI_INT, fbin);
+         NI_add_column(hni, NI_FLOAT , bb);
+         NI_add_column(hni, NI_DOUBLE, bd);
          NI_add_column(hni, NI_DOUBLE, cf);
-         if (HI_log) 
-            NI_set_attribute(hni, "ColumnLabels", 
-                        "Magnitude ; Log_Freq ; Log_Cum_Freq");
-         else 
-            NI_set_attribute(hni, "ColumnLabels", 
+         
+              if (!HI_log && !HI_pdf)
+            NI_set_attribute(hni, "ColumnLabels",
                         "Magnitude ; Freq ; Cum_Freq");
+         else if (!HI_log && HI_pdf)
+            NI_set_attribute(hni, "ColumnLabels",
+                        "Magnitude ; PDF ; CDF");
          sprintf(sstr,"%f",df);
          NI_set_attribute(hni, "BinWidth", sstr);
 
@@ -450,7 +546,7 @@ int main( int argc , char * argv[] )
          NI_write_element(ns,hni,NI_TEXT_MODE | NI_HEADERSHARP_FLAG );
          NI_stream_close(ns);
          NI_free_element(hni);
-         free(bb); free(cf);
+         free(bb); free(cf); free(bd);
       }
       ++i_roi_unq;
    } while (i_roi_unq < HI_N_roi_unq);
@@ -475,13 +571,19 @@ int main( int argc , char * argv[] )
 void HI_read_opts( int argc , char * argv[] )
 {
    int nopt = 1 ;
-   float val ;
-   int  ival , kk ;
+   int  ival;
 
    INIT_EDOPT( &HI_edopt ) ;
    HI_unq = NULL;
-   while( nopt < argc && argv[nopt][0] == '-' ){
+   HI_datatype = HI_UNKNOWN;
 
+   while( nopt < argc && argv[nopt][0] == '-' ){
+      
+      if( strcmp(argv[nopt],"-help") == 0 || strcmp(argv[nopt],"-h") == 0){
+        usage_3dhistog(strlen(argv[nopt])>3 ? 2:1);
+        exit(0);
+      }
+      
       /**** check for editing options ****/
 
       ival = EDIT_check_argv( argc , argv , nopt , &HI_edopt ) ;
@@ -489,8 +591,8 @@ void HI_read_opts( int argc , char * argv[] )
 
       if( strncmp(argv[nopt],"-nbin",5) == 0 ){
         HI_nbin = strtol( argv[++nopt] , NULL , 10 ) ;
-        if( HI_nbin < 10 && HI_nbin != 0 ) HI_syntax("illegal value of -nbin!") ;
         if( HI_nbin == 0 ) HI_nbin = NBIN_SPECIAL ;
+        else if( HI_nbin < 1 ) HI_syntax("illegal value of -nbin!") ;
         nopt++ ; continue ;
       }
       if( strncmp(argv[nopt],"-unq",4) == 0 ){
@@ -525,22 +627,27 @@ void HI_read_opts( int argc , char * argv[] )
         nopt++ ; continue ;
       }
 
-      if( strncmp(argv[nopt],"-notit",5) == 0 ){
+      if( strncmp(argv[nopt],"-notit",6) == 0 ){
          HI_notit = 1 ;
          nopt++ ; continue ;
       }
 
-      if( strncmp(argv[nopt],"-log10",5) == 0 ){
+      if( strncmp(argv[nopt],"-log10",6) == 0 ){
          HI_log = 1 ;
          nopt++ ; continue ;
       }
 
-      if( strncmp(argv[nopt],"-prefix",5) == 0 ){
+      if( strncmp(argv[nopt],"-prefix",6) == 0 ){
          nopt++ ;
-         if( nopt >= argc ) HI_syntax("need argument after -prefix!") ;         
+         if( nopt >= argc ) HI_syntax("need argument after -prefix!") ;
          HI_ni = argv[nopt] ;
          nopt++ ; continue ;
       }
+
+      if( strncmp(argv[nopt],"-pdf",6) == 0 ){
+         HI_pdf = 1 ;
+         nopt++ ; continue ;
+      }
       /* ----- -mask ----- */
 
       if( strncmp(argv[nopt],"-mask",5) == 0 )
@@ -567,10 +674,10 @@ void HI_read_opts( int argc , char * argv[] )
           nopt++ ; continue ;
 
       }
-      
+
       if( strncmp(argv[nopt],"-roi_mask",5) == 0 )
       {
-          THD_3dim_dataset * rset ; int ii,mc ;
+          THD_3dim_dataset * rset ;
           nopt++ ;
 
           if( nopt >= argc ) HI_syntax("need argument after -roi_mask!") ;
@@ -581,25 +688,25 @@ void HI_read_opts( int argc , char * argv[] )
           if (DSET_NVALS(rset)!=1) {
             HI_syntax("-roi_mask must have one sub-brick");
           }
-          HI_roi = EDIT_empty_copy(rset); 
-          EDIT_dset_items( HI_roi ,   
-                          ADN_nvals, 1, 
-                          ADN_ntt, 1, 
-                          ADN_malloc_type , DATABLOCK_MEM_MALLOC ,   
-                          ADN_none ) ; 
-          EDIT_substscale_brick( HI_roi, 0, DSET_BRICK_TYPE(rset,0), 
+          HI_roi = EDIT_empty_copy(rset);
+          EDIT_dset_items( HI_roi ,
+                          ADN_nvals, 1,
+                          ADN_ntt, 1,
+                          ADN_malloc_type , DATABLOCK_MEM_MALLOC ,
+                          ADN_none ) ;
+          EDIT_substscale_brick( HI_roi, 0, DSET_BRICK_TYPE(rset,0),
                                  DSET_ARRAY(rset,0), MRI_short, 1.0);
-          /* There can be a leak here if rset is not short 
+          /* There can be a leak here if rset is not short
              But that's OK */
           /* Now get unique values */
-          HI_roi_unq = UniqueShort(DSET_ARRAY(HI_roi,0), DSET_NVOX(HI_roi), 
+          HI_roi_unq = UniqueShort(DSET_ARRAY(HI_roi,0), DSET_NVOX(HI_roi),
                                     &HI_N_roi_unq, 0);
           if (!HI_roi_unq) {
              fprintf(stderr,"** ERROR: Failed to uniquate.\n") ;
              exit(1) ;
           }
-          fprintf(stdout,"# %d unique values in roi_mask %s\n", 
-                              HI_roi_unq[0] ? HI_N_roi_unq:HI_N_roi_unq-1, 
+          fprintf(stdout,"# %d unique values in roi_mask %s\n",
+                              HI_roi_unq[0] ? HI_N_roi_unq:HI_N_roi_unq-1,
                               argv[nopt] );
           nopt++ ; continue ;
       }
@@ -615,10 +722,21 @@ void HI_read_opts( int argc , char * argv[] )
          nopt++ ; continue ;
       }
 
+      if( strcmp(argv[nopt],"-int") == 0 ){
+         HI_datatype = HI_INTOUT ;
+         nopt++ ; continue ;
+      }
+
+      if( strcmp(argv[nopt],"-float") == 0 ){
+         HI_datatype = HI_FLOATOUT ;
+         nopt++ ; continue ;
+      }
+
 
       /**** unknown switch ****/
 
       fprintf(stderr,"*** unrecognized option %s\a\n",argv[nopt]) ;
+      suggest_best_prog_option(argv[0], argv[nopt]);
       exit(-1) ;
 
    }  /* end of loop over options */
@@ -626,7 +744,27 @@ void HI_read_opts( int argc , char * argv[] )
 #ifdef HIDEBUG
 printf("*** finished with options\n") ;
 #endif
+   
+   if( argc < 2 ){
+      ERROR_message("Too few options, try -help for details");
+      exit(1);
+   }
+   
+   if (HI_log && HI_pdf) {
+      ERROR_message("Option -log10 not available with -pdf");
+      exit(1);
+   }
 
+   if (HI_log && HI_ni) {
+      ERROR_message("Option -log10 not available with -prefix");
+      exit(1);
+   }
+   
+   if (HI_pdf && !HI_ni) {
+      ERROR_message("Option -pdf needs -prefix");
+      exit(1);
+   }
+   
    if( HI_doall ){
      if( HI_dind >= 0 ){
        fprintf(stderr,"** WARNING: -dind ignored with -doall!\n") ;
@@ -637,3 +775,54 @@ printf("*** finished with options\n") ;
    HI_nopt = nopt ;
    return ;
 }
+
+/* do simple check to see if data is all integer */
+static int
+integral_dset(THD_3dim_dataset *dset, int iv_bot, int iv_top)
+{
+   int iv_fim, fim_type;
+   float fimfac;
+
+   for( iv_fim=iv_bot ; iv_fim <= iv_top ; iv_fim++ ){
+     fim_type = DSET_BRICK_TYPE(dset,iv_fim) ;
+     fimfac = DSET_BRICK_FACTOR(dset,iv_fim) ;
+     if (fimfac == 0.0)  fimfac = 1.0;
+
+     /* test if all the data is integral  - used for calculating bins below */
+     if( (fim_type == MRI_short) || (fim_type == MRI_byte )){
+        if( fimfac != 1.0 ) return(0) ;
+     }
+     if( fim_type == MRI_float) return(0);
+   }
+   /* made it through all sub-bricks without float factor or datatype */
+   /*  then it must be integer */
+   return(1);
+}
+
+/* get min and max of data in range of sub-bricks of dataset */
+static int
+minmax_dset(THD_3dim_dataset *dset, float *dmin, float *dmax, int iv_bot, int iv_top)
+{
+   int iv_fim;
+   float fimfac;
+   float vbot , vtop, temp_fbot, temp_ftop ;
+
+   temp_fbot = BIG_NUMBER ;
+   temp_ftop = -temp_fbot ;
+
+   for( iv_fim=iv_bot ; iv_fim <= iv_top ; iv_fim++ ){
+     /* minimum and maximum for sub-brick */
+     vbot = mri_min( DSET_BRICK(dset,iv_fim) ) ;
+     vtop = mri_max( DSET_BRICK(dset,iv_fim) ) ;
+     fimfac = DSET_BRICK_FACTOR(dset,iv_fim) ;
+     if (fimfac == 0.0)  fimfac = 1.0;
+     vbot *= fimfac ; vtop *= fimfac ;
+
+     /* update global min and max */
+     if( vbot < temp_fbot ) temp_fbot = vbot;
+     if( vtop > temp_ftop ) temp_ftop = vtop;
+   }
+   *dmin = temp_fbot;
+   *dmax = temp_ftop;
+   return(0);
+}
