@@ -149,14 +149,14 @@ plot.1D.demo.str <- function(demo=0) {
                col.nozeros=TRUE, 
                col.grp=c(rep(1,2), rep(2,1)), 
                col.ystack=FALSE, grp.label=c('CSF', 'GM'),
-               oneplot=TRUE,
+               plotmode=1,
                prefix = 't3.jpg', verb = 1)"
    }else if (demo==4) {
       s <- "plot.1D( dmat = plot.1D.testmat(100, 3), 
                col.nozeros=TRUE, 
                col.grp=c(rep(1,2), rep(2,1)), 
                col.ystack=FALSE, grp.label=c('CSF', 'GM'),
-               oneplot=TRUE, leg.show = TRUE,
+               plotmode=1, leg.show = TRUE,
                prefix = 't4.jpg', verb = 1)"
    }else {
       err.AFNI("No such demo")
@@ -193,6 +193,9 @@ plot.1D.setupdevice <- function (P) {
       err.AFNI("Nothing to do");
       return(0)
    }
+   if (is.null(P$img.width)) { P$img.width<- 2000 }
+   if (is.null(P$img.height)) { P$img.height<- 2000 }
+   
    #Note, there are other ways to control graph size, see 
    #?par for pin and fin
    if (!is.null(P$prefix) && P$nodisp) { #render to device directly
@@ -270,8 +273,10 @@ plot.1D.save <- function (prefix='plot.pdf', dev=NULL) {
    }
    return(1)
 }
+
 plot.1D.puttitle <- function (P) {   
-   if (!is.null(P$ttl.main) || !is.null(P$ttl.sub)) {
+   if (!(is.null(P$ttl.main) || P$ttl.main == 'NONE') || 
+       !(is.null(P$ttl.sub) || P$ttl.sub == 'NONE')) {
       opar <- par();
       par(font.main = P$ttl.main.fontsize)
       par(font.sub = P$ttl.sub.fontsize) 
@@ -314,7 +319,9 @@ plot.1D.optlist <- function(...) {
    lldef <- list(dmat=NULL, dmat.err=NULL, dmat.colsel=NULL, 
             dmat.xval=NULL, dmat.TR = NULL, dmat.type = NULL,
             col.grp = NULL, col.ystack=FALSE, col.nozeros=FALSE, 
-            col.name=NULL, col.name.show=FALSE, col.name.x=NULL, col.name.y=NULL,
+            col.name=NULL, col.name.show=FALSE, 
+            col.name.x=NULL, col.name.y=NULL,
+            row.name=NULL,
             col.color = NULL, col.plot.char=NULL,
             col.plot.type ='l', col.line.type = 1, col.line.width=3,
             col.mean.line=FALSE,
@@ -322,7 +329,7 @@ plot.1D.optlist <- function(...) {
             ttl.main=NULL, ttl.main.fontsize = 10,
             ttl.sub=NULL, ttl.sub.fontsize = 10,
             prefix = NULL, showval=FALSE, save.Rdat=FALSE,
-            nodisp = FALSE, oneplot = FALSE, boxtype = 'n', multi.ncol=2,
+            nodisp = FALSE, plotmode = 2, boxtype = 'n', multi.ncol=2,
             NAval = 0, NANval = 0,
             colorset = seq(1,20),
             xax.lim=NULL, xax.step = NULL, xax.label=NULL, xax.tic.text = NULL, 
@@ -414,8 +421,8 @@ plot.1D.optlist <- function(...) {
                ll$multi.ncol <- 1
             if (!is.null(ll$col.ystack) && is.na(ll$col.ystack)) 
                ll$col.ystack <- TRUE
-            if (!is.null(ll$oneplot) && is.na(ll$oneplot)) 
-               ll$oneplot <- TRUE
+            if (!is.null(ll$plotmode) && is.na(ll$plotmode)) 
+               ll$plotmode <- 1
             if (!is.null(ll$col.text.lym) && is.na(ll$col.text.lym)) 
                ll$col.text.lym <- 'COL.NAME'
             if (!is.null(ll$col.text.rym) && is.na(ll$col.text.rym)) 
@@ -423,7 +430,7 @@ plot.1D.optlist <- function(...) {
          }
       } else { #Try information from dmat's attributes
         if (!is.null(nm <- attr(ll$dmat,"name"))) {
-           if (attr(ll$dmat,"name") == '3dhistog') {
+           if (attr(ll$dmat,"name") == '3dhistog' ) {
               if (dim(ll$dmat)[2] == 3) { 
                   #Don't go here if users send in a partial file
                   #like hist.1D[1,2]
@@ -435,7 +442,38 @@ plot.1D.optlist <- function(...) {
                      ll$xax.label <- colnames(ll$dmat)[1]
               }
            }
-        }   
+        }
+        if (!is.null(nm <- attr(ll$dmat,"name"))) {
+           if (attr(ll$dmat,"name") == 'seg_histogram' ) {
+              if (dim(ll$dmat)[2] == 3) { 
+                  #Don't go here if users send in a partial file
+                  #like hist.1D[1,2]
+                 if (!is.null(ll$dmat.xval) && is.na(ll$dmat.xval)) 
+                     ll$dmat.xval <- ll$dmat[,1]
+                 if (!is.null(ll$dmat.colsel) && is.na(ll$dmat.colsel)) 
+                     ll$dmat.colsel <- c(3) #Don't bother with non-normalized
+                 if (!is.null(ll$xax.label) && is.na(ll$xax.label)) 
+                    if (!is.null(attr(ll$dmat,"xlabel"))) {
+                        ll$xax.label <- attr(ll$dmat,"xlabel")
+                    } else {
+                       ll$xax.label <- colnames(ll$dmat)[1]
+                    }
+              }
+           }
+        } 
+        if (!is.null(nm <- attr(ll$dmat,"name"))) {
+           if (attr(ll$dmat,"name") == '3ddot' ||
+               length(grep('CorrMat',attr(ll$dmat,"name"))) ) {
+              if (dim(ll$dmat)[2] == dim(ll$dmat)[1] ) { 
+                 if (!is.null(ll$plotmode) && is.na(ll$plotmode)) 
+                    ll$plotmode <- 3
+                 if (!is.null(ll$row.name) && is.na(ll$row.name))
+                    ll$row.name <- colnames(ll$dmat)
+              }
+           }
+        }
+         
+           
       }
       #Now, apply all defaults for what remains uninitialized
       for (i in 1:length(ll)) 
@@ -535,7 +573,7 @@ plot.1D.eng <- function (P) {
    if (!is.null(P$dmat.err) && is.character(P$dmat.err)) {
       dmatv <- P$dmat.err
       for (i in 1:length(dmatv)) { #Won't work for different row numbers...
-         if (is.null(ffc <- read.AFNI.matrix(ffv[i]))) {
+         if (is.null(ffc <- read.AFNI.matrix(dmatv[i]))) {
             err.AFNI("Failed to read test file")
             return(0) 
          }
@@ -632,7 +670,7 @@ plot.1D.eng <- function (P) {
      
    #Set plot characters
    if(is.null(P$col.plot.char)) {
-      P$col.plot.char <- 19+P$col.color
+      P$col.plot.char <- 20+(P$col.color%%6)
    } else if (length(P$col.plot.char) != ncol(P$dmat)) {
       if (length(P$col.plot.char) == 1) {
          if (P$col.plot.char == -1) {
@@ -710,6 +748,10 @@ plot.1D.eng <- function (P) {
       P$col.name <- colnames(P$dmat)
    }
    
+   if (is.null(P$row.name)) {
+      P$row.name <- rownames(P$dmat)
+   }
+   
    if (P$col.name.show && is.null(P$col.name)) {
       err.AFNI("Want to show col.name but cannot find column names")
       return(0);
@@ -801,17 +843,17 @@ plot.1D.eng <- function (P) {
    }
    
    #Check if possible to do multi graph
-   if (!P$oneplot && length(P$dmat.colsel) > 10) {
+   if (P$plotmode == 2 && length(P$dmat.colsel) > 10) {
       warn.AFNI(paste("Too many columns for multi plots.\n",
                "Maximum allowed is 10, have ",length(P$dmat.colsel),"\n",
-               "Reverting to one plot mode"));
-      P$oneplot <- TRUE
+               "Reverting to one plot mode. Consider matrix mode."));
+      P$plotmode <- 1
    }
    
    #Also, multiplot does not behave well, meaning it does not call the 
-   #panel function if it is plotting just one colum, so force oneplot
+   #panel function if it is plotting just one colum, so force plotmode
    #for single column selections
-   if (length(P$dmat.colsel)==1) P$oneplot <- TRUE
+   if (length(P$dmat.colsel)==1) P$plotmode <- 1
    
    #Set Offset flag
    if (!is.null(P$col.ystack)) {
@@ -831,7 +873,7 @@ plot.1D.eng <- function (P) {
    }
 
    #Make sure col.ystack is allowed
-   if (!P$oneplot || length(P$dmat.colsel) == 1) {
+   if (P$plotmode != 1 || length(P$dmat.colsel) == 1) {
       if (P$col.ystack[1]) {
          note.AFNI("Stacking ignored, either multiplot or single column", 
                      callstr='');
@@ -910,10 +952,13 @@ plot.1D.eng <- function (P) {
       }
    }
    
-   if (P$oneplot) {
+   if (P$plotmode == 1) {
       if (P$verb) note.AFNI("Singleplotmode");
       tp = 'single'
       par(bty=P$boxtype)
+      #You can control the margins here if you like someday with:
+      #   par(omi = c(Bot, Left, Top, Right))
+      #   but you cannot use it to autocrop
       matplot(x=P$dmat.xval, P$mat2plt,             
            col = P$col.color[P$dmat.colsel], main = '',
            xlim=P$xax.lim[1:2], ylim=P$yax.lim[1:2], 
@@ -942,6 +987,7 @@ plot.1D.eng <- function (P) {
           
          #browser()
       }
+
       if (!is.null(P$xax.step) && is.null(P$xax.tic.text)) {
          axis(1,seq(from=P$xax.lim[1],to=P$xax.lim[2],by=P$xax.step));
       }
@@ -1006,8 +1052,8 @@ plot.1D.eng <- function (P) {
        
       if (!is.null(P$mat2plt.minus) && !is.null(P$mat2plt.plus)) {
          for (i in 1:length(P$dmat.colsel)) {
-            arrows(x0=x, y0=P$mat2plt[, i]-P$mat2plt.minus[,i], 
-                   x1=x, y1=P$mat2plt[, i]+P$mat2plt.plus[,i],
+            arrows(x0=P$dmat.xval, y0=P$mat2plt[, i]-P$mat2plt.minus[,i], 
+                   x1=P$dmat.xval, y1=P$mat2plt[, i]+P$mat2plt.plus[,i],
                    length=.05, angle=90, code = 3,
                    col = P$col.color[P$dmat.colsel[i]] ) 
          } 
@@ -1041,7 +1087,25 @@ plot.1D.eng <- function (P) {
                las=2, adj=0)
          par(ps = opar$ps)
       }
-   } else {
+   } else if (P$plotmode == 3) { # An image 
+      if (P$verb) note.AFNI("Matrix Mode");
+      if (is.null(P$col.name)) 
+         multinames <- paste('Series', P$dmat.colsel-1, sep='')       
+      else multinames <- P$col.name[P$dmat.colsel]
+      if (!is.null(P$row.name)) {
+         multirownames <- P$row.name 
+      } else multirownames <- paste('Row', P$dmat.colsel-1, sep='')    
+      
+      set.plot.1D.global.P(P)
+      colnames(P$mat2plt) <- multinames
+      rownames(P$mat2plt) <- multirownames
+      #Someday should compute margin width as a function of 
+      #row and col name lengths. For now, using oma which is
+      #in unit of lines. See also omi and omd
+      matrix.AFNI.show(P$mat2plt);
+      thisplot <- dev.cur()
+      P <- get.plot.1D.global.P()
+   } else if (P$plotmode == 2){
       if (P$verb) note.AFNI("Multiplotmode");
       tp = 'multiple'
       
@@ -1063,8 +1127,7 @@ plot.1D.eng <- function (P) {
          pch=P$col.plot.char[P$dmat.colsel]) 
       thisplot <- dev.cur()
       P <- get.plot.1D.global.P()
-   }
-   
+   }    
    if (P$grid.show) {
       grid()
    }
@@ -1203,7 +1266,7 @@ make.col.map <- function (fids=NULL, ncols=32, hex=FALSE) {
          }
       }
       #browser()
-      m[,j] <- r[1:ncols]
+      m[,j] <- r[ncols:1]
    }
    if (hex) m <- rgb(m)
    return(m)
@@ -1221,6 +1284,7 @@ matrix.AFNI.show <- function(x, ...){
      tt <- NULL
      xt <- NULL
      yt <- NULL
+     oma <- NULL
   # check for additional function arguments
   if( length(list(...)) ){
     Lst <- list(...)
@@ -1267,8 +1331,8 @@ matrix.AFNI.show <- function(x, ...){
          }
       }
     }
-    
   }
+  
 # check for null values
 if( is.null(xLabels) ){
    xLabels <- c(1:ncol(x))
@@ -1280,9 +1344,9 @@ if( is.null(yLabels) ){
 layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1), heights=c(1,1))
 
  # Red and green range from 0 to 1 while Blue ranges from 1 to 0
- ColorRamp <- rgb( seq(0,1,length=256),  # Red
-                   seq(0,1,length=256),  # Green
-                   seq(1,0,length=256))  # Blue
+ #ColorRamp <- rgb( seq(0,1,length=256),  # Red
+ #                  seq(0,1,length=256),  # Green
+ #                  seq(1,0,length=256))  # Blue
  ColorRamp <- make.col.map(ncols=256,hex=TRUE)
  ColorLevels <- seq(min, max, length=length(ColorRamp))
 
@@ -1293,6 +1357,12 @@ layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1), heights=c(1,1))
 
  # Data Map
  par(mar = c(3,5,2.5,2))
+   
+ #Adjust margins to accommodate width of strings
+   wwwc <- max(c(strwidth(colnames(x), 'inches')-0.8,0))
+   wwwr <- max(c(strwidth(rownames(x), 'inches')-1.2,0))
+   par(omi = c(wwwc, wwwr,0,0))
+   
  image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp, xlab="",
  ylab="", axes=FALSE, zlim=c(min,max))
  if( !is.null(title) ){
