@@ -22,6 +22,17 @@
    for (m_i=0; m_i<N_nmask; ++m_i) nmask[mask_record[m_i]] = 1;  \
 }
 
+typedef enum { NO_PRJ = -1,
+               E1_PLN_PRJ  , E2_PLN_PRJ, E3_PLN_PRJ, 
+               EZ_PLN_PRJ  , EY_PLN_PRJ, EX_PLN_PRJ, 
+               E1_DIR_PRJ  , E2_DIR_PRJ, E3_DIR_PRJ, 
+               EZ_DIR_PRJ  , EY_DIR_PRJ, EX_DIR_PRJ,
+               N_PRJ} SUMA_PC_PROJ; 
+
+typedef enum { NO_ROT = 0, ROT_2_Z = 1, ROT_2_Y, ROT_2_X } SUMA_PC_ROT;
+
+
+
 typedef enum { SUMA_SMOOTH_NOT_SET, SUMA_EQUAL, SUMA_FUJIWARA, SUMA_DESBRUN } SUMA_TAUBIN_SMOOTH_OPTIONS;
 
 static int SUMA_SSidbg=-1; /*!< Index of node for debug */
@@ -34,12 +45,17 @@ typedef struct {
 
 typedef struct {
    int N_TriIndex;            /*!< Number of triangles considered */
-   int *TriIndex;             /*!< vector (N_TriIndex x 1) of 1D indices of triangles considered */
-   int **IntersectedVoxels;   /*!< sparse matrix (N_TriIndex x N_InteresectedVoxels[i]) containing
-                              1D indices of voxels intersected by the triangle. For example,
-                              triangle of index TriIndex[i] intesects N_InteresectedVoxels[i]
-                              whose 1D indices are stored in IntersectedVoxels[i] */
-                              
+   int *TriIndex;             /*!< vector (N_TriIndex x 1) of 1D indices 
+                                   of triangles considered */
+   int **IntersectedVoxels;   /*!< sparse matrix (N_TriIndex x 
+                  N_InteresectedVoxels[i]) containing
+                  1D indices of voxels intersected by the triangle. For example,
+                  triangle of index TriIndex[i] intesects N_InteresectedVoxels[i]
+                  whose 1D indices are stored in IntersectedVoxels[i] */
+   float **SignedIJKDistance; /*!< sparse matrix (N_TriIndex x 
+                  N_InteresectedVoxels[i]) containing signed distance 
+                        *** in index units, NOT mm ***
+                  from triangle. */                           
    int *N_IntersectedVoxels;  /*!< Number of voxels intersected by some triangle */
 } SUMA_VTI; /*!< Voxel Triangle Intersection Structure. Create with SUMA_CreateVTI, Destroy with SUMA_FreeVTI*/
 
@@ -51,19 +67,30 @@ SUMA_Boolean SUMA_TriTrig(float *p1, float *p2, float *p3,
 
 SUMA_VTI *SUMA_CreateVTI(int N_TriIndex, int *TriIndex);
 SUMA_VTI * SUMA_FreeVTI(SUMA_VTI *vti);
-SUMA_VTI *SUMA_GetVoxelsIntersectingTriangle( SUMA_SurfaceObject *SO, SUMA_VOLPAR *VolPar, float *NodeIJKlist,
-                                                SUMA_VTI *vti);
-
-
-
-DList * SUMA_SPI_to_EdgeStrips(SUMA_SurfaceObject *SO, SUMA_SURF_PLANE_INTERSECT *SPI);
+SUMA_VTI *SUMA_GetVoxelsIntersectingTriangle( 
+         SUMA_SurfaceObject *SO, SUMA_VOLPAR *VolPar, float *NodeIJKlist,
+         SUMA_VTI *vti);
+THD_3dim_dataset *SUMA_SurfaceIntersectionVolume(
+                        SUMA_SurfaceObject *SOo, THD_3dim_dataset *gdset);
+THD_3dim_dataset *SUMA_VoxelToSurfDistances(SUMA_SurfaceObject *SO, 
+                     THD_3dim_dataset *master, byte *mask, 
+                     short *isin, short inval);
+THD_3dim_dataset *SUMA_VoxelizeSurface(SUMA_SurfaceObject *SO, 
+                                       THD_3dim_dataset *gdset, 
+                                       int closed_surf,
+                                       byte *mask);
+THD_3dim_dataset *SUMA_MaskizeSurface(SUMA_SurfaceObject *SO, 
+                                       THD_3dim_dataset *gdset, 
+                                       int method);
+DList * SUMA_SPI_to_EdgeStrips(SUMA_SurfaceObject *SO, 
+                               SUMA_SURF_PLANE_INTERSECT *SPI);
 SUMA_Boolean SUMA_isEdgeStripClosed(DList *edgestrip, SUMA_SurfaceObject *SO);
 int SUMA_isSelfIntersect(SUMA_SurfaceObject *SO, int FullCount, byte * report);
 int SUMA_VoxelNeighbors (int ijk, int ni, int nj, int nk, SUMA_VOX_NEIGHB_TYPES ntype, int *nl);
 byte *SUMA_FillToVoxelMask(byte *ijkmask, int ijkseed, int ni, int nj, int nk, int *N_in, byte *usethisisin); 
 SUMA_Boolean SUMA_VoxelsInBox(int *voxelsijk, int *N_in, float *c1, float *c2);
 SUMA_SurfaceObject *SUMA_Patch2Surf(float *NodeList, int N_NodeList, int *PatchFaces, int N_PatchFaces, int PatchDim);
-SUMA_PATCH * SUMA_getPatch (  int *NodesSelected, int N_Nodes, 
+SUMA_PATCH * SUMA_getPatch (  int *NodesSelected, int N_Nodes, int MaxNodeMax,
                               int *Full_FaceSetList, int N_Full_FaceSetList, 
                               SUMA_MEMBER_FACE_SETS *Memb, int MinHits,
                               int FixBowTie, int verb);
@@ -99,11 +126,18 @@ SUMA_CONTOUR_EDGES *SUMA_GetContour (SUMA_SurfaceObject *SO, int *Nodes,
                                      byte **isNodeInNodes, int verb);
 SUMA_Boolean SUMA_ShowPatch (SUMA_PATCH *Patch, FILE *Out) ;
 void SUMA_Set_OffsetDebugNode (int d);
-SUMA_Boolean SUMA_getoffsets  (int n, SUMA_SurfaceObject *SO, float *Off, float lim);
-SUMA_Boolean SUMA_getoffsets2 (int n, SUMA_SurfaceObject *SO, float lim, SUMA_GET_OFFSET_STRUCT *OffS, int * CoverThisNode, int N_CoverThisNode);
-DList * SUMA_getoffsets_ll (int n, SUMA_SurfaceObject *SO, float lim, int *CoverThisNode, int N_CoverThisNode);
+SUMA_Boolean SUMA_getoffsets  (int n, SUMA_SurfaceObject *SO, float *Off, 
+                               float lim);
+SUMA_Boolean SUMA_getoffsets2 (  int n, SUMA_SurfaceObject *SO, 
+                                 float lim, SUMA_GET_OFFSET_STRUCT *OffS, 
+                                 int * CoverThisNode, int N_CoverThisNode);
+byte *SUMA_NodeNeighborMask(SUMA_SurfaceObject *SO, int node, 
+                            int maxlay, int *N_inmask);
+DList * SUMA_getoffsets_ll (int n, SUMA_SurfaceObject *SO, float lim, 
+                            int *CoverThisNode, int N_CoverThisNode);
 SUMA_GET_OFFSET_STRUCT *SUMA_Initialize_getoffsets (int N_Node);
-SUMA_Boolean SUMA_AddNodeToLayer (int n, int LayInd, SUMA_GET_OFFSET_STRUCT *OffS);
+SUMA_Boolean SUMA_AddNodeToLayer (int n, int LayInd,
+                                  SUMA_GET_OFFSET_STRUCT *OffS);
 SUMA_GET_OFFSET_STRUCT * SUMA_Free_getoffsets (SUMA_GET_OFFSET_STRUCT *OffS);
 SUMA_Boolean SUMA_Recycle_getoffsets (SUMA_GET_OFFSET_STRUCT *OffS);
 float ** SUMA_CalcNeighbDist (SUMA_SurfaceObject *SO);
@@ -118,6 +152,8 @@ float * SUMA_Chung_Smooth (SUMA_SurfaceObject *SO, float **wgt,
 SUMA_Boolean SUMA_Chung_Smooth_dset (SUMA_SurfaceObject *SO, float **wgt, 
                            int N_iter, float FWHM, SUMA_DSET *dset, 
                            SUMA_COMM_STRUCT *cs, byte *nmask, byte strict_mask);
+SUMA_Boolean SUMA_DotNormals(SUMA_SurfaceObject *SO, float *dir, float **dots);
+
 /* NOTE THAT x passed to the macro must be in units of distance^2 */
 #define SUMA_CHUNG_KERNEL_NUMER(x,s) (exp(-(x)/(2.0*(s)*(s)))) 
 #define SUMA_FWHM_MEAN(fwhmv, N_fwhmv, meanfwhm, FWHM_mixmode, N) {\
@@ -177,13 +213,51 @@ float * SUMA_Taubin_Smooth (SUMA_SurfaceObject *SO, float **wgt,
                             int N_iter, int vpn, SUMA_INDEXING_ORDER d_order,
                             float *fout_user, SUMA_COMM_STRUCT *cs,
                             byte *nmask, byte strict_mask);
-SUMA_Boolean SUMA_ApplyAffine (float *NodeList, int N_Node, float M[][4], float *center);
+int SUMA_Taubin_Smooth_SO(  SUMA_SurfaceObject *SO, 
+                            SUMA_TAUBIN_SMOOTH_OPTIONS smopt, 
+                            float kpb, byte *mask, byte strict_mask,
+                            int Niter); 
+int SUMA_NN_Smooth_SO(   SUMA_SurfaceObject *SO, 
+                         byte *mask, byte strict_mask,
+                         int Niter);
+int SUMA_NN_GeomSmooth_SO(   SUMA_SurfaceObject *SO, 
+                         byte *nmask, byte strict_mask,
+                         int Niter);
+int SUMA_NN_GeomSmooth2_SO(   SUMA_SurfaceObject *SO, 
+                         byte *nmask, byte strict_mask,
+                         int Niter, int anchor_each,
+                              SUMA_SurfaceObject *SOe,
+                         float *anchor_loc, float *anchor_wght);
+int SUMA_NN_GeomSmooth3_SO(   SUMA_SurfaceObject *SO, 
+                         byte *nmask, byte strict_mask,
+                         int Niter, int anchor_each,
+                         SUMA_SurfaceObject *SOe,
+                         float *anchor_wght, THD_3dim_dataset *voxelize,
+                         SUMA_COMM_STRUCT *cs);
+SUMA_Boolean SUMA_ApplyAffine (float *NodeList, int N_Node, float M[][4], 
+                               float *center);
 float *SUMA_NN_GeomSmooth( SUMA_SurfaceObject *SO, int Niter, float *fin_orig, 
-                           int vpn, SUMA_INDEXING_ORDER d_order, float *fout_final_user,
+                           int vpn, SUMA_INDEXING_ORDER d_order, 
+                           float *fout_final_user,
                            SUMA_COMM_STRUCT *cs, byte *nmask, byte strict_mask);
-SUMA_Boolean SUMA_EquateSurfaceSize(SUMA_SurfaceObject *SO, SUMA_SurfaceObject *SOref, float max_off, SUMA_COMM_STRUCT *cs);
-SUMA_Boolean SUMA_EquateSurfaceVolumes(SUMA_SurfaceObject *SO, SUMA_SurfaceObject *SOref, float perc_tol, SUMA_COMM_STRUCT *cs);
-SUMA_Boolean SUMA_EquateSurfaceAreas(SUMA_SurfaceObject *SO, SUMA_SurfaceObject *SOref, float perc_tol, SUMA_COMM_STRUCT *cs);
+float *SUMA_NN_GeomSmooth2( SUMA_SurfaceObject *SO, int N_iter, float *fin_orig, 
+                           int vpn, SUMA_INDEXING_ORDER d_order, 
+                           float *fout_final_user,
+                           SUMA_COMM_STRUCT *cs, byte *nmask, int MaskEnforce,
+                           SUMA_SurfaceObject *SOe,
+                           float *anchor_loc, float *anchor_wght);
+float *SUMA_NN_GeomSmooth3( SUMA_SurfaceObject *SO, int N_iter, float *fin_orig, 
+                           int vpn, SUMA_INDEXING_ORDER d_order, 
+                           float *fout_final_user,
+                           SUMA_COMM_STRUCT *cs, byte *nmask, int MaskEnforce,
+                           SUMA_SurfaceObject *SOe,
+                           float *anchor_wght, THD_3dim_dataset *voxelize);
+SUMA_Boolean SUMA_EquateSurfaceSize(SUMA_SurfaceObject *SO, 
+               SUMA_SurfaceObject *SOref, float max_off, SUMA_COMM_STRUCT *cs);
+SUMA_Boolean SUMA_EquateSurfaceVolumes(SUMA_SurfaceObject *SO, 
+               SUMA_SurfaceObject *SOref, float perc_tol, SUMA_COMM_STRUCT *cs);
+SUMA_Boolean SUMA_EquateSurfaceAreas(SUMA_SurfaceObject *SO, 
+               SUMA_SurfaceObject *SOref, float perc_tol, SUMA_COMM_STRUCT *cs);
 double SUMA_Mesh_Volume(SUMA_SurfaceObject *SO, int *FSI, int N_FaceSet, 
                         int verb, int *prec_prob) ;
 double SUMA_Mesh_Area(SUMA_SurfaceObject *SO, int *FSI, int N_FaceSet) ;
@@ -192,17 +266,23 @@ double SUMA_Pattie_Volume (SUMA_SurfaceObject *SO1, SUMA_SurfaceObject *SO2,
                            SUMA_SurfaceObject *UseThisSO, 
                            int minhits, int FixBowTie, int adjust_contour,
                            byte *adj_N, int verb);
-double SUMA_NewAreaAtRadius(SUMA_SurfaceObject *SO, double r, double Rref, float *tmpList);
+double SUMA_NewAreaAtRadius(SUMA_SurfaceObject *SO, double r, double Rref, 
+                            float *tmpList);
 SUMA_Boolean SUMA_ProjectSurfaceToSphere(SUMA_SurfaceObject *SO, SUMA_SurfaceObject *SOref, float radius, SUMA_COMM_STRUCT *cs);
 SUMA_OFFSET_STRUCT *SUMA_FormNeighbOffset ( SUMA_SurfaceObject *SO, float OffsetLim, const char *opts, byte *nmask, float FWHM);
 SUMA_OFFSET_STRUCT * SUMA_free_NeighbOffset (SUMA_SurfaceObject *SO, SUMA_OFFSET_STRUCT *OffS_out);
-float *SUMA_Offset_GeomSmooth( SUMA_SurfaceObject *SO, int N_iter, float Offestlim, float *fin_orig, 
-                              int vpn, SUMA_INDEXING_ORDER d_order, float *fout_final_user,
-                              SUMA_COMM_STRUCT *cs);
-SUMA_Boolean SUMA_GetOffset2Offset (SUMA_GET_OFFSET_STRUCT *GOS, SUMA_OFFSET_STRUCT *OS);
+float *SUMA_Offset_GeomSmooth( SUMA_SurfaceObject *SO, int N_iter, 
+                               float Offestlim, float *fin_orig, 
+                               int vpn, SUMA_INDEXING_ORDER d_order, 
+                               float *fout_final_user,
+                               SUMA_COMM_STRUCT *cs);
+byte *SUMA_GetOffset2bytemask (SUMA_GET_OFFSET_STRUCT *GOS, byte *thismask);
+SUMA_Boolean SUMA_GetOffset2Offset (SUMA_GET_OFFSET_STRUCT *GOS, 
+                                    SUMA_OFFSET_STRUCT *OS);
 char * SUMA_ShowOffset_ll_Info (DList *list, int detail);
 char * SUMA_ShowOffset_Info (SUMA_GET_OFFSET_STRUCT *OffS, int detail);
-SUMA_Boolean SUMA_FixNN_Oversampling ( SUMA_SurfaceObject *SO, SUMA_DSET *dset, byte *nmask, 
+SUMA_Boolean SUMA_FixNN_Oversampling ( SUMA_SurfaceObject *SO, SUMA_DSET *dset, 
+                                       byte *nmask, 
                                        int icol, SUMA_Boolean MaskZeros); 
 
 /* Begin function prototypes for VolData.c */
@@ -211,9 +291,12 @@ THD_fvec3 SUMA_THD_3dfind_to_3dmm_vp( SUMA_VOLPAR *vp, THD_fvec3 iv );
 THD_fvec3 SUMA_THD_3dind_to_3dmm( SUMA_SurfaceObject *SO, THD_ivec3 iv );
 THD_fvec3 SUMA_THD_3dmm_to_3dfind( SUMA_SurfaceObject *SO , THD_fvec3 fv );
 THD_ivec3 SUMA_THD_3dmm_to_3dind( SUMA_SurfaceObject *SO  , THD_fvec3 fv );
-THD_ivec3 SUMA_THD_3dmm_to_3dind_warn( SUMA_SurfaceObject *SO  , THD_fvec3 fv, int *out );
-THD_fvec3 SUMA_THD_3dmm_to_dicomm( int xxorient, int yyorient, int zzorient , THD_fvec3 imv );
-THD_fvec3 SUMA_THD_dicomm_to_3dmm( SUMA_SurfaceObject *SO , THD_fvec3 dicv );
+THD_ivec3 SUMA_THD_3dmm_to_3dind_warn( SUMA_SurfaceObject *SO  , 
+                                       THD_fvec3 fv, int *out );
+THD_fvec3 SUMA_THD_3dmm_to_dicomm( int xxorient, int yyorient, int zzorient , 
+                                    THD_fvec3 imv );
+THD_fvec3 SUMA_THD_dicomm_to_3dmm( int xxorient, int yyorient, int zzorient , 
+                                    THD_fvec3 dicv );
 void SUMA_orcode_to_orstring (int xxorient, int yyorient, int zzorient, char *orstr);
 void SUMA_sizeto3d_2_deltaHEAD(THD_ivec3 orient, THD_fvec3 *delta);            
 void SUMA_originto3d_2_originHEAD(THD_ivec3 orient, THD_fvec3 *origin);
@@ -228,7 +311,15 @@ int SUMA_flip_orient(int xxorient);
 int SUMA_ok_orstring (char *orstr);
 SUMA_Boolean SUMA_orstring_to_orcode (char *orstr, int *orient);
 int SUMA_Subdivide_Mesh(float **NodeListp, int *N_Node, int **FaceSetListp, int *N_FaceSet, float maxarea);
-int SUMA_OrientTriangles (float *NodeList, int N_Node, int *FaceSetList, int N_FaceSet, int orient, int Force);
+SUMA_Boolean SUMA_FlipTriangles (int *FaceSetList,int N_FaceSet);
+SUMA_Boolean SUMA_FlipSOTriangles(SUMA_SurfaceObject *SO);
+int SUMA_OrientTriangles (float *NodeList, int N_Node, 
+                          int *FaceSetList, int N_FaceSet, 
+                          int orient, int Force, float *c, 
+                          byte *fliphappened);
+int SUMA_OrientSOTriangles(SUMA_SurfaceObject *SO,
+                           int orient, int Force,
+                           float *cu);
 SUMA_Boolean SUMA_Offset_Smooth_dset( SUMA_SurfaceObject *SO, 
                                           float FWHM, float OffsetLim, 
                                           int N_iter,
@@ -269,10 +360,19 @@ DList *SUMA_SliceAlongPlane(SUMA_SurfaceObject *SO, float *Eq, float step);
 SUMA_DSET *SUMA_RandomDset(int N_Node, int nc, unsigned int seed, float scale, byte norm); 
 
 SUMA_Boolean SUMA_FillRandXform(double xform[][4], int seed, int type); 
+SUMA_Boolean SUMA_FillScaleXform(double xform[][4], double sc[3]);
 
 float *SUMA_Project_Coords_PCA (float *xyz, int N_xyz, int iref, 
-                                int compnum, int rotate);
-                                
+                                SUMA_PC_PROJ compnum, SUMA_PC_ROT rotate);
+int SUMA_NodeDepth(float *NodeList, int N_Node, float **dpth, 
+                   float thr, byte **cmaskp);                                
+int SUMA_VoxelDepth(THD_3dim_dataset *dset, float **dpth,
+                    float thr, byte **cmaskp, int applymask);
+int SUMA_VoxelDepth_Z(THD_3dim_dataset *dset, float **dpth,
+                    float thr, byte **cmaskp, int applymask,
+                    float peakperc);
+int SUMA_VoxelPlaneCut(THD_3dim_dataset *dset, float *Eq,
+                       byte **cmaskp, int applymask);
 int SUMA_is_Flat_Surf_Coords_PCA (float *xyz, int N_xyz, 
                                   float tol, float sampfrac); 
 int SUMA_is_Constant_Z_Coord(float *NodeList, int N_Node, float tol);
