@@ -1,6 +1,6 @@
 #include "SUMA_suma.h"
 
-void usage_SurfToSurf (SUMA_GENERIC_ARGV_PARSE *ps)
+void usage_SurfToSurf (SUMA_GENERIC_ARGV_PARSE *ps, int detail)
 {
       static char FuncName[]={"usage_SurfToSurf"};
       char * s = NULL, *sio=NULL, *st = NULL, *sts = NULL;
@@ -91,6 +91,12 @@ void usage_SurfToSurf (SUMA_GENERIC_ARGV_PARSE *ps)
 "                      instead of the node normals of S1.\n"
 "                      Each row should contain one direction for each\n"
 "                      of the nodes forming S1.\n"
+"  -closest_possible OO: Flag allowing the substitution of the projection\n"
+"                        result with the closest node that could be found\n"
+"                        along any direction.\n"
+"                        0: Don't do that, direction results only.\n"
+"                        1: Use closest node if projection fails to hit target\n"
+"                        2: Use closest node if it is at a closer distance.\n"
 "  -make_consistent: Force a consistency check and correct triangle \n"
 "                    orientation of S1 if needed. Triangles are also\n"
 "                    oriented such that the majority of normals point\n"
@@ -106,6 +112,7 @@ void usage_SurfToSurf (SUMA_GENERIC_ARGV_PARSE *ps)
 "                     of SurfToSurf the next time around, assuming of course\n"
 "                     that the two surfaces involved are the same, and that \n"
 "                     only the input data differs.\n"
+"               MAP_INFO is also generated by MapIcosahedron.\n"
 "\n"
 "%s"
 "%s"
@@ -117,7 +124,8 @@ void usage_SurfToSurf (SUMA_GENERIC_ARGV_PARSE *ps)
       exit(0);
 }
 
-SUMA_GENERIC_PROG_OPTIONS_STRUCT *SUMA_SurfToSurf_ParseInput(char *argv[], int argc, SUMA_GENERIC_ARGV_PARSE *ps)
+SUMA_GENERIC_PROG_OPTIONS_STRUCT *SUMA_SurfToSurf_ParseInput(
+   char *argv[], int argc, SUMA_GENERIC_ARGV_PARSE *ps)
 {
    static char FuncName[]={"SUMA_BrainWrap_ParseInput"}; 
    SUMA_GENERIC_PROG_OPTIONS_STRUCT *Opt=NULL;
@@ -142,16 +150,19 @@ SUMA_GENERIC_PROG_OPTIONS_STRUCT *SUMA_SurfToSurf_ParseInput(char *argv[], int a
    Opt->in_name = NULL;
    Opt->out_prefix = NULL;
    Opt->fix_winding = 0;
+   Opt->iopt = 0;
+   Opt->oform = SUMA_NO_DSET_FORMAT;
    accepting_out = NOPE;
    while (kar < argc) { /* loop accross command ine options */
 		/*fprintf(stdout, "%s verbose: Parsing command line...\n", FuncName);*/
       
-      if (!brk && accepting_out) { /* make sure you have not begun with new options */
+      if (!brk && accepting_out) { 
+         /* make sure you have not begun with new options */
          if (*(argv[kar]) == '-') accepting_out = NOPE;
       }
 		
       if (strcmp(argv[kar], "-h") == 0 || strcmp(argv[kar], "-help") == 0) {
-			 usage_SurfToSurf(ps);
+			 usage_SurfToSurf(ps, strlen(argv[kar]) > 3 ? 2:1);
           exit (0);
 		}
 		
@@ -193,6 +204,24 @@ SUMA_GENERIC_PROG_OPTIONS_STRUCT *SUMA_SurfToSurf_ParseInput(char *argv[], int a
          brk = YUP;
       }
       
+      if (!brk && (strcmp(argv[kar], "-closest_possible") == 0))
+      {
+         if (kar+1 >= argc)
+         {
+            fprintf (SUMA_STDERR, "need a number after -closest_possible \n");
+            exit (1);
+         }
+         
+         Opt->iopt = atoi(argv[++kar]);
+         if (Opt->iopt != 0 && Opt->iopt != 1 && Opt->iopt != 2) {
+            SUMA_S_Errv("Must choose from 0, 1, or 2 for -closest_possible."
+                        " Have %d\n",
+                         Opt->iopt);
+            exit (1);
+         } 
+         brk = YUP;
+      }
+      
       if (!brk && (strcmp(argv[kar], "-output_params") == 0))
       {
          if (kar+1 >= argc)
@@ -297,7 +326,8 @@ SUMA_GENERIC_PROG_OPTIONS_STRUCT *SUMA_SurfToSurf_ParseInput(char *argv[], int a
             exit (1);
          }
          
-         Opt->out_prefix = SUMA_Extension(argv[++kar],".1D", YUP);
+         Opt->out_prefix = SUMA_RemoveDsetExtension_eng(argv[++kar],
+                                                      &(Opt->oform));
          brk = YUP;
       }
 
@@ -339,7 +369,8 @@ SUMA_GENERIC_PROG_OPTIONS_STRUCT *SUMA_SurfToSurf_ParseInput(char *argv[], int a
                   "Error %s:\n"
                   "Option %s not understood. Try -help for usage\n", 
                   FuncName, argv[kar]);
-			exit (1);
+			suggest_best_prog_option(argv[0], argv[kar]);
+         exit (1);
 		} else {	
 			brk = NOPE;
 			kar ++;
@@ -384,12 +415,13 @@ int main (int argc,char *argv[])
 	SUMAg_DOv = SUMA_Alloc_DisplayObject_Struct (SUMA_MAX_DISPLAYABLE_OBJECTS);
    ps = SUMA_Parse_IO_Args(argc, argv, "-i;-t;-spec;-s;-sv;-o;");
    
+   Opt = SUMA_SurfToSurf_ParseInput (argv, argc, ps);
    if (argc < 2) {
-      usage_SurfToSurf(ps);
+      SUMA_S_Err("Too few options");
+      usage_SurfToSurf(ps, 0);
       exit (1);
    }
    
-   Opt = SUMA_SurfToSurf_ParseInput (argv, argc, ps);
 
    /* if output surface requested, check on pre-existing file */
    if (ps->o_N_surfnames) {
@@ -404,8 +436,7 @@ int main (int argc,char *argv[])
    } 
    
    if (Opt->debug > 2) LocalHead = YUP;
-   
-   outname = SUMA_append_string(Opt->out_prefix,".1D");
+   outname = SUMA_append_extension(Opt->out_prefix,".1D");
    if (SUMA_filexists(outname) && !THD_ok_overwrite()) {
       fprintf(SUMA_STDERR,"Output file %s exists.\n", outname);
       exit(1);
@@ -455,11 +486,16 @@ int main (int argc,char *argv[])
       if (LocalHead) 
          fprintf(SUMA_STDERR,"%s: Checking orientation.\n", FuncName);
       orient = SUMA_OrientTriangles (SO1->NodeList, SO1->N_Node, 
-                                    SO1->FaceSetList, SO1->N_FaceSet, 1, 0);
+                                     SO1->FaceSetList, SO1->N_FaceSet, 
+                                     1, 0, NULL, NULL);
       if (orient < 0) { 
          /* flipping was done, dump the edge list since it is not 
             automatically updated (should do that in function, 
-            just like in SUMA_MakeConsistent,  shame on you) */ 
+            just like in SUMA_MakeConsistent,  shame on you) 
+            
+            If you revisit this section, use the newer:
+            SUMA_OrientSOTriangles
+         */ 
          if (SO1->EL) SUMA_free_Edge_List(SO1->EL); SO1->EL = NULL; 
          if (!SUMA_SurfaceMetrics(SO1, "EdgeList", NULL)) { 
             SUMA_SL_Err("Failed to create edge list for SO1"); exit(1);  
@@ -593,7 +629,8 @@ int main (int argc,char *argv[])
          /* When you are ready to work with dsets, you should 
          checkout the function morphDsetToStd. It uses M2M */
          im_data = mri_read_1D(Opt->in_name);
-         if (!im_data) { SUMA_SL_Err("Failed to read 1D file of data"); exit(1);}
+         if (!im_data) { 
+            SUMA_SL_Err("Failed to read 1D file of data"); exit(1);}
          far_data = MRI_FLOAT_PTR(im_data);
          nvec_data = im_data->nx;
          ncol_data = im_data->ny;
@@ -619,7 +656,7 @@ int main (int argc,char *argv[])
    if (!Opt->s) {
       SUMA_LH("Going for the mapping of SO1 --> SO2");
       M2M = SUMA_GetM2M_NN( SO1, SO2, nodeind, N_nodeind, 
-                            projdir, 0, Opt->NodeDbg);
+                            projdir, 0, Opt->NodeDbg, Opt->iopt);
       SUMA_S_Notev("Saving M2M into %s\n\n",
                Opt->out_prefix);
       if (!(SUMA_Save_M2M(Opt->out_prefix, M2M))) {
@@ -662,8 +699,8 @@ int main (int argc,char *argv[])
       }
    } else if (Opt->Data < 0) {
          SUMA_DSET *dset=NULL, *dseto=NULL;
-         char *oname=NULL, *uname=NULL;
-         int oform=SUMA_ASCII_NIML, iform=SUMA_NO_DSET_FORMAT;
+         char *oname=NULL, *uname=NULL, *s1=NULL, *s2=NULL;
+         int iform=SUMA_NO_DSET_FORMAT;
          if (Opt->NodeDbg>= 0) {
             SUMA_S_Notev("Processing dset %s\n", Opt->in_name);
          }
@@ -677,11 +714,15 @@ int main (int argc,char *argv[])
             SUMA_S_Errv("Failed to map %s\n", Opt->in_name);
             exit(1);
          }
-         uname = SUMA_append_replace_string(
-               SUMA_FnameGet(Opt->in_name,"pa", SUMAg_CF->cwd), 
-                             SUMA_FnameGet(Opt->in_name,"f",SUMAg_CF->cwd), 
-                             Opt->out_prefix, 0);
-         oname = SUMA_WriteDset_s (uname, dseto, oform, 1, 1);
+         s1 = SUMA_append_string(
+                  SUMA_FnameGet(Opt->in_name,"pa", SUMAg_CF->cwd),
+                  Opt->out_prefix); 
+         s2 = SUMA_RemoveDsetExtension_s(
+               SUMA_FnameGet(Opt->in_name,"l",SUMAg_CF->cwd), 
+               SUMA_NO_DSET_FORMAT);
+         uname = SUMA_append_extension(s1,s2);      
+         SUMA_free(s1); SUMA_free(s2);
+         oname = SUMA_WriteDset_s (uname, dseto, Opt->oform, 1, 1);
          if (Opt->NodeDbg>= 0) SUMA_S_Notev("Wrote %s\n", oname);
          if (oname) SUMA_free(oname); oname=NULL;
          if (uname) SUMA_free(uname); oname=NULL;
@@ -784,8 +825,11 @@ SS = SUMA_StringAppend_va(SS,
    fprintf(outptr,"%s\n",s); SUMA_free(s); s = NULL;
    
    /* put headers atop columns */
-   Nchar = 6; /* if you change this number you'll need to fix the formats below */
-   for (i=0; i<icol; ++i) { sprintf(sbuf,"#%s", MV_format_fval2(i, Nchar -1)); fprintf(outptr,"%6s   ", sbuf); }
+   Nchar = 6; /* if you change this number you'll need to fix  formats below */
+   for (i=0; i<icol; ++i) { 
+      sprintf(sbuf,"#%s", MV_format_fval2(i, Nchar -1)); 
+      fprintf(outptr,"%6s   ", sbuf); 
+   }
    fprintf(outptr,"\n");
    
    /* Now put in the values, make sure you parallel columns above! */
@@ -793,13 +837,17 @@ SS = SUMA_StringAppend_va(SS,
       fprintf(outptr,"%6s   ", MV_format_fval2(M2M->M1n[i], Nchar));
       if (Opt->NearestNode > 0) {
          for (j=0; j<Opt->NearestNode; ++j) { 
-            if (j < M2M->M2Nne_M1n[i]) fprintf(outptr,"%6s   ", MV_format_fval2(M2M->M2ne_M1n[i][j], Nchar)); 
+            if (j < M2M->M2Nne_M1n[i]) 
+               fprintf(outptr,"%6s   ", 
+                  MV_format_fval2(M2M->M2ne_M1n[i][j], Nchar)); 
             else fprintf(outptr,"%6s   ", "-1"); 
          } /* Neighboring nodes */
       } 
       if (Opt->NearestNode > 1) { /* add the weights */
          for (j=0; j<Opt->NearestNode; ++j) { 
-            if (j < M2M->M2Nne_M1n[i]) fprintf(outptr,"%6s   ", MV_format_fval2(M2M->M2we_M1n[i][j], Nchar)); 
+            if (j < M2M->M2Nne_M1n[i]) 
+               fprintf(outptr,"%6s   ", 
+                  MV_format_fval2(M2M->M2we_M1n[i][j], Nchar)); 
             else fprintf(outptr,"%6s   ", "0.0"); 
          } 
       }
@@ -820,7 +868,9 @@ SS = SUMA_StringAppend_va(SS,
             fprintf(outptr,"%6s   ", MV_format_fval2(dt[3*i+1], Nchar));
             fprintf(outptr,"%6s   ", MV_format_fval2(dt[3*i+2], Nchar));
          } else { /* Column major business */
-            for (j=0; j<ncol_data; ++j) { fprintf(outptr,"%6s   ", MV_format_fval2(dt[i+j*M2M->M1Nn], Nchar)); }
+            for (j=0; j<ncol_data; ++j) { 
+               fprintf(outptr,"%6s   ", 
+                     MV_format_fval2(dt[i+j*M2M->M1Nn], Nchar)); }
          }
       }
       fprintf(outptr,"\n");
@@ -832,8 +882,9 @@ SS = SUMA_StringAppend_va(SS,
       SUMA_LH("Writing surface");
       tmpfv = SO1->NodeList;
       SO1->NodeList = dt;
-      if (!SUMA_Save_Surface_Object (SO_name, SO1, ps->o_FT[0], ps->o_FF[0], NULL)) {
-         fprintf (SUMA_STDERR,"Error %s: Failed to write surface object.\n", FuncName);
+      if (!SUMA_Save_Surface_Object (SO_name, SO1, 
+                                     ps->o_FT[0], ps->o_FF[0], NULL)) {
+         SUMA_S_Err("Failed to write surface object.\n");
          exit (1);
       }
       SO1->NodeList = tmpfv; tmpfv = NULL;
@@ -842,7 +893,9 @@ SS = SUMA_StringAppend_va(SS,
    if (N_Spec) {
       int k=0; 
       for (k=0; k<N_Spec; ++k) {
-         if (!SUMA_FreeSpecFields(&(Spec[k]))) { SUMA_S_Err("Failed to free spec fields"); } 
+         if (!SUMA_FreeSpecFields(&(Spec[k]))) {
+            SUMA_S_Err("Failed to free spec fields");
+         } 
       }
       SUMA_free(Spec); Spec = NULL; N_Spec = 0;
    }
