@@ -41,7 +41,9 @@ const char *SUMA_WarpTypeName(SUMA_WARP_TYPES wt)
                     > 1 talk
    
 */
-int SUMA_ClosestNodeToVoxels(SUMA_SurfaceObject *SO, SUMA_VOLPAR *vp, int *closest_node, float *closest_dist, byte *vox_mask, int verb) 
+int SUMA_ClosestNodeToVoxels(SUMA_SurfaceObject *SO, SUMA_VOLPAR *vp, 
+                             int *closest_node, float *closest_dist, 
+                             byte *vox_mask, int verb) 
 {
    static char FuncName[]={"SUMA_ClosestNodeToVoxels"};
    float *p=NULL;
@@ -57,8 +59,10 @@ int SUMA_ClosestNodeToVoxels(SUMA_SurfaceObject *SO, SUMA_VOLPAR *vp, int *close
    }
    
    if (verb) {
-      fprintf(SUMA_STDERR,"%s: Have %d nodes in surface\n%dx%dx%d (%d) voxels in volume.\n", 
-                  FuncName, SO->N_Node, vp->nx, vp->ny, vp->nz, vp->nx * vp->ny * vp->nz);
+      fprintf(SUMA_STDERR, "%s: Have %d nodes in surface\n"
+                           "%dx%dx%d (%d) voxels in volume.\n", 
+                  FuncName, SO->N_Node, vp->nx, vp->ny, vp->nz, 
+                  vp->nx * vp->ny * vp->nz);
    }
    /* Now for each voxel, find the closest node (SLOW implementation) */
    cnt = 0;
@@ -73,7 +77,8 @@ int SUMA_ClosestNodeToVoxels(SUMA_SurfaceObject *SO, SUMA_VOLPAR *vp, int *close
             if (!vox_mask || (vox_mask && vox_mask[ijk])) {
                iv.xyz[0] = (float)i; iv.xyz[1] = (float)j; iv.xyz[2] = (float)k;    
                fv = SUMA_THD_3dfind_to_3dmm_vp(vp, iv);
-               iv = SUMA_THD_3dmm_to_dicomm(vp->xxorient, vp->yyorient, vp->zzorient,  fv);
+               iv = SUMA_THD_3dmm_to_dicomm(vp->xxorient, vp->yyorient, 
+                                            vp->zzorient,  fv);
                #if 0 /* macro not tested here so keeping older code below */
                SUMA_CLOSEST_NODE(SO, iv.xyz, closest_node[ijk], d);
                if (closest_dist) closest_dist[ijk] = (float)d;
@@ -88,12 +93,16 @@ int SUMA_ClosestNodeToVoxels(SUMA_SurfaceObject *SO, SUMA_VOLPAR *vp, int *close
                   }
                }
                #endif
-               if (closest_dist) { if (closest_dist[ijk] >= 0.0f) closest_dist[ijk] = (float)sqrt(closest_dist[ijk]); }
+               if (closest_dist) { 
+                  if (closest_dist[ijk] >= 0.0f) 
+                     closest_dist[ijk] = (float)sqrt(closest_dist[ijk]); }
                if (verb) {
                   ++cnt;
                   if (!(cnt % 1000)) {
                      fprintf(SUMA_STDERR,". @ %4d %4d %4d   (%3.2f%%)\n", 
-                              i, j, k, (float)cnt/(float)(vp->nx * vp->ny * vp->nz)*100.0); fflush(SUMA_STDERR);
+                             i, j, k, 
+                             (float)cnt/(float)(vp->nx * vp->ny * vp->nz)*100.0);
+                     fflush(SUMA_STDERR);
                   }
                }
             }
@@ -482,7 +491,7 @@ SUMA_Boolean SUMA_Free_VolPar (SUMA_VOLPAR *VP)
    static char FuncName[]={"SUMA_Free_VolPar"};
    
    SUMA_ENTRY;
-
+   if (!VP) SUMA_RETURN (YUP);
    if (VP->prefix != NULL) SUMA_free(VP->prefix);
    if (VP->idcode_str != NULL) SUMA_free(VP->idcode_str);
    if (VP->filecode != NULL) SUMA_free(VP->filecode);
@@ -908,6 +917,7 @@ SUMA_Boolean SUMA_Align_to_VolPar (SUMA_SurfaceObject *SO, void * S_Struct)
    switch (SO->FileType) {
       case SUMA_INVENTOR_GENERIC:
       case SUMA_OPENDX_MESH:
+      case SUMA_PREDEFINED:
       case SUMA_PLY:
       case SUMA_BYU:
       case SUMA_VEC:
@@ -1099,6 +1109,7 @@ SUMA_Boolean SUMA_Delign_to_VolPar (SUMA_SurfaceObject *SO, void * S_Struct)
    switch (SO->FileType) {
       case SUMA_INVENTOR_GENERIC:
       case SUMA_OPENDX_MESH:
+      case SUMA_PREDEFINED:
       case SUMA_PLY:
       case SUMA_BYU:
       case SUMA_VEC:
@@ -1881,7 +1892,7 @@ THD_fvec3 SUMA_THD_3dmm_to_dicomm( int xxorient, int yyorient, int zzorient,
    convert to input image oriented x,y,z from Dicom x,y,z
 -----------------------------------------------------------------------*/
 
-THD_fvec3 SUMA_THD_dicomm_to_3dmm( SUMA_SurfaceObject *SO ,
+THD_fvec3 SUMA_THD_dicomm_to_3dmm( int xxorient, int yyorient, int zzorient ,
                               THD_fvec3 dicv )
 {
    static char FuncName[]={"SUMA_THD_dicomm_to_3dmm"};
@@ -1892,7 +1903,7 @@ THD_fvec3 SUMA_THD_dicomm_to_3dmm( SUMA_SurfaceObject *SO ,
 
    xdic = dicv.xyz[0] ; ydic = dicv.xyz[1] ; zdic = dicv.xyz[2] ;
 
-   switch( SO->VolPar->xxorient ){
+   switch( xxorient ){
       case ORI_R2L_TYPE:
       case ORI_L2R_TYPE: xim = xdic ; break ;
       case ORI_P2A_TYPE:
@@ -1905,7 +1916,7 @@ THD_fvec3 SUMA_THD_dicomm_to_3dmm( SUMA_SurfaceObject *SO ,
          exit (1);
    }
 
-   switch( SO->VolPar->yyorient ){
+   switch( yyorient ){
       case ORI_R2L_TYPE:
       case ORI_L2R_TYPE: yim = xdic ; break ;
       case ORI_P2A_TYPE:
@@ -1918,7 +1929,7 @@ THD_fvec3 SUMA_THD_dicomm_to_3dmm( SUMA_SurfaceObject *SO ,
          exit (1);
    }
 
-   switch( SO->VolPar->zzorient ){
+   switch( zzorient ){
       case ORI_R2L_TYPE:
       case ORI_L2R_TYPE: zim = xdic ; break ;
       case ORI_P2A_TYPE:
@@ -2231,13 +2242,15 @@ SUMA_Boolean SUMA_vec_3dmm_to_3dfind (float *NodeList, int N_Node, SUMA_VOLPAR *
    SUMA_RETURN(YUP);
 }
 
-SUMA_Boolean SUMA_vec_dicomm_to_3dfind (float *NodeList, int N_Node, SUMA_VOLPAR *VolPar)
+SUMA_Boolean SUMA_vec_dicomm_to_3dfind (float *NodeList, 
+                                        int N_Node, SUMA_VOLPAR *VolPar)
 {
    static char FuncName[]={"SUMA_vec_dicomm_to_3dfind"};
 
    SUMA_ENTRY;
 
-   if (!NodeList || !VolPar) { SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
+   if (!NodeList || !VolPar) { 
+      SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
    
    
    if (!SUMA_vec_dicomm_to_3dmm(NodeList, N_Node, VolPar)) { SUMA_RETURN(NOPE); }
@@ -2246,13 +2259,15 @@ SUMA_Boolean SUMA_vec_dicomm_to_3dfind (float *NodeList, int N_Node, SUMA_VOLPAR
    SUMA_RETURN(YUP);
 }
 
-SUMA_Boolean SUMA_vec_3dfind_to_dicomm (float *NodeList, int N_Node, SUMA_VOLPAR *VolPar)
+SUMA_Boolean SUMA_vec_3dfind_to_dicomm (float *NodeList, int N_Node, 
+                                        SUMA_VOLPAR *VolPar)
 {
    static char FuncName[]={"SUMA_vec_3dfind_to_dicomm"};
 
    SUMA_ENTRY;
 
-   if (!NodeList || !VolPar) { SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
+   if (!NodeList || !VolPar) { 
+      SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
 
    if (!SUMA_vec_3dfind_to_3dmm(NodeList, N_Node, VolPar)) { SUMA_RETURN(NOPE); }
    if (!SUMA_vec_3dmm_to_dicomm(NodeList, N_Node, VolPar)) { SUMA_RETURN(NOPE); }
@@ -2260,63 +2275,64 @@ SUMA_Boolean SUMA_vec_3dfind_to_dicomm (float *NodeList, int N_Node, SUMA_VOLPAR
    SUMA_RETURN(YUP);
 }
 
-SUMA_Boolean SUMA_vec_3dmm_to_dicomm (float *NodeList, int N_Node, SUMA_VOLPAR *VolPar)
+SUMA_Boolean SUMA_vec_3dmm_to_dicomm (float *NodeList, int N_Node, 
+                                      SUMA_VOLPAR *VolPar)
 {
    static char FuncName[]={"SUMA_vec_3dmm_to_dicomm"};
    THD_fvec3 fv, iv;
-   int i, id;
-   SUMA_SurfaceObject SO;
+   int i, id, NodeDim=3;
    
    SUMA_ENTRY;
 
-   if (!NodeList || !VolPar) { SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
-   /* create dummy struct */
-   SO.NodeList = NodeList; SO.N_Node = N_Node; SO.VolPar = VolPar; SO.NodeDim = 3;
+   if (!NodeList || !VolPar) { 
+      SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
 
-   for (i=0; i < SO.N_Node; ++i) {
-      id = i * SO.NodeDim;
-      iv.xyz[0] = SO.NodeList[id] ;
-      iv.xyz[1] = SO.NodeList[id+1] ;
-      iv.xyz[2] = SO.NodeList[id+2] ;
+   for (i=0; i < N_Node; ++i) {
+      id = i * NodeDim;
+      iv.xyz[0] = NodeList[id] ;
+      iv.xyz[1] = NodeList[id+1] ;
+      iv.xyz[2] = NodeList[id+2] ;
 
       /* change mm to RAI coords */
-      fv = SUMA_THD_3dmm_to_dicomm( SO.VolPar->xxorient, SO.VolPar->yyorient, SO.VolPar->zzorient,  iv );
-      SO.NodeList[id] = fv.xyz[0];
-      SO.NodeList[id+1] = fv.xyz[1];
-      SO.NodeList[id+2] = fv.xyz[2];
+      fv = SUMA_THD_3dmm_to_dicomm( VolPar->xxorient, 
+                                    VolPar->yyorient, 
+                                    VolPar->zzorient,  iv );
+      NodeList[id] = fv.xyz[0];
+      NodeList[id+1] = fv.xyz[1];
+      NodeList[id+2] = fv.xyz[2];
    }
 
    SUMA_RETURN(YUP);
 }   
 
-SUMA_Boolean SUMA_vec_dicomm_to_3dmm (float *NodeList, int N_Node, SUMA_VOLPAR *VolPar)
+SUMA_Boolean SUMA_vec_dicomm_to_3dmm (float *NodeList, int N_Node, 
+                                      SUMA_VOLPAR *VolPar)
 {
    static char FuncName[]={"SUMA_vec_dicomm_to_3dmm"};
    THD_fvec3 fv, iv;
-   int i, id;
-   SUMA_SurfaceObject SO;
+   int i, id, NodeDim=3;
 
    SUMA_ENTRY;
 
-   if (!NodeList || !VolPar) { SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
-   /* create dummy struct */
-   SO.NodeList = NodeList; SO.N_Node = N_Node; SO.VolPar = VolPar; SO.NodeDim = 3;
+   if (!NodeList || !VolPar) { 
+      SUMA_SL_Err("Null NodeList || Null VolPar"); SUMA_RETURN(NOPE); }
+   
+   for (i=0; i < N_Node; ++i) {
+      id = i * NodeDim;
 
-   for (i=0; i < SO.N_Node; ++i) {
-      id = i * SO.NodeDim;
-
-      iv.xyz[0] = SO.NodeList[id] ;
-      iv.xyz[1] = SO.NodeList[id+1] ;
-      iv.xyz[2] = SO.NodeList[id+2] ;
+      iv.xyz[0] = NodeList[id] ;
+      iv.xyz[1] = NodeList[id+1] ;
+      iv.xyz[2] = NodeList[id+2] ;
 
       /* change mm to RAI coords */
-      fv = SUMA_THD_dicomm_to_3dmm( &SO, iv );
+      fv = SUMA_THD_dicomm_to_3dmm( VolPar->xxorient, 
+                                    VolPar->yyorient, VolPar->zzorient,iv );
       /* fprintf(SUMA_STDERR,"%s: In[%f %f %f] Out[%f %f %f]\n", 
                         FuncName, iv.xyz[0], iv.xyz[1], iv.xyz[2], 
                         fv.xyz[0], fv.xyz[1], fv.xyz[2]); */
-      SO.NodeList[id] = fv.xyz[0];
-      SO.NodeList[id+1] = fv.xyz[1];
-      SO.NodeList[id+2] = fv.xyz[2];
+      NodeList[id] = fv.xyz[0];
+      NodeList[id+1] = fv.xyz[1];
+      NodeList[id+2] = fv.xyz[2];
    }
 
    SUMA_RETURN(YUP);
