@@ -43,6 +43,7 @@ char * SUMA_New_Additions (int ver, SUMA_Boolean StampOnly)
 
    SS = SUMA_StringAppend (NULL, NULL);
    
+   #if 0 /* Stopped maintaining this list for a long time now. */
    if (ver == 0) { /* just the latest */
       s = SUMA_New_Additions_perver( verv[0], StampOnly);
       if (s) {
@@ -68,10 +69,10 @@ char * SUMA_New_Additions (int ver, SUMA_Boolean StampOnly)
          SS = SUMA_StringAppend (SS, s); SUMA_free(s); s = NULL;
       }
    }
-   
    /* add the CVS tag            ZSS: Looks like nobody likes tags. 
                                       Compile Date is enough*/
    SS = SUMA_StringAppend_va (SS, "\nCVS tag:\n   %s\n", SUMA_VERSION_LABEL);
+   #endif
    
    /* add the compile date */
    SS = SUMA_StringAppend_va (SS, "\nCompile Date:\n   %s\n",__DATE__);
@@ -442,6 +443,7 @@ void SUMA_Message_destroyed (void *p)
    
    SUMA_RETURNe;
 }
+
 char * SUMA_NIDO_Info(void)
 {
    static char FuncName[]={"SUMA_NIDO_Info"};
@@ -535,6 +537,16 @@ char * SUMA_NIDO_Info(void)
 "     You can do multi-line text.\n"
 "  coord attribute:\n"
 "     XYZ coordinates whose units are determined by nido_head's coord_type.\n" 
+"     See also p attribute\n"
+"  p attribute:\n"
+"     A convenience positioning attribute for placing text in fixed screen\n"
+"     coordinates. If present, it will override coord, h_align, and v_align\n"
+"     attributes. Its value is two to 3 characters long.\n"
+"     1st char: t for top, c for center or m for middle, b for bottom\n"
+"     2nd char: l for left, c for center or m for middle, r for right\n"
+"     3rd char: f for front, r for rear (optional)\n"
+"     h_align and v_align are set in a manner that makes sense for these \n"
+"     special position flags.\n"
 "  font attribute:\n"
 "     Sets the font for the text element. If not specified, font is set per \n"
 "     default_font.\n"
@@ -679,6 +691,53 @@ char * SUMA_NIDO_Info(void)
    SUMA_RETURN (s);
 
 }
+
+char * SUMA_OptList_string(HELP_OPT *hol)
+{
+   static char FuncName[]={"SUMA_OptList_string"};
+   char *s = NULL;
+   int i=0;
+   SUMA_STRING *SS = NULL;
+   
+   SUMA_ENTRY;
+   
+   SS = SUMA_StringAppend (NULL, NULL);
+   while (hol[i].name) {
+      SS = SUMA_StringAppend_va(SS,"   %s\n", hol[i].help);
+      if (hol[i].val) 
+         SS = SUMA_StringAppend_va(SS,"     default: %s\n", hol[i].val);
+      ++i;
+   }
+   
+   SUMA_SS2S(SS,s);
+   
+   SUMA_RETURN (s);
+}
+
+char *SUMA_OptList_get(HELP_OPT *hol, char *opname, char *what) 
+{
+   static char FuncName[]={"SUMA_OptList_default"};
+   int i = 0;
+   
+   SUMA_ENTRY;
+   
+   while (hol[i].name) {
+      if (!strcasecmp(hol[i].name, opname)) {
+         if (what[0] == 'd' || what[0] == 'D') {
+            SUMA_RETURN(hol[i].val);
+         } else if (what[0] == 'v' || what[0] == 'V') {
+            SUMA_RETURN(hol[i].val);
+         } else if (what[0] == 'h' || what[0] == 'H') {
+            SUMA_RETURN(hol[i].help);
+         } else {
+            SUMA_RETURN(NULL);
+         }
+      }
+      ++i;
+   }
+   SUMA_RETURN(NULL);  
+}
+
 char * SUMA_sources_Info(void)
 {
    static char FuncName[]={"SUMA_sources_Info"};
@@ -1036,9 +1095,10 @@ char * SUMA_help_Cmap_message_Info(SUMA_COLOR_MAP * ColMap)
    SUMA_ENTRY;
    
    SS = SUMA_StringAppend (NULL, NULL);
-
+   #if 0
    s = SUMA_New_Additions (0, 1);
    SS = SUMA_StringAppend (SS, s); SUMA_free(s); s = NULL;
+   #endif
    
    SS = SUMA_StringAppend (SS, 
       "\nColormap Keyboard Controls:\n"
@@ -1094,6 +1154,7 @@ static char PlotCommonHelp[]={
       "        A graphing window can be opened for each\n"
       "        dataset, and all graphs will update unless\n"
       "        '1 Only' is set in Surface Controller.\n"
+      "        For complex data its magnitude is plotted instead.\n"
       "        Use 'ctrl+h' in graph window for more help.\n" };
 
 char * SUMA_help_Plot_message_Info(void)
@@ -1144,9 +1205,10 @@ char * SUMA_help_message_Info(void)
    
    SS = SUMA_StringAppend (NULL, NULL);
 
+   #if 0 /* not maintained any more */
    s = SUMA_New_Additions (0, 1);
    SS = SUMA_StringAppend (SS, s); SUMA_free(s); s = NULL;
-   
+   #endif
    SS = SUMA_StringAppend (SS, 
       "\nKeyboard Controls\n"
       "   Note: On MACs, Alt is the Apple/Command key.\n"
@@ -1180,22 +1242,29 @@ char * SUMA_help_message_Info(void)
    SS = SUMA_StringAppend (SS, 
       "     Ctrl+d: draw ROI controller.\n\n");
    SS = SUMA_StringAppend (SS, 
-      "     D: Attch to the current dataset 'parent' a dot product\n"
-      "        transform. The 'child' (transformed) dataset\n"
-      "        is created by calculating the dot product between\n"
-      "        each node time series and the time series of the current\n"
-      "        node. Each time you 'shift+ctrl+click (drag too if you like)'\n"
-      "        on the surface, with the child dataset in view, the dot product\n"
-      "        is recalculated.\n"
-      "        To stop the interactive dot product computations,\n"
-      "        switch back to the parent dset and press 'D' again.\n"
-      "        If the parent dataset is properly detrended and each \n"
-      "        time series is normalized so that its stdev is 1.0\n"
-      "        then the dot product is the cross correlation coefficient.\n"
-      "        Detrending and normalization can be carried out with:\n"
-      "           3dDetrend -polort 4 -normalize \\\n"
-      "                     -prefix dt.TS.niml.dset \\\n"
-      "                     v2s.TS.niml.dset\n"
+   "     D: Attch to the current dataset 'parent' a dot product\n"
+   "        transform. The 'child' (transformed) dataset\n"
+   "        is created by calculating the dot product between\n"
+   "        each node time series and the time series of the current\n"
+   "        node. Each time you 'shift+ctrl+click (drag too if you like)'\n"
+   "        on the surface, with the child dataset in view, the dot product\n"
+   "        is recalculated. \n"
+   "        You can save the resultant datasets with 'ctrl+W' key (see below).\n"
+   "        Dset names are automatically formed.\n"
+   "        To stop the interactive dot product computations,\n"
+   "        switch back to the parent dset and press 'D' again.\n"
+   "        If the parent dataset is properly detrended and each \n"
+   "        time series is normalized so that its stdev is 1.0\n"
+   "        then the dot product is the cross correlation coefficient.\n"
+   "        Detrending and normalization can be carried out with:\n"
+   "           3dDetrend -polort 4 -normalize \\\n"
+   "                     -prefix dt.TS.niml.dset \\\n"
+   "                     v2s.TS.niml.dset\n"
+   "        You can get a good feel for what this 'D' does by running\n"
+   "               @Install_InstaCorr_Demo\n"
+   "        That script will download and setup demo data for resting-state\n"
+   "        correlations. In particular, script @RunSingleSurfInstaCorr of the\n"
+   "        demo illustrates the 'D' feature.\n"
       "\n\n");
    if (SUMAg_CF->Dev) SS = SUMA_StringAppend (SS, 
       "     d: Show all DO objects in DOv.\n\n");
@@ -1224,10 +1293,9 @@ char * SUMA_help_message_Info(void)
       "        in Focus. Does not update in other viewers\n"
       "        or in AFNI.\n");
    SS = SUMA_StringAppend (SS, 
-      "     j: Set the cross hair to a certain node on \n"
-      "        SO in Focus.\n"
-      "        Does update in other viewers\n"
-      "        if linked by index"
+      "     j: Set the cross hair to a certain node on SO in Focus.\n"
+      "        Append/prepend L or R to switch hemispheres.\n"
+      "        Does update in other viewers if linked by index\n"
       "        and AFNI if connected\n");
    SS = SUMA_StringAppend (SS, 
       "     Ctrl+j: Set the cross hair's XYZ location. \n"
@@ -1263,11 +1331,29 @@ char * SUMA_help_message_Info(void)
       "     m: momentum, toggle\n\n");
    if (SUMAg_CF->Dev) SS = SUMA_StringAppend (SS, 
       "     n: bring a node to direct view (does not work AT ALL)\n");
+   SS = SUMA_StringAppend (SS, 
+      "     Ctrl+o: Set new center of rotation.\n"
+      "            Enter nothing to go back to default.\n"
+      "\n");
    SS = SUMA_StringAppend (SS, 
       "     Ctrl+n: Open a new surface viewer window.\n\n");
    SS = SUMA_StringAppend (SS, 
-      "     p: Viewer rendering mode  \n"
-      "        (Fill, Line, Points, Hide), switch.\n\n");
+"     p: Viewer rendering mode  \n"
+"        (Fill, Line, Points, Hide), switch.\n\n"
+"     Ctrl+p: Cycle between restrictions of where DO node-based \n"
+"             objects are displayed. Available modes are:\n"
+"         All: No restrictions\n"
+"         n3Crosshair: Crosshair node + 3 neighboring layers\n"
+"         n2Crosshair: Crosshair node + 2 neighboring layers\n" 
+"         n1Crosshair: Crosshair node only\n"
+"         None: Show nothing.\n"
+"              See also -do_draw_mask option in DriveSuma\n"
+"        ** DO stands for displayable objects, see 'Ctrl+Alt+s'\n"
+"           below.\n"
+"        ** For the moment, 'Ctrl+p' only applies to segment \n"
+"        and sphere DOs  that are node based. \n"
+"        If you need it applied to other DOs, let me know.\n"
+      );
    SS = SUMA_StringAppend (SS, 
       "     P: Reset viewer and all surfaces to Fill  \n"
       "        rendering mode.\n\n");
@@ -1279,12 +1365,27 @@ char * SUMA_help_message_Info(void)
       "        window has no visible controls for saving\n"
       "        the image. Either take another picture, or\n"
       "        use 'Shift+right click' to get a menu.\n\n");
-   if (SUMAg_CF->Dev) SS = SUMA_StringAppend (SS, 
-      "     Alt+r: Set new center of rotation.\n"
-      "            Enter nothing to go back to default.\n"
-      "\n");
+   SS = SUMA_StringAppend_va (SS, 
+      "     Ctrl+r: Record current image directly to disk.\n"
+   "             Images are saved with a date stamp of the\n"
+   "             format PREFIX.X.yymmdd_hhmmss.MMM.jpg where:\n"
+   "          PREFIX controlled with SUMA_AutoRecordPrefix.\n"
+   "             See environment variable SUMA_AutoRecordPrefix for\n"
+   "             controlling prefix and output image type (suma -update_env).\n"
+   "          X  The character indicating which viewer is recording (you can\n"
+   "             record from multiple viewers at once.\n"
+   "          yy, mm, dd, hh, mm, ss for year, month, day, hours, minutes,\n"
+   "             and seconds, respectively. MMM is a millisecond marker to\n"
+   "             avoid overwriting files. Unlike the other recording mode \n"
+   "            (with the 'R' key), there is no rejection of identical images\n"
+   "\n"
+   "             This option is useful for saving a large number of images\n"
+   "             without running out of memory in the recorder GUI. \n"
+   "\n"
+   "             Your current PREFIX is: %s%s\n"
+   "\n", SUMAg_CF->autorecord->Path, SUMAg_CF->autorecord->FileName);
    SS = SUMA_StringAppend (SS, 
-      "     Ctrl+r: Increase the image oversampling factor.\n"
+      "     Alt+r: Increase the image oversampling factor.\n"
       "             By increasing this factor, you can create\n"
       "             images at a resolution higher than that \n"
       "             of the SUMA window. This is done by subdividing \n"
@@ -1301,10 +1402,13 @@ char * SUMA_help_message_Info(void)
       "             This limitation is due to the graphics card\n"
       "             on your system. SUMA will take care not to exceed\n"
       "             this limit.\n");
-   SS = SUMA_StringAppend (SS, 
-      "     R: Toggle continuous recording \n"
-      "        to an a la AFNI image viewer.\n"
-      "        Identical images are rejected.\n\n");
+   SS = SUMA_StringAppend(SS, 
+   "     Ctrl+R: Toggle continuous jpeg saving to disk.\n"
+   "             Naming of output images is automatic, same as in Ctrl+r.\n"
+   "             See help for Ctrl+r above for more info.\n"
+   "     R: Toggle continuous recording \n"
+   "        to an a la AFNI image viewer.\n"
+   "        Identical images are rejected.\n\n");
    SS = SUMA_StringAppend (SS, 
       "     s: NO LONGER IN USE. \n"
       "        View the surface's structure contents.\n"
@@ -1313,113 +1417,115 @@ char * SUMA_help_message_Info(void)
       "     Ctrl+s: Open controller for \n"
       "             surface in Focus.\n");
    SS = SUMA_StringAppend (SS, 
-      "     Ctrl+Alt+s: Input filename containing displayable objects.\n"
-      "                 Files are of 1D format with a necessary comment\n"
-      "                 at the top to indicate the type of objects in \n"
-      "                 the file.\n"
-      "                 Note 1: Repeatedly loading files with the same \n"
-      "                 name will replace currently loaded versions.\n"
-      "                 Note 2: Node-based (Types 3 and 4) objects\n"
-      "                 will follow a node when its coordinates change.\n"
-      "          Type 1:Segments between (x0,y0,z0) and (x1,y1,z1) \n"
-      "                 1st line must be '#segments' (without quotes),\n"
-      "                 or '#oriented_segments' (slower to render).\n"
-      "                 One can also use #node-based_segments or \n"
-      "                 #node-based_oriented_segments and use a node index\n"
-      "                 in place of (x,y,z) triplets.\n"
-      "                 Remainder of file is N rows, each defining a \n"
-      "                 segment (or a vector) between two points.\n"
-      "                 Column content depends on the number of columns\n"
-      "                 in the file:\n"
-      "              For node-based:\n"
-      "                 2  cols: n0 n1\n"
-      "                 3  cols: n0 n1 th\n"
-      "                          with th being line thickness\n"
-      "                 6  cols: n0 n1 c0 c1 c2 c3\n"
-      "                          with c0..3 being the RGBA values\n"
-      "                          between 0 and 1.0\n"
-      "                 7  cols: n0 n1 c0 c1 c2 c3 th\n"
-      "              For coordinate-based\n"
-      "                 6  cols: x0 y0 z0 x1 y1 z1\n"
-      "                 7  cols: x0 y0 z0 x1 y1 z1 th\n"
-      "                          with th being line thickness\n"
-      "                 10 cols: x0 y0 z0 x1 y1 z1 c0 c1 c2 c3\n"
-      "                          with c0..3 being the RGBA values\n"
-      "                          between 0 and 1.0\n"
-      "                 11 cols: x0 y0 z0 x1 y1 z1 c0 c1 c2 c3 th\n"
-      "          Type 2:Spheres centered at (ox, oy, oz) \n"
-      "                 1st line must be '#spheres' (without quotes).\n"
-      "                 Remainder of file is N rows, each defining a \n"
-      "                 sphere.\n"
-      "                 Column content depends on the number of columns\n"
-      "                 in the file:\n"
-      "                 3  cols: ox oy oz\n"
-      "                 4  cols: ox oy oz rd\n"
-      "                          with rd being the radius of the sphere\n"
-      "                 5  cols: ox oy oz rd st\n"
-      "                          with st being the style of the sphere's\n"
-      "                          rendering. Choose from:\n"
-      "                             0: points\n"
-      "                             1: Lines\n"
-      "                             2: Filled\n"
-      "                 7  cols: ox oy oz c0 c1 c2 c3 \n"
-      "                          with c0..3 being the RGBA values\n"
-      "                          between 0 and 1.0\n"
-      "                 8  cols: ox oy oz c0 c1 c2 c3 rd\n"
-      "                 9  cols: ox oy oz c0 c1 c2 c3 rd st\n"
-      "          Type 3:Vectors (vx, vy, vz) at surface nodes \n"
-      "                 1st line must be '#node-based_vectors' (without quotes)\n"
-      "                 or '#node-based_ball-vectors' (slower to render).\n"
-      "                 Remainder of file is N rows, each defining a \n"
-      "                 a vector at a particular node of the current surface.\n"
-      "                 Column content depends on the number of columns\n"
-      "                 in the file:\n"
-      "                 3  cols: vx, vy, vz \n"
-      "                          node index 'n' is implicit equal to row index.\n"
-      "                          Vector 'v' is from coordinates of node 'n' to \n"
-      "                          coordinates of node 'n' + 'v'\n"
-      "                 4  cols: n, vx, vy, vz \n"
-      "                          Here the node index 'n' is explicit. You can\n"
-      "                          have multiple vectors per node, one on \n"
-      "                          each row.\n"
-      "                 5  cols: n, vx, vy, vz, gn\n"
-      "                          with gn being a vector gain factor\n"
-      "                 8  cols: n, vx, vy, vz, c0 c1 c2 c3\n"
-      "                          with with c0..3 being the RGBA values\n"
-      "                          between 0 and 1.0\n"
-      "                 9  cols: n, vx, vy, vz, c0 c1 c2 c3 gn\n"   
-      "          Type 4:Spheres centered at nodes n of the current surface\n"
-      "                 1st line must be '#node-based_spheres' (without quotes).\n"
-      "                 Remainder of file is N rows, each defining a \n"
-      "                 sphere.\n"
-      "                 Column content depends on the number of columns\n"
-      "                 in the file, see Type 2 for more details:\n"
-      "                 1  cols: n\n"
-      "                 2  cols: n rd\n"
-      "                 3  cols: n rd st\n"
-      "                 5  cols: n c0 c1 c2 c3 \n"
-      "                 6  cols: n c0 c1 c2 c3 rd\n"
-      "                 7  cols: n c0 c1 c2 c3 rd st\n"
-      "          Type 5:Planes defined with: ax + by + cz + d = 0.\n"
-      "                 1st line must be '#planes' (without quotes).\n"
-      "                 Remainder of file is N rows, each defining a \n"
-      "                 plane.\n"
-      "                 Column content depends on the number of columns\n"
-      "                 in the file:\n"
-      "                 7  cols: a b c d cx cy cz\n"
-      "                          with the plane's equation being:\n"
-      "                          ax + by + cz + d = 0\n"
-      "                          cx,cy,cz is the center of the plane's\n"
-      "                          representation. \n"
-      "                          Yes, d is not of much use here.\n"
-      "                 There are no node-based planes at the moment.\n"
-      "                 They are a little inefficient to reproduce with\n"
-      "                 each redraw. Complain if you need them.\n"
-      "         Type 6: Another class of displayble objects is described in\n"
-      "                 the output of suma -help_nido and the demonstration\n"
-      "                  script\n"
-      "                 @DO.examples. This new class allows for displaying \n"
-      "                 text and figures in both screen and world space.\n"
+"     Ctrl+Alt+s: Input filename containing displayable objects.\n"
+"                 Files are of 1D format with a necessary comment\n"
+"                 at the top to indicate the type of objects in \n"
+"                 the file.\n"
+"                 Note 1: Repeatedly loading files with the same \n"
+"                 name will replace currently loaded versions.\n"
+"                 Note 2: Node-based (Types 3 and 4) objects\n"
+"                 will follow a node when its coordinates change.\n"
+"                 Note 3: See also 'ctrl+p' for restricting which \n"
+"                 node-based objects get displayed.\n"
+"          Type 1:Segments between (x0,y0,z0) and (x1,y1,z1) \n"
+"                 1st line must be '#segments' (without quotes),\n"
+"                 or '#oriented_segments' (slower to render).\n"
+"                 One can also use #node-based_segments or \n"
+"                 #node-based_oriented_segments and use a node index\n"
+"                 in place of (x,y,z) triplets.\n"
+"                 Remainder of file is N rows, each defining a \n"
+"                 segment (or a vector) between two points.\n"
+"                 Column content depends on the number of columns\n"
+"                 in the file:\n"
+"              For node-based:\n"
+"                 2  cols: n0 n1\n"
+"                 3  cols: n0 n1 th\n"
+"                          with th being line thickness\n"
+"                 6  cols: n0 n1 c0 c1 c2 c3\n"
+"                          with c0..3 being the RGBA values\n"
+"                          between 0 and 1.0\n"
+"                 7  cols: n0 n1 c0 c1 c2 c3 th\n"
+"              For coordinate-based\n"
+"                 6  cols: x0 y0 z0 x1 y1 z1\n"
+"                 7  cols: x0 y0 z0 x1 y1 z1 th\n"
+"                          with th being line thickness\n"
+"                 10 cols: x0 y0 z0 x1 y1 z1 c0 c1 c2 c3\n"
+"                          with c0..3 being the RGBA values\n"
+"                          between 0 and 1.0\n"
+"                 11 cols: x0 y0 z0 x1 y1 z1 c0 c1 c2 c3 th\n"
+"          Type 2:Spheres centered at (ox, oy, oz) \n"
+"                 1st line must be '#spheres' (without quotes).\n"
+"                 Remainder of file is N rows, each defining a \n"
+"                 sphere.\n"
+"                 Column content depends on the number of columns\n"
+"                 in the file:\n"
+"                 3  cols: ox oy oz\n"
+"                 4  cols: ox oy oz rd\n"
+"                          with rd being the radius of the sphere\n"
+"                 5  cols: ox oy oz rd st\n"
+"                          with st being the style of the sphere's\n"
+"                          rendering. Choose from:\n"
+"                             0: points\n"
+"                             1: Lines\n"
+"                             2: Filled\n"
+"                 7  cols: ox oy oz c0 c1 c2 c3 \n"
+"                          with c0..3 being the RGBA values\n"
+"                          between 0 and 1.0\n"
+"                 8  cols: ox oy oz c0 c1 c2 c3 rd\n"
+"                 9  cols: ox oy oz c0 c1 c2 c3 rd st\n"
+"          Type 3:Vectors (vx, vy, vz) at surface nodes \n"
+"                 1st line must be '#node-based_vectors' (without quotes)\n"
+"                 or '#node-based_ball-vectors' (slower to render).\n"
+"                 Remainder of file is N rows, each defining a \n"
+"                 a vector at a particular node of the current surface.\n"
+"                 Column content depends on the number of columns\n"
+"                 in the file:\n"
+"                 3  cols: vx, vy, vz \n"
+"                          node index 'n' is implicit equal to row index.\n"
+"                          Vector 'v' is from coordinates of node 'n' to \n"
+"                          coordinates of node 'n' + 'v'\n"
+"                 4  cols: n, vx, vy, vz \n"
+"                          Here the node index 'n' is explicit. You can\n"
+"                          have multiple vectors per node, one on \n"
+"                          each row.\n"
+"                 5  cols: n, vx, vy, vz, gn\n"
+"                          with gn being a vector gain factor\n"
+"                 8  cols: n, vx, vy, vz, c0 c1 c2 c3\n"
+"                          with with c0..3 being the RGBA values\n"
+"                          between 0 and 1.0\n"
+"                 9  cols: n, vx, vy, vz, c0 c1 c2 c3 gn\n"   
+"          Type 4:Spheres centered at nodes n of the current surface\n"
+"                 1st line must be '#node-based_spheres' (without quotes).\n"
+"                 Remainder of file is N rows, each defining a \n"
+"                 sphere.\n"
+"                 Column content depends on the number of columns\n"
+"                 in the file, see Type 2 for more details:\n"
+"                 1  cols: n\n"
+"                 2  cols: n rd\n"
+"                 3  cols: n rd st\n"
+"                 5  cols: n c0 c1 c2 c3 \n"
+"                 6  cols: n c0 c1 c2 c3 rd\n"
+"                 7  cols: n c0 c1 c2 c3 rd st\n"
+"          Type 5:Planes defined with: ax + by + cz + d = 0.\n"
+"                 1st line must be '#planes' (without quotes).\n"
+"                 Remainder of file is N rows, each defining a \n"
+"                 plane.\n"
+"                 Column content depends on the number of columns\n"
+"                 in the file:\n"
+"                 7  cols: a b c d cx cy cz\n"
+"                          with the plane's equation being:\n"
+"                          ax + by + cz + d = 0\n"
+"                          cx,cy,cz is the center of the plane's\n"
+"                          representation. \n"
+"                          Yes, d is not of much use here.\n"
+"                 There are no node-based planes at the moment.\n"
+"                 They are a little inefficient to reproduce with\n"
+"                 each redraw. Complain if you need them.\n"
+"         Type 6: Another class of displayble objects is described in\n"
+"                 the output of suma -help_nido and the demonstration\n"
+"                  script\n"
+"                 @DO.examples. This new class allows for displaying \n"
+"                 text and figures in both screen and world space.\n"
       );
    SS = SUMA_StringAppend (SS, 
       "     Alt+s: Switch mouse buttons 1 and 3.\n\n");
@@ -1440,10 +1546,15 @@ char * SUMA_help_message_Info(void)
       "        Use: View->Viewer Controller->More.\n"
       "\n");
    SS = SUMA_StringAppend (SS, 
-      "     w: This option has been dropped.\n"
-      "        Instead, use 'r' or 'R' recording options\n"
-      "        or use a screen grab instead. \n"
-      "        (like xv on unix systems, and grab on Macs.)\n");
+      "     w: Whereami window of little use at the moment.\n"
+      "\n"        );
+   SS = SUMA_StringAppend (SS, 
+      "     ctrl+W: Write items stowed in SUMA's save list.\n"
+      "             This is used to write temporary dsets that\n"
+      "             are created on the fly in SUMA. Such sets include\n"
+      "             those created via the 'D' option above,\n"
+      "             or the results sent by 3dGroupInCorr\n"
+      "\n");
    SS = SUMA_StringAppend (SS, 
       "     W: Write ascii files containing the NodeList,\n"
       "        the FaceSetList and the nodecolors of the \n"
@@ -1528,6 +1639,9 @@ char * SUMA_help_message_Info(void)
    SS = SUMA_StringAppend (SS, 
       "     F8: Viewing mode (Perspective or Orthographic Projection), toggle.\n"
       );
+   SS = SUMA_StringAppend (SS, 
+      "     F9: Labels at cross hair, toggle.\n"
+      );
    SS = SUMA_StringAppend (SS, 
       "     F12: Time 20 scene renderings.\n\n");
    SS = SUMA_StringAppend (SS, 
@@ -1536,7 +1650,7 @@ char * SUMA_help_message_Info(void)
    SS = SUMA_StringAppend (SS, 
       "     ESCAPE: close the surface viewer window.\n");
    SS = SUMA_StringAppend (SS, 
-      "     Shft+ESCAPE: close all surface viewer windows.\n\n");
+      "     Shift+ESCAPE: close all surface viewer windows.\n\n");
    SS = SUMA_StringAppend (SS, 
       "     Mouse Controls:\n");
    SS = SUMA_StringAppend (SS, 
@@ -1560,22 +1674,29 @@ char * SUMA_help_message_Info(void)
    SS = SUMA_StringAppend (SS, 
       "     Button 2-Motion: translation\n"); 
    SS = SUMA_StringAppend (SS, 
-      "     Button 1+2-Motion OR \n"
-      "      Shift+Button2-Motion: \n"
+      "     Button 1+2-Motion    OR \n"
+      "     Shift+Button2-Motion: \n"
       "          Zoom in/out\n");
    SS = SUMA_StringAppend (SS, 
       "     Button 3-Press: Node picking \n"
       "                     Initiates a path to new node in DrawROI mode.\n"
-      "                     No calls in Dot xform mode, or GroupInCorr\n" );
-   SS = SUMA_StringAppend (SS, 
-      "     shft+ctrl+Button 3-Press: Pick and initiate call in Dot xform\n"
-      "                               mode, or to GroupInCorr\n"
-      );
+      "                     No calls in Dot xform mode, or GroupInCorr\n" 
+      "     Shift+Button 3-Press: Same as without shift, except does not draw\n"
+      "                           in DrawROI mode.\n"
+      "     Ctrl+Button 3-Press: Yoke intensity selection to index of \n"
+      "                          selected node*K. This is only possible if\n"
+      "                          the currently visualized dataset has K times \n"
+      "                          many sub-bricks as the surface has nodes. \n"
+      "                          K is an integer.\n"
+      "     Shift+Ctrl+Button 3-Press: Pick and initiate call in Dot xform\n"
+      "                               mode, or to GroupInCorr\n");
    SS = SUMA_StringAppend (SS, 
       "     Button 3-Motion: continuous picking\n"
       "                      No calls for dot product (InstaCorr)\n"
       "                           or GroupInCorr, while dragging.\n" 
-      "     shft+ctrl+Button 3-Motion: Continuous picking and calls \n"
+      "     Ctrl+Button 3-Motion: continous yoking of intensity selection to\n"
+      "                           selected node*K.\n" 
+      "     Shift+Ctrl+Button 3-Motion: Continuous picking and calls \n"
       "                                for dot product (InstaCorr)\n"
       "                               or GroupInCorr, while dragging.\n"
       ); 
@@ -1621,13 +1742,13 @@ char * SUMA_help_message_Info(void)
       "\n");
    SS = SUMA_StringAppend_va( SS,
                               "SUMA's list of environment variables:\n");
-   s = SUMA_env_list_help();
+   s = SUMA_env_list_help(0);
    SS = SUMA_StringAppend( SS, s); SUMA_free(s); s = NULL;
    SS = SUMA_StringAppend( SS, "\n");
    
    SS = SUMA_StringAppend (SS, 
       "    More help at \n"
-      "    http://afni.nimh.nih.gov/pub/dist/doc/SUMA/SUMA_doc.htm\n");
+      "    http://afni.nimh.nih.gov/pub/dist/edu/latest/suma/suma.pdf\n");
    SS = SUMA_StringAppend (SS, 
       "\n");
    
@@ -1666,6 +1787,10 @@ char * SUMA_help_xform_dot_message_Info(void)
 "by pressing 'D' on a 'parent' time series dataset. \n"
 "The result of the transform is one (or more) 'child' dataset that\n"
 "will initiate a new transform call with every new node selected.\n"
+"\n"
+"You can save the resultant datasets by pressing 'ctrl+W' in the SUMA\n"
+"window. The datasets are automatically named to reflect the seed location\n"
+"and the dataset whose voxels time series are dotted by the seed's.\n"
 "\n"  
    ); 
    
@@ -1822,9 +1947,11 @@ void SUMA_Version (FILE *Out)
    static char FuncName[]={"SUMA_Version"};
    char *s = NULL;
    
+
    if (Out == NULL) {
 		Out = stdout;
 	}
+   
    s = SUMA_New_Additions (0, 0);
 	if (s) {
       fprintf (Out, "\n   %s\n", s);
@@ -1964,7 +2091,8 @@ char * SUMA_Help_AllSurfCont ()
          "++ Switch Dset:\n%s\n"
          "\n", 
          SUMA_SurfContHelp_DsetLblTblr0, SUMA_SurfContHelp_DsetLblTblr1, 
-         SUMA_SurfContHelp_DsetOrd, SUMA_SurfContHelp_DsetOpa, SUMA_SurfContHelp_DsetDim,
+         SUMA_SurfContHelp_DsetOrd, SUMA_SurfContHelp_DsetOpa, 
+         SUMA_SurfContHelp_DsetDim,
          SUMA_SurfContHelp_DsetView, SUMA_SurfContHelp_DsetSwitch);
    
    SS = SUMA_StringAppend_va(SS,       
@@ -1980,16 +2108,21 @@ char * SUMA_Help_AllSurfCont ()
          "\n"
          "++ Mapping Data: \n"
          "\n"
+         "+++ I,T Link:\n%s\n"
          "+++ I\n%s\n"
          "++++ v:\n%s\n"
          "+++ T\n%s\n"
          "++++ v\n%s\n"
          "+++ B\n%s\n"
          "++++ v\n%s\n"
+         "\n"
+         "     Note: %s\n"
          "\n", 
+         SUMA_SurfContHelp_Link, 
          SUMA_SurfContHelp_SelInt, SUMA_SurfContHelp_SelIntTgl,
          SUMA_SurfContHelp_SelThr, SUMA_SurfContHelp_SelThrTgl, 
-         SUMA_SurfContHelp_SelBrt, SUMA_SurfContHelp_SelBrtTgl );
+         SUMA_SurfContHelp_SelBrt, SUMA_SurfContHelp_SelBrtTgl,
+         SUMA_SurfContHelp_ArrowFieldMenu );
          
    SS = SUMA_StringAppend_va(SS, 
          "++ Mapping Parameters Table:\n%s\n"
