@@ -23,11 +23,13 @@ void SUMA_VolSurf_help (FILE *Out);
 void SUMA_Version (FILE *Out);
 char * SUMA_New_Additions_perver (int ver, SUMA_Boolean StampOnly);
 char * SUMA_New_Additions (int ver, SUMA_Boolean StampOnly);
-char * SUMA_env_list_help(void);
+char * SUMA_env_list_help(int DEFAULT_values);
 char *SUMA_All_Programs(void );
 void SUMA_SetcSV (Widget w, XtPointer clientData, XEvent * event, Boolean * cont);
 void SUMA_unSetcSV (Widget w, XtPointer clientData, XEvent * event, Boolean * cont);
 char *SUMA_sources_Info(void );
+char * SUMA_OptList_string(HELP_OPT *hol);
+char *SUMA_OptList_get(HELP_OPT *hol, char *opname, char *what);
 char *SUMA_NIDO_Info(void );
 char * SUMA_Help_AllSurfCont (void);
 
