@@ -70,7 +70,7 @@ typedef struct {
       int disable_done ;      /* 21 Aug 2008 [rickr] */
 
       int yes_niml ;          /* 28 Feb 2002 */
-               
+
                /* port_niml no longer in use. ZSS June 2011 */
                /* int port_niml ;         10 Dec 2002 */
 
@@ -238,7 +238,7 @@ typedef struct {                             /* 29 Mar 1999 */
 
 typedef struct {
       int   i1 , j2 , k3 ;  /* integer coordinates of current point */
-      float xi , yj , zk ;  /* float (mm) coordinates (take priority) */
+      float xi , yj , zk ;  /* float (DICOM mm) coordinates (take priority) */
 
       int   i1_old , j2_old , k3_old ;  /* for jumpback */
 
@@ -305,7 +305,10 @@ typedef struct {
 
       int   i1_icor , j2_icor , k3_icor;  /* for InstaCorr -- 08 May 2009 */
       float xi_icor , yj_icor , zk_icor ; /* DICOM coords -- 17 Mar 2010 */
-
+      
+      float *th_sort;  /* sorted values of overlay threshold */
+      int N_th_sort; /* number of values stored in th_sort */
+      char  th_sortid[256]; /* indentifier of provenance of th_sort */
 } AFNI_view_info ;
 
 #define AXIAL    1       /* 20 Feb 2003: view_setter codes */
@@ -359,16 +362,23 @@ typedef struct {
 
 #define AFNI_XHAIRS_LASTOPTION 8
 
+#define AFNI_HAVE_WAMI_WIDGET(im3d) \
+   ( ( im3d && im3d->vwid && im3d->vwid->imag && \
+       (im3d->vwid->imag->pop_whereami_twin ||  \
+        im3d->vwid->imag->pop_whereami_htmlwin) ) ? 1 : 0 )
+
 typedef struct {
       Widget frame , rowcol ;
       Widget topper , popmenu , pop_bkgd_lab ,
              pop_jumpback_pb , pop_imageonly_pb , pop_jumpto_pb , pop_talto_pb ;
       Widget pop_jumpto_ijk_pb ;
+      Widget pop_jumpto_clus_pb ;                                   /* 19 Oct 2012*/
 
       Widget crosshair_frame , crosshair_rowcol , crosshair_label ;
 
       Widget crosshair_menu, crosshair_dicom_pb, crosshair_spm_pb ; /* 12 Mar 2004 */
-      Widget crosshair_ijk_pb ;   /* 04 Oct 2010 */
+      Widget crosshair_ijk_pb ;                                     /* 04 Oct 2010 */
+      Widget crosshair_jtxyz_pb , crosshair_jtijk_pb ;              /* 01 Aug 2011 */
 
       Widget        xhair_rowcol ;
       MCW_arrowval *crosshair_av ;
@@ -391,7 +401,8 @@ typedef struct {
 
       Widget pop_whereami_pb , pop_ttren_pb ;
       MCW_textwin *pop_whereami_twin ;
-
+      MCW_htmlwin *pop_whereami_htmlwin ;
+      
       Widget pop_sumato_pb ;
       Widget pop_mnito_pb ;  /* 01 May 2002 */
 
@@ -424,6 +435,8 @@ typedef struct {
   Widget top_menu , histrange_pb ;
   MCW_bbox *histsqrt_bbox ;
 
+  MCW_bbox *usemask_bbox ;  /* zero-th row of controls [01 Aug 2011] */
+
   MCW_arrowval *cmode_av ;  /* first row of controls */
   Widget clust3d_pb, savetable_pb, index_lab, prefix_tf, done_pb ;
   Widget savemask_pb ;      /* 01 May 2008 */
@@ -434,6 +447,7 @@ typedef struct {
 
   Widget dset_lab ;         /* label after second row */
 
+  Widget clusters_lab ;     /* label at top of clusters table */
   int nrow, nall, is_open ;
   Widget *clu_rc ;          /* rows of widgets */
   Widget *clu_lab ;
@@ -554,15 +568,24 @@ typedef struct {
 
 extern void AFNI_sesslab_EV( Widget, XtPointer, XEvent *, Boolean * ) ; /* 30 Apr 2010 */
 
+extern void flush_vinfo_sort(AFNI_view_info *vinfo, char *sel);/*ZSS: 04/26/12*/
+extern void flush_3Dview_sort(struct Three_D_View *im3d, char *sel);
+extern float *get_3Dview_sort(struct Three_D_View *im3d, char *sel);
+extern float  get_3Dview_func_thresh(struct Three_D_View *im3d, 
+                                     int apply_power);
+extern float AFNI_thresh_from_percentile(struct Three_D_View *im3d, float perc);
+
 #define OPEN_PANEL(iq,panel)                                            \
    {  XtManageChild( (iq)->vwid->  panel  ->frame ) ;                    \
       if( ! (iq)->vwid->view->  panel ## _pb_inverted ){                  \
          MCW_invert_widget( (iq)->vwid->view->define_ ## panel ## _pb ) ;  \
-         (iq)->vwid->view->  panel ## _pb_inverted = True ; }             \
-      XMapRaised( XtDisplay( (iq)->vwid->  panel  ->frame ) ,            \
+         (iq)->vwid->view->  panel ## _pb_inverted = True ;                \
+         (iq)->vwid->func->inten_pbar->update_me = 2 ;                     \
+         update_MCW_pbar((iq)->vwid->func->inten_pbar) ; }                 \
+      XMapRaised( XtDisplay( (iq)->vwid->  panel  ->frame ) ,              \
                    XtWindow( (iq)->vwid->  panel  ->frame )  ) ; }
 
-#define CLOSE_PANEL(iq,panel)                                            \
+#define CLOSE_PANEL(iq,panel)                                           \
    {  XtUnmanageChild( (iq)->vwid->  panel  ->frame ) ;                  \
       if( (iq)->vwid->view->  panel ## _pb_inverted ){                    \
          MCW_invert_widget( (iq)->vwid->view->define_ ## panel ## _pb ) ;  \
@@ -663,6 +686,8 @@ typedef struct {
       generic_func *pbar_transform2D_func ;
       int           pbar_transform2D_index ;
 
+      Widget pbar_flip_pb ;                     /* 08 Feb 2012 */
+
       Widget options_rowcol , options_top_rowcol , options_label ;
       MCW_arrowval *options_vedit_av ;
       Widget ulaclu_rowcol , vedit_frame ;
@@ -680,6 +705,7 @@ typedef struct {
 
       Widget range_frame , range_rowcol , range_label ;
       MCW_bbox     *range_bbox ;
+      MCW_bbox     *perc_bbox ;
       MCW_arrowval *range_av ;
 
 #ifdef USE_FUNC_FIM
@@ -690,6 +716,7 @@ typedef struct {
       Widget bkgd_lab ;
 
       MCW_arrowval *range_rotate_av ;  /* 30 Mar 2001 */
+      Widget pbar_flip_pb2 ;           /* 08 Feb 2012 */
 
       MCW_bbox *see_ttatlas_bbox ;     /* 25 Jul 2001 */
 
@@ -716,6 +743,8 @@ extern void AFNI_func_fdr_CB    (Widget,XtPointer,XtPointer) ;    /* 29 Jan 2008
 #define RANGE_AUTOBUT 0
 #define RANGE_AUTOVAL (1 << RANGE_AUTOBUT)
 
+#define PERC_AUTOBUT 0  /* ZSS: April 27 2012 */
+
 /** On Motif 2.0 on Linux, resized pbar pieces causes the
     threshold scale to behave bizarrely.  This macro is a fixup **/
 
@@ -788,6 +817,7 @@ typedef struct {
       Widget         misc_1dchain_pb ;  /* 07 Aug 2001 */
 #endif
 
+      MCW_bbox     * thr_lock_bbox;     /* 16 Nov 2011 */
       MCW_bbox     * ijk_lock_bbox ;    /* 11 Sep 2000 */
 
       Widget         misc_savelayout_pb ; /* 23 Sep 2000 */
@@ -881,6 +911,7 @@ typedef struct {
 
       Widget picture ;
       int    picture_index ;
+      Widget tips_pb ;                    /* 27 Jun 2011 */
 
       Widget file_dialog , file_sbox ;
       XtCallbackProc file_cb ;
@@ -900,6 +931,17 @@ typedef struct {
       int butx , buty ;        /* 17 May 2005 */
 } AFNI_widget_set ;
 
+#define TIPS_PLUS_SHIFT   2
+#define TIPS_MINUS_SHIFT -60
+#define TIPS_TOTAL_SHIFT -90
+
+#define SHIFT_TIPS(iq,xx)                                                 \
+ do{ if( (iq)->vwid->tips_pb != NULL ){                                   \
+       XtVaSetValues( (iq)->vwid->tips_pb, XmNleftOffset, (xx), NULL ) ;  \
+       XMapRaised( XtDisplay( (iq)->vwid->tips_pb) ,                      \
+                    XtWindow( (iq)->vwid->tips_pb)  ) ;                   \
+     } } while(0)
+
 /** picture controls **/
 
 #define PICTURE_SET(im,px)                                            \
@@ -1058,6 +1100,7 @@ typedef struct Three_D_View {
       VEDIT_settings vedset ;                   /* 05 Sep 2006 */
       char *vedlabel ;                          /* 27 Mar 2007 */
       int   vedskip ;
+      int   vednomask ;                         /* 01 Aug 2011 */
 
       ICOR_setup   *iset ;                       /* 05 May 2009 */
       ICALC_setup  *icalc_setup ;                /* 18 Sep 2009 */
@@ -1069,6 +1112,7 @@ typedef struct Three_D_View {
       float cont_range_fval;
       int cont_pbar_index, int_pbar_index;
       int first_integral;
+      int cont_perc_thr;       /* ZSS percentile thresholding. April 26 2012 */
 } Three_D_View ;
 
 /*! Force re-volume-editing when this viewer is redisplayed */
@@ -1092,6 +1136,7 @@ typedef struct Three_D_View {
      (iq)->vwid->func->clu_tabNN1 = NULL ;                                 \
       (iq)->vwid->func->clu_tabNN2 = NULL ;                                \
        (iq)->vwid->func->clu_tabNN3 = NULL ;                               \
+     (iq)->vednomask = 0 ;                                                 \
      if( (iq)->vedset.code ) redis++ ;                                     \
      (iq)->vedset.flags = (iq)->vedset.code = 0; AFNI_set_thr_pval((iq));  \
      if( (iq)->vinfo->func_visible && redis ) AFNI_redisplay_func((iq)) ;  \
@@ -1405,7 +1450,8 @@ typedef struct {
    MCW_function_list registered_fim ;            /* 30 Jan 2000 */
 
    int ijk_lock ;                                /* 11 Sep 2000 */
-
+   int thr_lock ;
+   
    THD_session *session ;                        /* 20 Dec 2001 */
 
    MCW_function_list registered_slice_proj ;     /* 31 Jan 2002 */
@@ -1423,12 +1469,10 @@ typedef struct {
    AFNI_library_type GLOBAL_library ;
    int GLOBAL_num_dsets = 0 ;
    char *GLOBAL_motd = NULL ;                     /* 29 Nov 2005 */
-   char *GLOBAL_browser = NULL ;                  /* 30 Dec 2005 */
 #else
    extern AFNI_library_type GLOBAL_library ;
    extern int GLOBAL_num_dsets ;
    extern char *GLOBAL_motd ;
-   extern char *GLOBAL_browser ;
 #endif
 
 extern void AFNI_display_motd( Widget w ) ;       /* 29 Nov 2005 */
@@ -1484,6 +1528,7 @@ extern void AFNI_quit_CB           ( Widget wcall , XtPointer cd , XtPointer cbs
 extern void AFNI_quit_timeout_CB   ( XtPointer , XtIntervalId * ) ;
 extern void AFNI_startup_timeout_CB( XtPointer , XtIntervalId * ) ;
 extern void AFNI_vcheck_flasher    ( Three_D_View * ) ;
+extern void AFNI_tips_CB           ( Widget , XtPointer , XtPointer ) ;
 
 extern void AFNI_startup_layout_CB  ( XtPointer, XtIntervalId * ) ;    /* 23 Sep 2000 */
 extern void AFNI_save_layout_CB     ( Widget, XtPointer, XtPointer ) ;
@@ -1517,6 +1562,9 @@ extern void AFNI_thrdrag_lock_carryout( Three_D_View * ) ;
 extern void AFNI_range_lock_carryout( Three_D_View * ) ;  /* 23 Feb 2004 */
 
 extern void AFNI_ijk_lock_change_CB( Widget , XtPointer , XtPointer ) ;
+extern int AFNI_thresh_lock_env_val( void );
+extern void AFNI_func_thrlock_change_CB( Widget , XtPointer , XtPointer );
+extern void AFNI_set_all_thrlock_bboxes(Three_D_View *im3d, int bval) ;
 
 extern XtPointer AFNI_brick_to_mri( int n , int type , FD_brick * br );
 
@@ -1546,8 +1594,11 @@ extern void AFNI_viewbut_EV( Widget, XtPointer, XEvent *, Boolean * ) ;
 extern void AFNI_cluster_EV( Widget, XtPointer, XEvent *, Boolean * ) ;
 extern void AFNI_clus_update_widgets( Three_D_View *im3d ) ;
 extern void AFNI_clus_popdown( Three_D_View *im3d ) ;
+extern int AFNI_clus_find_xyz( Three_D_View *im3d , float x,float y,float z ) ;
+extern void AFNI_clus_action_CB( Widget w , XtPointer cd , XtPointer cbs ) ;
 
 extern void AFNI_update_dataset_viewing( THD_3dim_dataset * ); /* 21 Jul 2009 */
+extern void AFNI_alter_wami_text(Three_D_View *im3d, char *utlab); 
 
 #define AFNI_SEE_FUNC_ON(iq) ( MCW_set_bbox( (iq)->vwid->view->see_func_bbox, 1 ), \
                                AFNI_see_func_CB( NULL , (XtPointer)(iq) , NULL )  )
@@ -1585,6 +1636,7 @@ extern void AFNI_see_func_CB          ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_marks_edits_CB       ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_marks_transform_CB   ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_imag_pop_CB          ( Widget , XtPointer , XtPointer ) ;
+extern void AFNI_crosshair_pop_CB     ( Widget , XtPointer , XtPointer ) ; /* 01 Aug 2011 */
 extern void AFNI_define_CB            ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_underlay_CB          ( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_choose_dataset_CB    ( Widget , XtPointer , XtPointer ) ;
@@ -1601,11 +1653,13 @@ extern void AFNI_do_many_writes      ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_finalize_dataset_CB ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_jumpto_CB           ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern int  AFNI_jumpto_dicom        ( Three_D_View * , float, float, float  ) ;
+extern int  AFNI_creepto_dicom       ( Three_D_View * , float, float, float  ) ;
 extern int  AFNI_jumpto_ijk          ( Three_D_View * , int, int, int  ) ;
 extern void AFNI_jumpto_ijk_CB       ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_sumato_CB           ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_mnito_CB            ( Widget , XtPointer , MCW_choose_cbs * ) ;
-extern void AFNI_check_obliquity     ( Widget , THD_3dim_dataset * ) ;
+extern void AFNI_check_obliquity     ( Widget , THD_3dim_dataset * , 
+                                                THD_3dim_dataset * ) ;
 
 extern void AFNI_crosshair_pop_CB    ( Widget , XtPointer , XtPointer ) ; /* 12 Mar 2004 */
 extern void AFNI_crosshair_EV        ( Widget , XtPointer , XEvent * , Boolean * ) ;
@@ -1721,6 +1775,7 @@ extern XmString AFNI_range_label( Three_D_View * ) ;
 extern XmString AFNI_autorange_label( Three_D_View * ) ;
 
 extern void AFNI_range_bbox_CB( Widget , XtPointer , XtPointer ) ;
+extern void AFNI_perc_bbox_CB( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_range_av_CB  ( MCW_arrowval * , XtPointer ) ;
 extern void AFNI_inten_bbox_CB( Widget , XtPointer , XtPointer ) ;
 extern void AFNI_wrap_bbox_CB( Widget , XtPointer , XtPointer ) ;
@@ -1828,7 +1883,7 @@ extern MRI_IMAGE * AFNI_func_overlay( int , FD_brick * ) ;
 
 extern MRI_IMAGE * AFNI_newfunc_overlay( MRI_IMAGE *, float,float ,  /* 30 Jan 2003 */
                                          MRI_IMAGE *,
-                                         float,float, rgbyte * ) ;
+                                         float,float, rgbyte *, int ) ;
 
 extern void AFNI_syntax(void) ;
 
@@ -1955,6 +2010,8 @@ extern void AFNI_process_timeindex  ( Three_D_View * ) ; /* 29 Jan 2003 */
 extern void AFNI_do_bkgd_lab( Three_D_View * ) ;         /* 08 Mar 2002 */
 
 extern MRI_IMAGE * AFNI_ttatlas_overlay(Three_D_View *, int,int,int,int, MRI_IMAGE *) ;
+extern void reset_atlas_ovdset(void);
+extern THD_3dim_dataset *current_atlas_ovdset(void);
 
 extern void AFNI_3d_linefill( int  ,int * ,int * ,int * ,
                               int *,int **,int **,int ** ) ;
@@ -1981,7 +2038,7 @@ extern void TTRR_popup( Three_D_View * ) ;  /* 12 Jul 2001 */
 typedef struct {
    int num , meth , hemi ;
    byte *ttbrik ;
-   byte *ttval  ;
+   short *ttval  ;
    byte *ttovc  ;
 } TTRR_params ;
 
@@ -2139,7 +2196,7 @@ extern char * INIT_labovr[] ;
 /**********************************************/
 /***** Setup constants for the color pbar *****/
 
-extern void AFNI_setup_inten_pbar( Three_D_View * ) ;
+extern void AFNI_setup_inten_pbar( MCW_pbar * ) ;
 
 #define DEFAULT_PANES_POS  8
 #define DEFAULT_PANES_SGN  9
