@@ -1,6 +1,5 @@
 #include "afni.h"
 #include "vol2surf.h"
-#include "thd_atlas.h"
 
 /*----------------------------------------------------------------------
  * history:
@@ -610,37 +609,37 @@ ENTRY("AFNI_process_NIML_data") ;
 
    if( strcmp(nel->name,"SUMA_ixyz") == 0 ){
 
-     process_NIML_SUMA_ixyz(nel, ct_start) ;  /* surface nodes for a dataset */
+     process_NIML_SUMA_ixyz(nel, ct_start) ;    /* surface nodes for a dataset */
 
    } else if( strcmp(nel->name,"SUMA_ijk") == 0 ){
 
-     process_NIML_SUMA_ijk(nel, ct_start) ;   /* surface triangles from SUMA */
+     process_NIML_SUMA_ijk(nel, ct_start) ;     /* surface triangles from SUMA */
 
    } else if( strcmp(nel->name,"SUMA_node_normals") == 0 ){
 
-     process_NIML_SUMA_node_normals(nel, ct_start) ;/* node normals for surf */
+     process_NIML_SUMA_node_normals(nel, ct_start) ;  /* node normals for surf */
 
    } else if( strcmp(nel->name,"SUMA_crosshair_xyz") == 0 ){
 
-     process_NIML_SUMA_crosshair_xyz(nel) ;            /* new focus position */
+     process_NIML_SUMA_crosshair_xyz(nel) ;              /* new focus position */
 
    } else if( strcmp(nel->name,"Node_ROI") == 0 ){
 
-     process_NIML_Node_ROI(nel, ct_start) ;         /* ROI drawing from SUMA */
+     process_NIML_Node_ROI(nel, ct_start) ;           /* ROI drawing from SUMA */
 
-   } else if( strncmp(nel->name,"VOLUME_DATA",11) == 0 ){     /* 10 Mar 2005 */
+   } else if( strncmp(nel->name,"VOLUME_DATA",11) == 0 ){       /* 10 Mar 2005 */
 
-     process_NIML_AFNI_volumedata( nel , ct_start ) ;     /* AFNI sub-bricks */
+     process_NIML_AFNI_volumedata( nel , ct_start ) ;       /* AFNI sub-bricks */
 
-   } else if( strcmp(nel->name,"MRI_IMAGE") == 0 ){           /* 22 Mar 2005 */
+   } else if( strcmp(nel->name,"MRI_IMAGE") == 0 ){             /* 22 Mar 2005 */
 
-     process_NIML_MRI_IMAGE( nel , ct_start ) ;       /* store as a .1D file */
+     process_NIML_MRI_IMAGE( nel , ct_start ) ;         /* store as a .1D file */
 
-   } else if( strcmp(nel->name,"3dGroupInCorr_setup") == 0 ){ /* 22 Dec 2009 */
+   } else if( strcmp(nel->name,"3dGroupInCorr_setup") == 0 ){   /* 22 Dec 2009 */
 
      GICOR_setup_func( ns_listen[chan] , nel ) ;
 
-   } else if( strcmp(nel->name,"3dGroupInCorr_dataset") == 0 ){  /* 23 Dec 2009 */
+   } else if( strcmp(nel->name,"3dGroupInCorr_dataset") == 0 ){ /* 23 Dec 2009 */
 
      GICOR_process_dataset( nel , ct_start ) ;
 
@@ -649,7 +648,7 @@ ENTRY("AFNI_process_NIML_data") ;
      sprintf(msg,"*** ERROR:\n\n"
                  " Unknown NIML input: \n"
                  "  <%.222s ...> \n"
-                 " Ignoring it, and hoping it goes away.\n" ,
+                 " Ignoring it, and hoping it goes away quietly\n" ,
                  nel->name) ;
      AFNI_popup_message(msg) ;
    }
@@ -682,13 +681,12 @@ static void AFNI_niml_redisplay_CB( int why, int q, void *qq, void *qqq )
 
 ENTRY("AFNI_niml_redisplay_CB") ;
 
-   /* check inputs for reasonability */
-
+   /* check inputs for reasonability 
+      Do Not return here if function not visible    ZSS: Oct 2011 */
    if( dont_tell_suma            ||
        dont_overlay_suma         ||
-       !IM3D_OPEN(im3d)          ||
-       !im3d->vinfo->func_visible  ) EXRETURN ;
-
+       !IM3D_OPEN(im3d)           ) EXRETURN ;
+   
    sess  = im3d->ss_now   ; if( sess->su_num  == 0    ) EXRETURN ;
    adset = im3d->anat_now ;
    fdset = im3d->fim_now  ; if( fdset         == NULL ) EXRETURN ;
@@ -724,39 +722,46 @@ ENTRY("AFNI_niml_redisplay_CB") ;
      sA = ldp_list.list[kldp].sA;   /* for the sake of laziness */
      sB = ldp_list.list[kldp].sB;
 
-     rdata = NULL;   /* if we want these values, send them to A_vol2surf */
-     rthresh = 0.0;
+     if (!im3d->vinfo->func_visible) {   /* ZSS Oct 2011 */
+        nmap = 0;
+     } else {
+        rdata = NULL;   /* if we want these values, send them to A_vol2surf */
+        rthresh = 0.0;
 
-     if( ldp_list.list[kldp].use_v2s ){            /* vol2surf was requested */
-       nmap = AFNI_vol2surf_func_overlay(im3d, &map, sA,sB, 0, NULL, &rthresh);
-     } else if ( ldp_list.list[kldp].nsurf > 1 ){  /* use v2s with defaults */
-       nmap = AFNI_vol2surf_func_overlay(im3d, &map, sA,sB, 1, NULL, &rthresh);
-     } else {  /* one surface, no request: use vnlist */
-       /* okay, no more vnlist...  :(                   25 Oct 2004 [rickr] */
-       /* nmap = AFNI_vnlist_func_overlay( im3d, sA, &map,&nvused ) ;       */
+        if( ldp_list.list[kldp].use_v2s ){        /* vol2surf was requested */
+          nmap = AFNI_vol2surf_func_overlay(im3d, &map, sA,sB, 
+                                            0, NULL, &rthresh);
+        } else if ( ldp_list.list[kldp].nsurf > 1 ){  /* use v2s with defaults */
+          nmap = AFNI_vol2surf_func_overlay(im3d, &map, sA,sB, 
+                                            1, NULL, &rthresh);
+        } else {  /* one surface, no request: use vnlist */
+          /* okay, no more vnlist...  :(                   25 Oct 2004 [rickr] */
+          /* nmap = AFNI_vnlist_func_overlay( im3d, sA, &map,&nvused ) ;       */
+          nmap = AFNI_vol2surf_func_overlay(im3d, &map, sA, -1, 
+                                            1, NULL, &rthresh);
+        }
 
-       nmap = AFNI_vol2surf_func_overlay(im3d, &map, sA, -1, 1, NULL, &rthresh);
-     }
+   #if 0
+        if( serrit ) fprintf(stderr,"AFNI_niml_redisplay_CB: nmap=%d\n",nmap) ;
 
-#if 0
-     if( serrit ) fprintf(stderr,"AFNI_niml_redisplay_CB: nmap=%d\n",nmap) ;
+        /* we always use v2s now */
+        if( ! v2s && ( nmap < 0 || sess->su_surf[sA]->vn == NULL ) )
+        {
+          if( gv2s_plug_opts.sopt.debug > 0 )
+            fprintf(stderr,"** afni: bad surface %d, ret: %d,%p\n", 
+                           sA, nmap, map);
+          continue ; /* this is bad */
+        }
+   #endif
 
-     /* we always use v2s now */
-     if( ! v2s && ( nmap < 0 || sess->su_surf[sA]->vn == NULL ) )
-     {
-       if( gv2s_plug_opts.sopt.debug > 0 )
-         fprintf(stderr,"** afni: bad surface %d, ret: %d,%p\n", sA, nmap, map);
-       continue ; /* this is bad */
-     }
-#endif
-
-     /* base the error checking on which mapping method was used */
-     if( nmap < 0 || (nmap > 0 && !map) ) /* 29 Sep 2004 [rickr] */
-     {
-       if( gv2s_plug_opts.sopt.debug > 0 )
-         fprintf(stderr,"** bad v2s map %d, ret: %d,%p\n", sA, nmap, map);
-       continue ; /* this is bad */
-     }
+      /* base the error checking on which mapping method was used */
+      if( nmap < 0 || (nmap > 0 && !map) ) /* 29 Sep 2004 [rickr] */
+      {
+        if( gv2s_plug_opts.sopt.debug > 0 )
+          fprintf(stderr,"** bad v2s map %d, ret: %d,%p\n", sA, nmap, map);
+        continue ; /* this is bad */
+      }
+   }
 
      if( nmap > 0 ){  /*--- make a data element with data ---*/
 
