@@ -8,6 +8,7 @@
 #define _MCW_MRILIB_HEADER_
 
 #define MRILIB_7D
+#define _GNU_SOURCE             /* 23 Jun 2011 */
 
 #define COXEMAIL "rwcox@nih.gov"        /* or /dev/null, if you prefer */
 
@@ -15,6 +16,24 @@
 extern "C" {                    /* care of Greg Balls    7 Aug 2006 [rickr] */
 #endif
 
+/*------------------------------------------------------------------*/
+
+#undef INLINE
+#ifdef __GNUC__
+# define INLINE __inline__
+#else
+# define INLINE /*nada*/
+#endif
+
+#undef RESTRICT
+#ifdef __GNUC__
+# define RESTRICT __restrict__
+#else
+# define RESTRICT /*nada*/
+#endif
+
+/*------------------------------------------------------------------*/
+
 extern int MRILIB_verb ;                /* 01 May 2009 */
 
 extern char MRILIB_orients[] ;          /* 12 Mar 2001 */
@@ -40,7 +59,6 @@ extern float MRILIB_slicespacing ;
 extern int MRILIB_DomainMaxNodeIndex ;         /* 32 Dec 2007 */
 
 extern int   assume_dicom_mosaic ;   /* mri_read_dicom.c  13 Mar 2006 [rickr] */
-extern int   use_last_elem;          /* mri_read_dicom.c  10 Apr 2009 [rickr] */
 extern int   use_new_mosaic_code;    /* mri_process_siemens.c 23 Dec 2010 [r] */
 
 /* siemens slice timing info from mri_read.c         13 Apr 2011 [rickr] */
@@ -51,6 +69,8 @@ extern int     populate_g_siemens_times(int tunits);
 extern int     get_and_display_siemens_times(void);
 extern int     valid_g_siemens_times(int nz, float TR, int verb);
 
+/*----------------------------------------------------------------------------*/
+
 #ifdef  __cplusplus
 }
 #endif
@@ -86,12 +106,34 @@ extern AFD_dicom_header **MRILIB_dicom_header ;
 
 /*------------- Macros to be used in OpenMP enabled AFNI code ----------------*/
 
+/* to replace memcpy and memset, which cause trouble inside a parallel region */
+
+#ifdef USE_OMP
+static INLINE void AAmemcpy( void *ooo , void *iii , size_t nnn )
+{ register size_t jj ; register char *oar, *iar ;
+  if( ooo == NULL || iii == NULL || nnn == 0 ) return ;
+  oar = (char *)ooo ; iar = (char *)iii ;
+  for( jj=0 ; jj < nnn ; jj++ ) *oar++ = *iar++ ;
+}
+static INLINE void AAmemset( void *ooo , int c , size_t nnn )
+{ register size_t jj ; register char cc , *oar ;
+  if( ooo == NULL || nnn == 0 ) return ;
+  oar = (char *)ooo ; cc = (char)c ;
+  for( jj=0 ; jj < nnn ; jj++ ) *oar++ = cc ;
+}
+#else
+# define AAmemcpy memcpy
+# define AAmemset memset
+#endif
+
+/* to disable ENTRY/RETURN macros (which use static variables) */
+
 #if defined(USE_OMP) && defined(USE_TRACING)
 # define AFNI_OMP_START DBG_stoff++
 # define AFNI_OMP_END   DBG_stoff--
 #else
-# define AFNI_OMP_START /*nada*/
-# define AFNI_OMP_END   /*nada*/
+# define AFNI_OMP_START   /*nada*/
+# define AFNI_OMP_END     /*nada*/
 #endif
 
 #ifdef USE_OMP
@@ -120,9 +162,11 @@ extern AFD_dicom_header **MRILIB_dicom_header ;
     "* How many threads are useful?  That varies with the program, and how well\n" \
     "   it was coded.  You'll have to experiment on your own systems!\n"           \
     "* The number of CPUs on this particular computer system is ...... %d.\n"      \
+    "* The maximum number of CPUs that will be used is ............... %d.\n"      \
     "%s"                                                                           \
     " =========================================================================\n" \
-    , (pnam) , omp_get_num_procs() , (extra==NULL) ? "\0" : extra                  \
+    , (pnam) , omp_get_num_procs() , omp_get_max_threads() ,                       \
+      (extra==NULL) ? "\0" : extra                                                 \
   )
 #else
 # define PRINT_AFNI_OMP_USAGE(pnam,extra)                                          \
@@ -612,6 +656,9 @@ static int MRI_mm ;
                         (MRI_mm==3||MRI_mm==4) ? (a) :                   \
                         (MRI_mm==7||MRI_mm==0) ? (b) : (c) )
 
+#define ABS(a) ( (a) < 0 ? (-(a)):(a) )
+#define SIGN(a) ( (a) < 0 ? -1:1 )
+
 /*! Order-statistic filter of 3. */
 
 #define OSFSUM(p,q,r) (0.70*(p)+0.15*((q)+(r)))
@@ -722,6 +769,11 @@ extern void mri_add_name( char * , MRI_IMAGE * ) ;
 
 extern MRI_IMAGE ** mri_stat_seq( MRI_IMAGE * ) ;
 
+extern MRI_IMAGE * mri_extract_from_mask( MRI_IMAGE *, byte *, int ) ;
+extern void binarize_mask( int , byte * ) ;
+
+/*--------------------------------*/
+
 #define NSTAT_MEAN        0
 #define NSTAT_SUM         1
 #define NSTAT_SIGMA       2
@@ -738,11 +790,19 @@ extern MRI_IMAGE ** mri_stat_seq( MRI_IMAGE * ) ;
 #define NSTAT_FRANK       22      /* ZSS Jan 10 */
 #define NSTAT_P2SKEW      23      /* ZSS March 04 10*/
 #define NSTAT_KURT        24      /* ZSS Jan   04 11*/
+#define NSTAT_mMP2s0      25
+#define NSTAT_mMP2s1      26
+#define NSTAT_mMP2s2      27
+#define NSTAT_mmMP2s0     28
+#define NSTAT_mmMP2s1     29
+#define NSTAT_mmMP2s2     30
+#define NSTAT_mmMP2s3     31
 
 #define NSTAT_FWHMx      63   /*these should be after all other NSTAT_* values */
 #define NSTAT_FWHMy      64
 #define NSTAT_FWHMz      65
 #define NSTAT_FWHMbar    66
+#define NSTAT_FWHMbar12  67
 
 #define NBISTAT_BASE               66601
 #define NBISTAT_SPEARMAN_CORR      66601
@@ -759,10 +819,17 @@ extern MRI_IMAGE ** mri_stat_seq( MRI_IMAGE * ) ;
 #define NBISTAT_NCD                66612
 #define NBISTAT_KENDALL_TAUB       66613 /* 29 Apr 2010 */
 #define NBISTAT_TICTACTOE_CORR     66614 /* 30 Mar 2011 */
+#define NBISTAT_L2SLOPE            66615 /* 26 Apr 2012 */
+#define NBISTAT_L1SLOPE            66616 /* 26 Apr 2012 */
+#define NBISTAT_QUANTILE_CORR      66617 /* 11 May 2012 */
 
 #define NBISTAT_BC_PEARSON_M       66691
 #define NBISTAT_BC_PEARSON_V       66692
 
+#define NBISTAT_EUCLIDIAN_DIST     66693 /* 4 May 2012, ZSS */
+#define NBISTAT_CITYBLOCK_DIST     66694 /* 4 May 2012, ZSS */
+
+
 extern float mri_nstat  ( int , int , float * , float) ;  /* 19 Aug 2005 */
 extern float mri_nbistat( int , MRI_IMAGE *, MRI_IMAGE * ) ; /* 26 Oct 2006 */
 extern void mri_nbistat_setclip( float, float , float, float ) ;
@@ -801,15 +868,17 @@ extern void   mri_dicom_nohex ( int ) ;
 extern void   mri_dicom_setvm ( int ) ;     /* 28 Oct 2002 */
 extern void   mri_dicom_seterr( int ) ;     /* 05 Nov 2002 */
 extern void   mri_dicom_header_use_printf( int ) ; /* 02 May 2008 */
+extern void   mri_dicom_header_show_size_offset( int ); /* 17 Oct 2012 [rcr] */
 
 extern MRI_IMARR * mri_read_dicom( char * )  ;
 extern int         mri_imcount_dicom( char * ) ;
 extern char *      mri_dicom_sexinfo( void ) ;   /* 23 Dec 2002 */
 extern char *      mri_dicom_sex1010( void ) ;
 extern int         mri_possibly_dicom( char * ) ;        /* 07 May 2003 */
-extern int         mri_siemens_slice_times( int *, int *, float ** ); 
+extern int         mri_siemens_slice_times( int *, int *, float ** );
 extern int         mri_sst_get_verb( void );
 extern int         mri_sst_set_verb( int );
+extern char *      mri_dicom_hdrinfo( char *fname, int natt, char **att , int nposn ) ;
 
 /*! Set the data pointer in an MRI_IMAGE to NULL. */
 
@@ -1072,6 +1141,7 @@ extern void isort_float( int , float * ) ;
 extern void isort_pair ( int , float * , int * ) ;
 
 extern void mri_xsort_inplace( MRI_IMAGE *im , int rev ) ;
+extern void mri_csort_inplace( MRI_IMAGE *im , int rev , int jc ) ; /* 07 Oct 2011 */
 
 extern MRI_IMAGE * mri_nsize( MRI_IMAGE * ) ;
 
@@ -1241,6 +1311,10 @@ extern void memplot_to_RGB_sef( MRI_IMAGE *im , MEM_plotdata *mp ,
 extern void memplot_to_jpg( char * , MEM_plotdata * ) ; /* 05 Dec 2007 */
 extern void memplot_to_png( char * , MEM_plotdata * ) ;
 
+extern void memplot_to_mri_set_dothick( int ) ;         /* 30 Apr 2012 */
+extern void memplot_to_mri_set_dofreee( int ) ;         /* 30 Apr 2012 */
+extern MRI_IMAGE * mri_downsize_by2( MRI_IMAGE * ) ;    /* 27 Apr 2012 */
+
 /************************ Statistics routines *************************/
 
 /**
@@ -1351,6 +1425,8 @@ typedef struct { int nar ; double *ar , dx,x0 ; } doublevec ;
 extern float  interp_floatvec ( floatvec  *fv , float  x ) ;
 extern double interp_doublevec( doublevec *dv , double x ) ;
 
+extern float interp_inverse_floatvec( floatvec *fv , float y ) ;
+
 typedef struct { int nvec ; floatvec *fvar ; } floatvecvec ;
 
 /*----------*/
@@ -1362,6 +1438,8 @@ typedef struct { int nar ; int *ar ; } intvec ;
         free(iv); (iv) = NULL;                 \
   } } while(0)
 
+typedef struct { int nvec ; intvec *ivar ; } intvecvec ;
+
 #define MAKE_intvec(iv,n)                           \
   do{ (iv) = (intvec *)malloc(sizeof(intvec)) ;     \
       (iv)->nar = (n) ;                             \
@@ -1374,6 +1452,12 @@ typedef struct { int nar ; int *ar ; } intvec ;
         (iv)->ar  = (int *)realloc((iv)->ar,sizeof(int)*(m)); \
   }} while(0)
 
+#define APPEND_intvec(iv,jv)                                   \
+  do{ int ni = (iv)->nar ;                                     \
+      RESIZE_intvec((iv),ni+(jv)->nar) ;                       \
+      memcpy( (iv)->ar+ni, (jv)->ar, sizeof(int)*(jv)->nar ) ; \
+  } while(0)
+
 /*----------*/
 
 typedef struct { int nar ; short *ar ; } shortvec ;
@@ -1468,6 +1552,8 @@ extern int SYM_expand_errcount(void) ; /* 03 May 2007 */
 
 #include "misc_math.h"        /* 21 Jun 2010 [rickr] */
 
+#include "thd_atlas.h"        /* 22 Feb 2012 [rickr] */
+
 THD_string_array * mri_read_1D_headerline( char *fname ) ; /* 18 May 2010 */
 
 /*------------------------------------------------------------------------*/
@@ -1544,6 +1630,7 @@ extern MRI_IMAGE * mri_matrix_evalrpn  ( char * ) ;
 extern char      * mri_matrix_evalrpn_help(void) ;
 extern void        mri_matrix_evalrpn_verb(int) ;
 extern float mri_matrix_size( MRI_IMAGE * ) ;
+extern MRI_IMARR * mri_matrix_psinv_ortproj( MRI_IMAGE *, int ) ; /* 13 Dec 2011 */
 
 extern MRI_IMARR * mri_matrix_psinv_pair( MRI_IMAGE *, float ) ;
 extern MRI_IMAGE * mri_matrix_singvals  ( MRI_IMAGE * ) ;
@@ -1592,22 +1679,6 @@ extern double mri_entropy8 ( MRI_IMAGE * ) ;  /* 09 Jan 2004 */
 
 extern float mri_scaled_diff( MRI_IMAGE *bim, MRI_IMAGE *nim, MRI_IMAGE *msk ) ;
 
-/*------------------------------------------------------------------*/
-
-#undef INLINE
-#ifdef __GNUC__
-# define INLINE __inline__
-#else
-# define INLINE /*nada*/
-#endif
-
-#undef RESTRICT
-#ifdef __GNUC__
-# define RESTRICT __restrict__
-#else
-# define RESTRICT /*nada*/
-#endif
-
 #ifdef  __cplusplus
 }
 #endif
@@ -1703,6 +1774,8 @@ extern void mri_metrics( MRI_IMAGE *, MRI_IMAGE *, float * ) ;
 
 #define GA_MATCH_NCDZLIB           14  /* very experimental */
 
+#define GA_MATCH_PEARCLP_SCALAR    15
+
 #define GA_MATCH_METHNUM_SCALAR    14  /* Largest value in sequence above */
 
  /* methods for smoothing images */
@@ -1981,6 +2054,7 @@ extern void mri_blur3D_inmask_speedy( MRI_IMAGE *, byte *,
                                       float,float,float,int );
 extern void mri_blur3D_addfwhm( MRI_IMAGE *, byte *, float ) ;
 extern void mri_blur3D_addfwhm_speedy( MRI_IMAGE *, byte *, float ) ;
+extern void mri_blur3D_inmask_NN( MRI_IMAGE *im, byte *mask, int  ) ;
 extern void mri_blur3D_getfac ( float, float, float, float,
                                 int *, float *, float *, float * ) ;
 
@@ -2085,31 +2159,33 @@ extern void RBF_setup_kranges( RBF_knots *rbk , RBF_evalgrid *rbg ) ;
 
 #undef  VECTORME
 #define VECTORME(inpp,outp)                                                   \
- do{ int vv ; MRI_IMARR *qimar=NULL ; MRI_IMAGE *fim ;                        \
+ do{ int vv ; MRI_IMARR *qxmpq=NULL; MRI_IMAGE *qxm=NULL;                     \
      (outp) = NULL ;                                                          \
      switch( (inpp)->kind ){                                                  \
-       case MRI_fvect:   qimar = mri_fvect_to_imarr(inpp) ; break ;           \
-       case MRI_rgb:     qimar = mri_rgb_to_3float (inpp) ; break ;           \
-       case MRI_complex: qimar = mri_complex_to_pair(inpp); break ;           \
+       default:                                             break ;           \
+       case MRI_fvect:   qxmpq = mri_fvect_to_imarr(inpp) ; break ;           \
+       case MRI_rgb:     qxmpq = mri_rgb_to_3float (inpp) ; break ;           \
+       case MRI_complex: qxmpq = mri_complex_to_pair(inpp); break ;           \
      }                                                                        \
-     if( qimar == NULL ) break ;                                              \
-     for( vv=0 ; vv < IMARR_COUNT(qimar) ; vv++ ){                            \
-       CALLME( IMARR_SUBIM(qimar,vv) , fim ) ;                                \
-       mri_free(IMARR_SUBIM(qimar,vv)) ;                                      \
-       IMARR_SUBIM(qimar,vv) = fim ;                                          \
+     if( qxmpq == NULL ) break ;                                              \
+     for( vv=0 ; vv < IMARR_COUNT(qxmpq) ; vv++ ){                            \
+       CALLME( IMARR_SUBIM(qxmpq,vv) , qxm ) ;                                \
+       mri_free(IMARR_SUBIM(qxmpq,vv)) ;                                      \
+       IMARR_SUBIM(qxmpq,vv) = qxm ;                                          \
      }                                                                        \
      switch( (inpp)->kind ){                                                  \
-       case MRI_fvect:   (outp) = mri_imarr_to_fvect(qimar) ;                 \
+       default:          break ;                                              \
+       case MRI_fvect:   (outp) = mri_imarr_to_fvect(qxmpq) ;                 \
                          break ;                                              \
-       case MRI_rgb:     (outp) = mri_3to_rgb(IMARR_SUBIM(qimar,0),           \
-                                              IMARR_SUBIM(qimar,1),           \
-                                              IMARR_SUBIM(qimar,2) ) ;        \
+       case MRI_rgb:     (outp) = mri_3to_rgb(IMARR_SUBIM(qxmpq,0),           \
+                                              IMARR_SUBIM(qxmpq,1),           \
+                                              IMARR_SUBIM(qxmpq,2) ) ;        \
                          break ;                                              \
-       case MRI_complex: (outp) = mri_pair_to_complex(IMARR_SUBIM(qimar,0),   \
-                                                      IMARR_SUBIM(qimar,1) ); \
+       case MRI_complex: (outp) = mri_pair_to_complex(IMARR_SUBIM(qxmpq,0),   \
+                                                      IMARR_SUBIM(qxmpq,1) ); \
                          break ;                                              \
      }                                                                        \
-     DESTROY_IMARR(qimar) ;                                                   \
+     DESTROY_IMARR(qxmpq) ;                                                   \
    } while(0)
 /*----------------------------------------------------------------------------*/
 
@@ -2121,6 +2197,73 @@ extern MRI_IMAGE * mri_svdproj( MRI_IMARR *imar , int nev ) ;
 extern MRI_IMAGE * mri_first_principal_vector( MRI_IMARR *imar ) ;
 extern int mri_principal_vectors( MRI_IMARR *imar, int nvec, float *sval, float *uvec ) ;
 
+/*----------------------------------------------------------------------------*/
+/* for mri_nwarp.c */
+
+typedef struct {
+  int    nx ,  ny ,  nz ;
+  float *xd , *yd , *zd , *hv , *je , *se ;
+  mat33 emat ; int use_emat ;
+   /* stuff below here is for conversion to/from 3D dataset format */
+  mat44 cmat , imat ;      /* cmat: i->x ; imat: x->i */
+  char *geomstring ;
+  int view ;
+} IndexWarp3D ;
+
+typedef struct {
+  int nwarp ;
+  IndexWarp3D **warp ;
+} IndexWarp3DArray ;
+
+typedef struct {
+  MRI_IMAGE *im ;
+  IndexWarp3D *warp ;
+} Image_plus_Warp ;
+
+typedef struct {
+  IndexWarp3D *fwarp ;
+  IndexWarp3D *iwarp ;
+} IndexWarp3D_pair ;
+
+typedef struct {
+  mat44 fwarp ;
+  mat44 iwarp ;
+} mat44_pair ;
+
+extern IndexWarp3D * IW3D_create( int nx , int ny , int nz ) ;
+extern void IW3D_destroy( IndexWarp3D *AA ) ;
+extern float IW3D_normL1  ( IndexWarp3D *AA , IndexWarp3D *BB ) ;
+extern float IW3D_normL2  ( IndexWarp3D *AA , IndexWarp3D *BB ) ;
+extern float IW3D_normLinf( IndexWarp3D *AA , IndexWarp3D *BB ) ;
+extern IndexWarp3D * IW3D_empty_copy( IndexWarp3D *AA ) ;
+extern IndexWarp3D * IW3D_copy( IndexWarp3D *AA , float fac ) ;
+extern IndexWarp3D * IW3D_sum( IndexWarp3D *AA, float Afac, IndexWarp3D *BB, float Bfac ) ;
+extern void IW3D_scale( IndexWarp3D *AA , float fac ) ;
+extern IndexWarp3D * IW3D_from_dataset( THD_3dim_dataset *dset , int empty , int ivs ) ;
+extern THD_3dim_dataset * IW3D_to_dataset( IndexWarp3D *AA , char *prefix ) ;
+extern float_pair IW3D_load_hexvol( IndexWarp3D *AA ) ;
+extern float_pair IW3D_load_energy( IndexWarp3D *AA ) ;
+extern IndexWarp3D * IW3D_compose( IndexWarp3D *AA , IndexWarp3D *BB     , int icode ) ;
+extern IndexWarp3D * IW3D_invert ( IndexWarp3D *AA , IndexWarp3D *BBinit , int icode ) ;
+extern IndexWarp3D * IW3D_sqrtinv( IndexWarp3D *AA , IndexWarp3D *BBinit , int icode ) ;
+extern IndexWarp3D * IW3D_from_poly( int npar, float *par, IndexWarp3D *WW ) ;
+extern THD_3dim_dataset * NwarpCalcRPN( char *expr, char *prefix, int icode, int acode ) ;
+extern void NwarpCalcRPN_verb(int i) ;
+
+extern void THD_interp_floatim( MRI_IMAGE *fim ,
+                                int np , float *ip , float *jp , float *kp ,
+                                int code, float *outar ) ;
+extern MRI_IMARR * THD_setup_nwarp( MRI_IMARR *bimar, mat44 cmat_bim ,
+                                    int incode      , float wfac     ,
+                                    mat44 cmat_src  ,
+                                    mat44 cmat_out  ,
+                                    int nx_out      , int ny_out     , int nz_out  ) ;
+extern THD_3dim_dataset * THD_nwarp_dataset( THD_3dim_dataset *dset_nwarp ,
+                                             THD_3dim_dataset *dset_src  ,
+                                             THD_3dim_dataset *dset_mast ,
+                                             char *prefix , int interp_code ,
+                                             float dxyz_mast , float wfac , int nvlim ) ;
+
 /*----------------------------------------------------------------------------*/
 
 #define CPU_IS_64_BIT() ((sizeof(void *) == 8) ? 1 : 0 )
