@@ -522,8 +522,8 @@ static char * HISTO_main( PLUGIN_interface * plint )
      for( ii=0 ; ii <= nbin ; ii++ ) hbin[ii] = MIN( hbin[ii] , maxcount ) ;
    }
    hrad = AFNI_numenv("AFNI_1DPLOT_THIK") ;
-   if( hrad <= 0.0f || hrad >= 0.02f ) hrad = 0.005f ;
-   plot_ts_setthik(hrad) ;
+   if( hrad <= 0.0f || hrad >= 0.02f ) hrad = 0.004f ;
+   plot_ts_setTHIK(hrad) ; plot_ts_setthik(0.0015f) ;
    plot_ts_xypush(0,-1) ;
    sprintf(buf,"\\noesc %s[%d] %d voxels",DSET_FILECODE(input_dset),iv,mcount);
    PLUTO_histoplot( nbin,hbot,htop,hbin , NULL , NULL ,  buf , 0,NULL ) ;
@@ -661,7 +661,7 @@ static char *  CORREL_main( PLUGIN_interface * plint )
    MCW_idcode * idc ;
    THD_3dim_dataset * input_dset , * mask_dset = NULL ;
    MRI_IMAGE * tsim , * flim ;
-   int ignore , nvox , ntim , polort , miv , it , ip , nupdt , nbin ;
+   int ignore , nvox , ntim , polort , miv=0 , it , ip , nupdt , nbin ;
    int mcount , ii , jj ;
    float * tsar ;
    float mask_bot=666.0 , mask_top=-666.0 , hbot,htop ;
