@@ -3,6 +3,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <cs.h>
+#include "mrilib.h"
 #include "f2c.h"
 #include <time.h>
 #include <sys/types.h>
@@ -446,7 +447,7 @@ int powell_newuoa_constrained( int ndim, double *x, double *cost ,
        (void)calfun_(&n,x01[tt],x01val+tt) ; ntot += icode+1 ;
        if( x01val[tt] < vbest ){ vbest = x01val[tt]; tbest = tt; }
        if( verb > 1 )
-         ININFO_message("%2d: cost = %g %c",tt,x01val[tt],(tbest==tt)?'*':' ') ;
+         ININFO_message("%2d: cost = %g %c nfunc=%d",tt,x01val[tt],(tbest==tt)?'*':' ',icode) ;
      }
 
      /* Step 2b: sort results by new x01val costs */
@@ -501,7 +502,7 @@ int powell_newuoa_constrained( int ndim, double *x, double *cost ,
      (void)calfun_(&n,x01[tt],x01val+tt) ; ntot += icode+1 ;
      if( x01val[tt] < vbest ){ vbest = x01val[tt]; tbest = tt; }
      if( verb > 1 )
-       ININFO_message("%2d: cost = %g %c",tt,x01val[tt],(tbest==tt)?'*':' ') ;
+       ININFO_message("%2d: cost = %g %c  nfunc=%d",tt,x01val[tt],(tbest==tt)?'*':' ',icode) ;
    }
 
    /*-- Rescale bestest output vector back to 'true' range --*/
