@@ -16,6 +16,10 @@ import afni_util as UTIL
 import afni_base as BASE
 import lib_textdata as TD
 
+MTYPE_NONE = 0   # no modulation        (these work as a bit mask)
+MTYPE_AMP  = 1   # amplitude modulation
+MTYPE_DUR  = 2   # duration modulation
+
 class Afni1D:
    def __init__(self, filename="", from_mat=0, matrix=None, verb=1):
       """1D matrix class (2-D array)
@@ -29,7 +33,7 @@ class Afni1D:
             name       : some text to associate with the datase
             verb       : verbose level of operations
 
-         mat           : data
+         mat           : data (transposed, as vectors of data over time)
          fname         : filename
          nvec, nt      : dimensions
          labels        : simple text labels
@@ -53,6 +57,9 @@ class Afni1D:
       self.nroi      = 1
       self.GLapplied = 0        # was goodlist applied (1=yes, 2=zero-padded)
 
+      # misc variables (from attributes)
+      self.command   = ''       # from CommandLine
+
       # list variables (from attributes)
       self.labels   = []        # label per vector       (from ColumnLabels)
       self.groups   = []        # integral column groups (from ColumnGroups)
@@ -128,13 +135,26 @@ class Afni1D:
       if len(self.run_len) == 0:
          if whine: print '** RIIC: ready be len(run_len) == 0!'
          return 0
-      nt = sum(self.run_len)
+      nt = UTIL.loc_sum(self.run_len)
       if nt != self.nt:
          if whine:
             print '** RIIC: nt=%d != sum of run_len: %s'%(self.nt,self.run_len)
          return 0
       return 1
 
+   # --- begin: mathematical manipulations ---
+
+   def transpose(self):
+      """transpose the matrix and swap nrows and ncols"""
+      if self.verb > 3: print '-- Afni1D transpose...'
+      if not self.ready:
+         print "** matrix '%s' is not ready for transposing" % self.name
+         return
+      self.mat  = [[row[i] for row in self.mat] for i in range(self.nt)]
+      newnt     = self.nvec
+      self.nvec = self.nt
+      self.nt   = newnt
+
    def demean(self):
       """demean each vector per run (mean of each run will be zero)
 
@@ -164,12 +184,14 @@ class Afni1D:
 
       return 0
 
-   def derivative(self):
+   def derivative(self, direct=0):
       """change each value to its derivative (cur val - previous)
          new time[0] will always be 0
 
          process the data per run, so derivatives do not cross boundaries
 
+         direct: 0 means backward difference, 1 means forward
+
          return 0 on success"""
 
       if self.verb > 3: print '-- Afni1D derivative...'
@@ -187,13 +209,20 @@ class Afni1D:
           print "-- derivative: over %d runs of lengths %s" \
                 % (self.nruns, self.run_len)
 
-      # apply derivative to each vector as one run, then clear run breaks
+      # apply derivative to each run of each vector
       for ind in range(self.nvec):
-         UTIL.derivative(self.mat[ind], in_place=1)
-         offset = 0
-         for t in self.run_len:
-            self.mat[ind][offset] = 0
-            offset += t
+         UTIL.derivative(self.mat[ind], in_place=1, direct=direct)
+         # if forward diff, clear final indexes, else clear initial
+         if direct:
+            offset = -1
+            for t in self.run_len:
+               offset += t
+               self.mat[ind][offset] = 0
+         else:
+            offset = 0
+            for t in self.run_len:
+               self.mat[ind][offset] = 0
+               offset += t
 
       return 0
 
@@ -234,7 +263,7 @@ class Afni1D:
 
       return 0
 
-   def collapse_cols(self, method):
+   def collapse_cols(self, method, weight=None):
       """collapsed the matrix to a single array of length nt (nvec will = 1)
 
          collapsing will apply 'method' across the nvec per time point
@@ -244,6 +273,7 @@ class Afni1D:
              method = 'max'             : max across nvec
              method = 'maxabs'          : max abs across nvec
              method = 'euclidean_norm'  : sqrt(sum squares)
+             method = 'weighted_enorm'  : sqrt(sum weighted squares)
 
          Note: the result will still be a trivial 2-D array, where element 0
                is the collapsed time series.  This allows other functionality
@@ -274,6 +304,17 @@ class Afni1D:
       elif method == 'euclidean_norm':
          mat = [UTIL.euclidean_norm([self.mat[v][t] for v in range(self.nvec)])\
                                                     for t in range(self.nt)]
+      elif method == 'weighted_enorm':
+         if weight == None:
+            print '** missing weight vector for weighted_enorm'
+            return 1
+         if len(weight) != self.nvec:
+            print '** weighted_enorm weight vector length (%d) != nvec (%d)' \
+                  % (len(weight), self.nvec)
+            return 1
+         mat = [UTIL.weighted_enorm(
+                  [self.mat[v][t] for v in range(self.nvec)], weight) \
+                  for t in range(self.nt)]
       else:
          print "** collapse_cols: unknown method:", method
          return 1
@@ -286,6 +327,63 @@ class Afni1D:
       self.mat = [mat]
       return 0
 
+   def mat_times_vec(self, vec):
+      """modify matrix as self.mat * vec"""
+      lv = len(vec)
+      if lv == 0 or self.nt == 0 or self.nvec == 0:
+         if self.verb > 1: print '** have mat_times_vec with empty inputs'
+         return
+
+      for col, mvec in enumerate(self.mat):
+         val = vec[col]
+         for ind in range(self.nt):
+            mvec[ind] *= val
+
+   def unitize(self):
+      """make every vector unit length"""
+
+      for ind, vec in enumerate(self.mat):
+         vlen = UTIL.L2_norm(vec)
+         self.mat[ind] = UTIL.lin_vec_sum(1.0/vlen, vec, 0, None)
+
+   def project_out_vec(self, vec=None):
+      """project a vector out of the matrix vectors, if None, use mean
+
+         for each vector y, subtract <vT,y>/<vT.v> * v
+
+         return 0 on success, else 1
+      """
+      if vec == None: vec = self.get_mean_vec()
+      if len(vec) != self.nt:
+         print '** project_out_vec: bad length %d != %d' % (len(vec), self.nt)
+         return 1
+
+      vlen = UTIL.L2_norm(vec)
+      if vlen == 0: # then nothing to do
+         if self.verb > 0: print '-- project_out_vec: empty vector to remove'
+         return 0
+      vunit = UTIL.lin_vec_sum(1.0/vlen, vec, 0, None)
+
+      self.mat = [UTIL.proj_out_vec(mv, vunit, unit_v2=1) for mv in self.mat]
+
+   # --- end: mathematical manipulations ---
+
+   # --- these functions return some aspect of the matrix ---
+
+   def get_mean_vec(self):
+      """return a single vector as the mean of all matrix vectors"""
+      v = [0] * self.nt
+      if self.nt == 0: return []
+      for ind in range(self.nt):
+         v[ind] = UTIL.loc_sum([self.mat[i][ind] for i in range(self.nvec)])
+      return UTIL.lin_vec_sum(1.0/self.nt, v, 0, None)
+
+   # --- end: functions returning some aspect of the matrix ---
+
+   def copy(self):
+      """return a complete (deep)copy of the current object"""
+      return copy.deepcopy(self)
+
    def extreme_mask(self, emin, emax, inclusive=1):
       """convert to a time series mask of 0/1 values where the result is
          1, if value is extreme (>emax or <emin, with = if inclusive)
@@ -300,7 +398,7 @@ class Afni1D:
          else:         print '-- extreme_mask: excluding [%g,%g]'%(emin,emax)
 
       if not self.ready:
-         print '** extremem_mask: Afni1D is not ready'
+         print '** extreme_mask: Afni1D is not ready'
          return 1
 
       if emin > emax:
@@ -311,19 +409,49 @@ class Afni1D:
                         for vec in self.mat]
 
       if self.verb > 1:
-         count = sum([val for val in self.mat[0] if val == 1])
+         count = UTIL.loc_sum([val for val in self.mat[0] if val == 1])
          print '++ extreme_mask: removing %d of %d vals' % (count,self.nt)
 
       return 0
 
-   def mask_first_TRs(self, nfirst, cval=0):
-      """set the first nfirst TRs to cval"""
+   def moderate_mask(self, emin, emax, inclusive=1):
+      """convert to a time series mask of 0/1 values where the result is
+         1, if value is moderate (within (emin,emask), with = if inclusive)
+         0, otherwise (extreme)
+
+         For example, these would be the TRs to keep in a censor.1D file.
+        
+         return 0 on success"""
 
       if self.verb > 3:
-         print '-- masking first %d TRs, cval = %d ...' % (nfirst, cval)
+         if inclusive: print '-- moderate_mask: keeping (%g,%g)'%(emin,emax)
+         else:         print '-- moderate_mask: keeping [%g,%g]'%(emin,emax)
 
       if not self.ready:
-         print '** mask_first_TRs: Afni1D is not ready'
+         print '** moderate_mask: Afni1D is not ready'
+         return 1
+
+      if emin > emax:
+         print '** moderate_mask: emin > emax (', emin, emax, ')'
+         return 1
+
+      self.mat = [UTIL.vec_moderates(vec,emin,emax,inclusive)[1] \
+                        for vec in self.mat]
+
+      if self.verb > 1:
+         count = UTIL.loc_sum([val for val in self.mat[0] if val == 1])
+         print '++ moderate_mask: keeping %d of %d vals' % (count,self.nt)
+
+      return 0
+
+   def set_first_TRs(self, nfirst, newval=1):
+      """set the first nfirst TRs to newval"""
+
+      if self.verb > 3:
+         print '-- setting first %d TRs, newval = %d ...' % (nfirst, newval)
+
+      if not self.ready:
+         print '** set_first_TRs: Afni1D is not ready'
          return 1
 
       # apply derivative to each vector as one run, then clear run breaks
@@ -335,7 +463,7 @@ class Afni1D:
                      % (nfirst, rlen, run+1)
                return 1
             # apply censor val for first nfirst TRs
-            for tr in range(nfirst): self.mat[ind][offset+tr] = cval
+            for tr in range(nfirst): self.mat[ind][offset+tr] = newval
             offset += rlen
 
       return 0
@@ -355,6 +483,21 @@ class Afni1D:
 
       return 0
 
+   def clear_prior_TRs(self):
+      """if one TR is clear, also clear the prior one"""
+
+      if self.verb > 3: print '-- clearing prior TRs...'
+
+      if not self.ready:
+         print '** clear_prior_TRs: Afni1D is not ready'
+         return 1
+
+      for v in range(self.nvec):
+         for t in range(self.nt-1):
+            if not self.mat[v][t+1] and self.mat[v][t]: self.mat[v][t] = 0
+
+      return 0
+
    def pad_into_many_runs(self, rindex1, nruns, rlengths=[]):
       """pad over time so that this is run #rindex1 out of nruns runs
          (rindex1 should be 1-based)
@@ -425,10 +568,19 @@ class Afni1D:
       # update run info
       self.nruns   = NR
       self.run_len = rlens
-      self.nt      = sum(rlens)
+      self.nt      = UTIL.loc_sum(rlens)
 
       return 0
 
+   def randomize_trs(self, seed=0):
+      """reorder the matrix columns randomly"""
+      if self.verb > 1: print '-- randomizing %d trs (seed=%d)' \
+                              % (self.nt, seed)
+      if seed:
+         import random
+         random.seed(seed)
+      for vec in self.mat: UTIL.shuffle(vec)
+
    def sort(self, reverse=0):
       """sort data over time axis (possibly reverse order)"""
 
@@ -444,13 +596,13 @@ class Afni1D:
 
       if self.verb > 3: print '-- Afni1D reverse...'
 
-      ilist = UTIL.decode_1D_ints('$..0(-1)',verb=self.verb,max=self.nt-1)
+      ilist = UTIL.decode_1D_ints('$..0(-1)',verb=self.verb,imax=self.nt-1)
       if self.reduce_by_tlist(ilist): return 1
 
       return 0
 
    def show_censor_count(self, invert=0, column=0):
-      """display the total number of TRs censored (set) in the given column
+      """display the total number of TRs censored (clear) in the given column
 
          If multi-column data, can choose one.
 
@@ -466,10 +618,11 @@ class Afni1D:
          print "** Afni1D not ready for write_timing to '%s'" % fname
          return 1
 
-      total = self.mat[0].count(0)              # start with inverted count
-      if not invert: total = self.nt - total
+      total = self.mat[0].count(0)              # start with censor count
+      if invert: total = self.nt - total
 
-      print 'total number of censored TRs = %d' % total
+      if self.verb: print 'total number of censored TRs = %d' % total
+      else:         print total
 
       return 0
 
@@ -490,14 +643,18 @@ class Afni1D:
       if not fname:
          print "** missing filename for write"
          return 1
-      if os.path.exists(fname) and not overwrite:
-         print "** output file '%s' exists and 'overwrite' not set..." % fname
-         return 1
 
-      fp = open(fname, 'w')
-      if not fp:
-         print "** failed to open '%s' for writing" % fname
-         return 1
+      if fname == '-' or fname == 'stdout': fp = sys.stdout
+      else:
+         # normal file name: check existence and open
+         if os.path.exists(fname) and not overwrite:
+            print "** output file '%s' exists and 'overwrite' not set..."%fname
+            return 1
+
+         fp = open(fname, 'w')
+         if not fp:
+            print "** failed to open '%s' for writing" % fname
+            return 1
 
       for row in range(self.nt):
          fp.write(sep.join(['%g' % self.mat[i][row] for i in range(self.nvec)]))
@@ -685,17 +842,6 @@ class Afni1D:
 
       return 0
 
-   def transpose(self):
-      """transpose the matrix and swap nrows and ncols"""
-      if self.verb > 3: print '-- Afni1D transpose...'
-      if not self.ready:
-         print "** matrix '%s' is not ready for transposing" % self.name
-         return
-      self.mat  = [[row[i] for row in self.mat] for i in range(self.nt)]
-      newnt     = self.nvec
-      self.nvec = self.nt
-      self.nt   = newnt
-
    def simplecopy(self):
       return Afni1D(from_mat=1, matrix=self.mat, verb=self.verb)
 
@@ -994,7 +1140,7 @@ class Afni1D:
          if not UTIL.vals_are_positive(rlens):
             print "** set_runs: non-positive run length in list: %s" % rlens
             return 1
-         if sum(rlens) != self.nt:
+         if UTIL.loc_sum(rlens) != self.nt:
             print "** set_runs: sum of run lengths != nt (%d): %s" \
                   % (self.nt, rlens)
             return 1
@@ -1194,6 +1340,174 @@ class Afni1D:
       if verb > 0: print 'rows = %d, cols = %d' % (self.nt, self.nvec)
       else:        print '%d %d' % (self.nt, self.nvec)
 
+   def apply_censor_dset(self, cset=None):
+      """remove censored TRs from self.mat
+
+         ** for now, it becomes 1 combined run
+      """
+
+      mat = self.mat
+      nt = self.nt
+
+      if self.verb > 1: print '-- censoring input data...'
+
+      if cset == None:   return
+      if cset.nvec == 0: return
+      if self.nvec == 0: return
+
+      if cset.nt != nt:
+         print '** ACD: censort length (%d) does not match NT (%d)' \
+               % (cset.nt, nt)
+         return
+
+      clist = cset.mat[0]       # get censor list for ease of typing
+
+      if self.verb > 2:
+         nkeep = UTIL.loc_sum([v for v in clist if v])
+         print '-- censoring from length %d to length %d' % (nt, nkeep)
+
+      # if nothing is censored out, we're done
+      if UTIL.vals_are_constant(clist, 1): return
+
+      # else, make a new matrix
+      cmat = [[col[ind] for ind in range(nt) if clist[ind]] for col in mat]
+
+      del(self.mat)
+      self.mat = cmat
+
+      # now clean up some extra fields
+
+      self.nt = len(cmat[0])
+      self.nrowfull = self.nt
+
+      # could deal with run lengths here, but save for later
+      self.nruns = 1
+      self.run_len = [self.nt]
+      self.goodlist = []
+      self.runstart = []
+
+      del(self.cormat)
+      del(self.cosmat)
+      self.cormat = []
+      self.cosmat = []
+      self.cormat_ready = 0
+
+      return
+
+   def get_censored_mat(self, cset=None):
+      """return self.mat, restricted to uncensored TRs"""
+
+      mat = self.mat
+      nt = self.nt
+
+      if cset == None:   return mat
+      if cset.nvec == 0: return mat
+
+      if cset.nt != nt:
+         print '** GCM: censort length (%d) does not match NT (%d)' \
+               % (cset.nt, nt)
+         return None
+
+      clist = cset.mat[0]       # get censor list for ease of typing
+
+      # if nothing is censored out, return the original
+      if UTIL.vals_are_constant(clist, 1): return mat
+
+      # else, make a copy
+      return [[col[ind] for ind in range(nt) if clist[ind]] for col in mat]
+
+   def get_max_displacement(self, dtype=1, cset=None):
+      """compute the maximum pairwise displacement among the
+         N-dimensional coordinates (e.g. motion params)
+
+         dtype = 1: enorm
+
+            - so compute max(dist(x,y)) over all x, y coordinate pairs
+
+         dtype = 0: max abs
+
+            compute max absolue pairwise displacement among the N dimensions
+
+            - so compute max(abs(v_i,j - v_i,k)) over all all dimensions i
+              and j,k index pairs
+            - or compute as max(max(v_i)-min(v_i)) over dimensions i
+
+          cset is an optional censor dataset (1=keep, 0=censor)
+      """
+
+      maxdiff = 0
+
+      mat = self.get_censored_mat(cset)
+      if mat == None: mat = self.mat
+
+      if self.nvec > 0: nt = len(mat[0])
+      else:             nt = 0
+
+      if dtype == 0:    # max vector displacement
+         maxdiff = max([max(vec)-min(vec) for vec in mat])
+
+      else:             # max euclidean distance
+
+         # get all distances, a 2D list of the form [[DIST, i, j]]
+         # (so length is NT choose 2 = (NT*(NT-1)/2), which is possibly long)
+
+         en = UTIL.euclidean_norm
+         dlist = [[en([mat[i][r1]-mat[i][r2] for i in range(self.nvec)]),
+                     r1, r2] for r1 in range(nt) for r2 in range(r1)]
+
+         # and grab the biggest
+         dlist.sort()
+         maxdiff = dlist[-1][0]
+         
+         if self.verb > 1:
+            i = dlist[-1][1]
+            j = dlist[-1][2]
+            print '++ max edisp %s between indices %d and %d' % (maxdiff, i, j)
+
+         del(dlist)
+
+      return maxdiff
+
+   def get_max_displacement_str(self, mesg='', enorm=2, verb=1):
+      """display the maximum displacement based on enorm
+            enorm = 1   : absolute displacement over dims only
+            enorm = 2   : enorm only
+            enorm = 3   : both
+      """
+
+      rstr = ''         # return string
+
+      if enorm == 1 or enorm == 3:
+         maxabval = self.get_max_displacement(0)
+      if enorm:
+         maxenorm = self.get_max_displacement(1)
+
+      if verb > 0:
+         if enorm == 1:   rstr = 'max_param_diff = %g' % maxabval
+         elif enorm == 3: rstr = 'max_param_diff = %g, max_enorm_diff = %g' \
+                                 % (maxabval, maxenorm)
+         else:            rstr = 'max_enorm_diff = %g' % maxenorm
+      else:
+         if enorm == 1:   rstr = '%g' % maxabval
+         elif enorm == 3: rstr = '%g %g' % (maxabval, maxenorm)
+         else:            rstr = '%g' % maxenorm
+
+      if mesg: return "%s %s" % (mesg, rstr)
+      else: return rstr
+
+   def show_min_mean_max_stdev(self, col=-1, verb=1):
+      """show min, mean, max, stdev for each column (unless col specified)"""
+
+      if verb: print "file %s (len %d)" % (self.fname, self.nt)
+      for cind, col in enumerate(self.mat):
+         if verb:
+            ps = "    col %d: " % cind
+            form = "min = %7.4f, mean = %7.4f, max = %7.4f, stdev = %7.4f"
+         else:
+            ps = ''
+            form = "%7.4f %7.4f %7.4f %7.4f"
+         print ps + form % UTIL.min_mean_max_stdev(col)
+
    def get_indices_str(self, ind_types):
       """return an index list (sub-brick selector form) for the
          following groups:
@@ -1303,7 +1617,9 @@ class Afni1D:
       return 0
 
    def init_from_general_name(self, name):
-      """might contain [] or {} selection"""
+      """might contain [] or {} selection
+         might be '-' or stdin"""
+
       aname = BASE.afni_name(self.fname)
 
       if self.init_from_1D(aname.rpve()): return 1 # failure
@@ -1315,11 +1631,11 @@ class Afni1D:
       # apply column and/or row selectors
       if aname.colsel:
          ilist = UTIL.decode_1D_ints(aname.colsel, verb=self.verb,
-                                     max=self.nvec-1)
+                                     imax=self.nvec-1)
          if ilist == None: return 1
          if self.reduce_by_vec_list(ilist): return 1
       if aname.rowsel:
-         ilist = UTIL.decode_1D_ints(aname.rowsel,verb=self.verb,max=self.nt-1)
+         ilist = UTIL.decode_1D_ints(aname.rowsel,verb=self.verb,imax=self.nt-1)
          if ilist == None: return 1
          if self.reduce_by_tlist(ilist): return 1
 
@@ -1384,6 +1700,10 @@ class Afni1D:
                           (len(self.groups), self.nvec)
                if self.verb > verb_level:
                   print "-- label %s: groups %s" % (label,self.groups)
+            elif label == 'CommandLine':
+               self.command = data
+               if self.verb > verb_level:
+                  print "-- label %s: command %s" % (label,self.command)
             elif label == 'RowTR':
                self.tr = float(data)
                if self.verb > verb_level:
@@ -1477,7 +1797,7 @@ ERR_ST_NUM_RUNS   =  8       # number of runs mismatch
 ERR_ST_TOO_BIG    = 16       # val >= run length
 
 
-class AfniData:
+class AfniData(object):
    def __init__(self, filename="", verb=1):
       """akin to a 2D float class, but do not require a square matrix
 
@@ -1498,11 +1818,10 @@ class AfniData:
       self.ncols     = 0        # non-zero if rectangular
       self.row_lens  = []       # row lengths, if not rectangular
 
-      self.rect      = 0        # rectangular?
-      self.square    = 0        # square?
       self.binary    = 0        # 0/1 file?
       self.empty     = 0        # no data at all
       self.married   = 0        # data has modulators or durations
+      self.mtype     = 0        # married type (bits, amp, dur)
 
       self.ready     = 0        # data is ready
 
@@ -1519,31 +1838,8 @@ class AfniData:
       self.cormat_ready = 0     # correlation mat is set
 
       # initialize...
-      if self.init_from_filename(self.fname): return None
-
-
-   def show(self):
-      print self.make_show_str()
-
-   def make_show_str(self):
-      if self.ready: rstr = 'ready'
-      else:          rstr = 'not ready'
-
-      mstr = "++ name     : %s (%s)\n" \
-             "   fname    : %s\n" \
-             "++ nrows    : %d\n" \
-             "   ncols    : %d\n" \
-             "   row_lens : %s\n" \
-             "++ rect     : %d\n" \
-             "   square   : %d\n" \
-             "   binary   : %d\n" \
-             "++ tr       : %g\n" \
-             "   nruns    : %d\n" \
-             % (self.name, rstr, self.fname,
-                self.nrows, self.ncols, self.row_lens,
-                self.rect, self.square, self.binary, self.tr, self.nruns)
-
-      return mstr
+      if self.fname: 
+         if self.init_from_filename(self.fname): return None
 
    # some accessor functions to match Afni1D
    def set_nruns(nruns): self.nruns = nruns
@@ -1551,6 +1847,275 @@ class AfniData:
    def set_run_lengths(run_lengths):
       self.row_lens = run_lengths
 
+   def is_rect(self):
+      return TD.data_is_rect(self.mdata)
+
+   def is_square(self):
+      if not self.is_rect(): return 0
+
+      rlen = len(self.mat)
+
+      if rlen == 0: return 1
+      if rlen == len(self.mat[0]): return 1
+      
+      return 0
+
+   def get_duration(self):
+      """return 0       : if empty
+                dur     : if constant
+                -1      : otherwise (existing, but not constant)
+      """
+
+      if self.mtype == 0 or len(self.mdata) == 0: return 0
+
+      # have actual married data
+
+      dur = -2
+      for row in self.mdata:
+         for entry in row:
+            if dur == -2: dur = entry[2]
+            elif dur != entry[2]:
+               dur = -1
+               break
+         if dur == -1: break
+      if dur == -2: return 0    # uninitialized means 0
+      return dur                # -1 or good value
+
+   def extend_data_rows(self, newdata):
+      """extend each row by the corresponding row of newdata"""
+      if not self.ready or not newdata.ready:
+         print '** timing elements not ready for extending rows (%d,%d)' % \
+               (self.ready, newdata.ready)
+         return 1
+
+      if self.nrows != newdata.nrows:
+         print '** timing nrows differ for extending (%d, %d)' % \
+               (self.nrows,newdata.nrows)
+         return 1
+
+      if self.mtype != newdata.mtype:
+         print '** timing elements differ in married type (%s, %s)' % \
+               (self.married_type_string(self.mtype),
+                self.married_type_string(newdata.mtype))
+         return 1
+
+      if self.verb > 1: print '++ MTiming: extending %d rows' % self.nrows
+
+      for ind in range(self.nrows):
+         self.data[ind].extend(newdata.data[ind])
+
+      for ind in range(self.nrows):
+         self.mdata[ind].extend(newdata.mdata[ind])
+
+      return 0
+
+   def show_married_info(self):
+      print '-- modulation type: %s' % self.married_info_string()
+
+   def married_info_string(self):
+      if   self.mtype == MTYPE_NONE: return 'not a married format'
+
+      namp = self.num_amplitudes()
+      adur = self.ave_dur_modulation()
+
+      if self.mtype == MTYPE_AMP:
+         return 'amplitude modulation (%d modulators)' % namp
+
+      if self.mtype == MTYPE_DUR:
+         return 'duration modulation (average duration %g)' % adur
+
+      if self.mtype == MTYPE_AMP|MTYPE_DUR:
+         return 'amp and dur modulation (%d amp mods, ave dur %g)'%(namp, adur)
+
+      return '** invalid modulation type %d' % self.mtype
+
+   def num_amplitudes(self):
+      if not self.mtype & MTYPE_AMP: return 0
+      if not self.mdata: return 0
+      for row in self.mdata:
+         if len(row) == 0: continue
+         return len(row[0][1])  # have amplitudes, return the length
+      return 0 # no valid rows found
+
+   def ave_dur_modulation(self):
+      if not self.mtype & MTYPE_DUR: return 0
+      if not self.mdata: return 0
+      sum, count = 0.0, 0
+      for row in self.mdata:
+         if len(row) == 0: continue
+         count += len(row)
+         sum += UTIL.loc_sum([entry[2] for entry in row])
+      if count == 0: return 0
+      return sum*1.0/count # be wary of integers
+
+   def married_type_string(self, mtype=None):
+      if mtype == None: mtype = self.mtype
+
+      if   mtype == MTYPE_NONE:            return 'None'
+      elif mtype == MTYPE_AMP:             return 'Amp Mod'
+      elif mtype == MTYPE_DUR:             return 'Dur Mod'
+      elif mtype == MTYPE_AMP | MTYPE_DUR: return 'Amp/Dur Mod'
+      else:                                return 'Unknown'
+
+   def randomize_trs(self, seed=0):
+      """reorder the matrix rows randomly"""
+      if self.verb > 1: print '-- randomizing %d trs (seed=%d)' \
+                              % (len(self.data, seed))
+      if seed:
+         import random
+         random.seed(seed)
+      UTIL.shuffle(self.data)
+
+   def sort(self, rev=0):
+      """sort each row (optionally reverse order)"""
+      if not self.ready: return 1
+
+      if self.verb > 1: print '-- sorting AfniData ...'
+
+      for row in self.data:
+         if rev: row.sort(reverse=True)
+         else:   row.sort()
+
+      for row in self.mdata:
+         if rev: row.sort(reverse=True)
+         else:   row.sort()
+
+      return 0
+
+   def transpose(self):
+      """the tranpose operation requires rectangular data"""
+      if not self.ready: return 1
+      if not self.is_rect():
+         print '** cannot take transpose, data is not rectangular'
+         return 1
+
+      if self.verb > 1: print '-- AData: taking transpose...'
+
+      # det mdata and data
+      newdata = []
+      if not self.empty:
+         for col in range(len(self.data[0])):
+            newdata.append([self.data[row][col] for row in range(self.nrows)])
+      del(self.data)
+      self.data = newdata
+
+      newdata = []
+      if not self.empty:
+         for col in range(len(self.mdata[0])):
+            newdata.append([self.mdata[row][col] for row in range(self.nrows)])
+      del(self.mdata)
+      self.mdata = newdata
+
+      self.nrows = len(self.mdata)
+
+      return 0
+
+   def copy(self):
+      """return a complete (deep)copy of the current AfniTiming"""
+      return copy.deepcopy(self)
+
+   def is_empty(self):
+      """empty is either nrows == 0 or each row is empty"""
+      if not self.ready: return 1       # seems safer
+
+      if self.nrows < 1: return 1
+
+      # now check each row
+      for row in self.mdata:
+         if len(row) > 0: return 0      # found something
+
+      return 1
+
+   def make_single_row_string(self, row=-1, nplaces=3, flag_empty=0,
+                              check_simple=1):
+      """return a string of row data, to the given number of decimal places
+         if row is non-negative, return a string for the given row"""
+      if not self.ready: return ''
+      if row < 0 or row >= self.nrows:
+         if self.verb > 0: print '** row %d out of range for printing' % row
+         return ''
+
+      if check_simple and self.dur_len == 0.0: simple = 1
+      elif self.mtype == 0:                    simple = 1
+      else:                                    simple = 0
+
+      data = self.mdata[row]
+      rstr = ''
+      if self.verb > 2 and not flag_empty: rstr += 'run %02d : ' % (row+1)
+
+      # rcr - fix
+      # if flagging an empty run, use '*' characters
+      if len(data) == 0 and flag_empty:
+         if row == 0: rstr += '* *'
+         else:        rstr += '*'
+
+      for val in data:
+         if simple:
+            if nplaces >= 0: rstr += '%.*f ' % (nplaces, val[0])
+            else:            rstr += '%g ' % (val[0])
+         else:
+            if self.mtype & MTYPE_AMP and len(val[1]) > 0:
+               alist = ['*%s'%a for a in val[1]]
+               astr = ''.join(alist)
+            else: astr = ''
+
+            if self.mtype & MTYPE_DUR:
+               dstr = ':%s' % val[2]
+            else: dstr = ''
+
+            if nplaces >= 0: rstr += '%.*f%s%s ' % (nplaces, val[0], astr, dstr)
+            else: rstr += '%s%s%s ' % (val[0], astr, dstr)
+
+      return rstr + '\n'
+
+   def make_data_string(self, row=-1, nplaces=3, flag_empty=0, check_simple=1,
+                        mesg=''):
+      """return a string of row data, to the given number of decimal places
+         if row is non-negative, return a string for the given row, else
+         return a string of all rows
+            row          : make a string for just the single row
+            nplaces      : number of decimal places to show
+            flag_empty   : if empty row, use the '*' format
+            check_simple : if set and dur_len=0, use timing format
+            mesg         : display the message before data
+      """
+
+      # init return string based on message
+      if len(mesg) > 0: rstr = "%s :\n" % mesg
+      else:             rstr = ''
+
+      if row >=0:
+         return rstr+self.make_single_row_string(row, nplaces, flag_empty,
+                                                 check_simple)
+
+      # make it for all rows
+      for ind in range(self.nrows):
+         rstr += self.make_single_row_string(ind, nplaces, flag_empty,
+                                             check_simple)
+
+      return rstr
+
+   def write_as_timing(self, fname='', nplaces=-1):
+      """write the current M timing out, with nplaces right of the decimal"""
+      if not self.ready:
+         print '** M Timing: not ready to write'
+         return 1
+
+      if fname == '': fname = self.fname
+
+      fp = open(fname, 'w')
+      if not fp:
+         print "** failed to open '%s' for writing Mtiming" % fname
+         return 1
+
+      if self.verb > 0:
+         print "++ writing %d MTiming rows to %s" % (self.nrows, fname)
+
+      fp.write(self.make_data_string(nplaces=nplaces, flag_empty=1))
+      fp.close()
+
+      return 0
+
    def looks_like_1D(self, run_lens=[], nstim=0, verb=1):
       """check if the file looks like 1D
          - if so and verb, show warnings, too"""
@@ -1593,7 +2158,7 @@ class AfniData:
       # error occur only as singles, to test against verb > 1
 
       errors = 0
-      if not self.rect:
+      if not self.is_rect():
          errors |= ERR_ANY_MISC
          if verb > 1: print "** file %s is not rectangular" % self.fname
 
@@ -1603,7 +2168,7 @@ class AfniData:
       nruns = len(rlens)
       if nruns > 0:
          # if TR, scale it in
-         tot_dur = sum(rlens)
+         tot_dur = UTIL.loc_sum(rlens)
 
          # if nrows is too small, error -- if too big, just warn
          if tot_dur > self.nrows:
@@ -1643,7 +2208,7 @@ class AfniData:
       nruns = len(rlens)
       if nruns > 0:
          # if TR, scale it in
-         tot_dur = sum(rlens)
+         tot_dur = UTIL.loc_sum(rlens)
 
          if tot_dur < self.nrows:
             print "** warning for 1D file %s, more rows than TRs: %d > %d" \
@@ -1841,7 +2406,7 @@ class AfniData:
          if verb > 1: print "** file %s is not a single column" % self.fname
 
       # must be rectangular
-      if not self.rect:
+      if not self.is_rect():
          errors |= ERR_ANY_MISC
          if verb > 1: print "** file %s is not a rectangular" % self.fname
 
@@ -1889,7 +2454,7 @@ class AfniData:
       else:        rlens = run_lens
 
       # note the total duration (tr == -1 implies just count TRs)
-      endoftime = sum(rlens)
+      endoftime = UTIL.loc_sum(rlens)
       if tr > 0.0: endoftime *= tr
 
       if data[-1] >= endoftime:
@@ -1917,8 +2482,8 @@ class AfniData:
          return 1
 
       # note whether the data is married (modulation or duration)
-      if TD.married_type(mdata):
-         self.married = 1
+      self.mtype = TD.married_type(mdata)
+      if self.mtype: self.married = 1
 
       # data will ignore any married information
       self.data     = [[val[0] for val in row] for row in mdata]
@@ -1944,9 +2509,6 @@ class AfniData:
          self.ncols = self.row_lens[0]
          del(self.row_lens)
          self.row_lens = []
-         self.rect  = 1
-
-      if self.rect and (self.nrows == self.ncols): self.square = 1
 
       # check to see if it is a 0/1 file
       self.binary = 1
@@ -1959,6 +2521,38 @@ class AfniData:
       
       return 0
 
+   def show(self, mesg=''):
+      print self.make_show_str(mesg=mesg)
+
+   def make_show_str(self, mesg=''):
+      if self.ready: rstr = 'ready'
+      else:          rstr = 'not ready'
+
+      rect = self.is_rect()
+
+      if mesg: mstr = '%s : ' % mesg
+      else:    mstr = ''
+
+      mstr = "--- %sAfniData element ---\n" \
+             "   name     : %s (%s)\n"      \
+             "   fname    : %s\n" \
+             "   nrows    : %d\n" \
+             "   ncols    : %d\n" \
+             "   rect     : %d\n" \
+             "   row_lens : %s\n" \
+             "   binary   : %d\n" \
+             "   tr       : %g\n" \
+             "   nruns    : %d\n" \
+             "   married  : %d\n" \
+             "   mtype    : %d\n" \
+             "   verb     : %d\n" \
+             % ( mstr, self.name, rstr, self.fname,
+                self.nrows, self.ncols, rect, self.row_lens,
+                self.binary, self.tr, self.nruns, self.married, self.mtype,
+                self.verb)
+
+      return mstr
+
 if __name__ == '__main__':
    print '** this is not a main module'
    sys.exit(1)
