@@ -42,6 +42,7 @@
  *        -show_bad_backslash  show any lines with only whitespace after '\'
  *        -show_bad_char       show any non-printable characters
  *        -show_file_type      show whether UNIX, Mac or MS file type
+ *        -test                same as -show_bad_all
  *
  *     raw ascii options:
  *
@@ -128,9 +129,10 @@ static char g_history[] =
  " 3.7  August 2, 2007  - added -disp_hex, -disp_hex{1,2,4}\n"
  " 3.8  June 19, 2008   - removing printing of pointers in disp_ functions\n"
  " 3.9  Oct 27, 2010    - added -show_bad_char and -show_bad_all\n"
+ " 3.10 Oct 18, 2012    - option -test is same as -show_bad_all\n"
  "----------------------------------------------------------------------\n";
 
-#define VERSION         "3.9 (October 27, 2010)"
+#define VERSION         "3.10 (October 18, 2012)"
 
 
 /* ----------------------------------------------------------------------
@@ -158,6 +160,44 @@ char g_rep_output_data[MAX_STR_LEN];    /* in case user doesn't pass it in */
 static int swap_2 ( void * ptr );
 static int swap_4 ( void * ptr );
 
+/*------------------------------------------------------------
+ * see if there is a BOM (byte order mark) at the beginning of
+ * the file
+ *
+ * return index into BOM_list
+ *------------------------------------------------------------
+*/
+static char * BOM_list[] = {
+   "no BOM",
+   "UTF-32, big-endian",
+   "UTF-32, little-endian",
+   "UTF-16, big-endian",
+   "UTF-16, little-endian",
+   "UTF-8"
+};
+
+#include "nifti1.h"
+
+static int BOM_val_type(char * data, int len)
+{
+   char bom_32be[4] = { 0x00, 0x00, 0xfe, 0xff };
+   char bom_32le[4] = { 0xfe, 0xff, 0x00, 0x00 };
+   char bom_16be[2] = { 0xfe, 0xff };
+   char bom_16le[2] = { 0xff, 0xfe };
+   char bom_8[3]    = { 0xef, 0xbb, 0xbf };
+
+   if( ! data )   return 0;
+
+   if( len >= 4 && ! memcmp(data, bom_32be, 4) ) return 1;
+   if( len >= 4 && ! memcmp(data, bom_32le, 4) ) return 2;
+   if( len >= 2 && ! memcmp(data, bom_16be, 2) ) return 3;
+   if( len >= 2 && ! memcmp(data, bom_16le, 2) ) return 4;
+   if( len >= 3 && ! memcmp(data, bom_8,    3) ) return 5;
+
+   return 0;
+}
+
+
 /*------------------------------------------------------------
  *  - check that the program is used correctly
  *  - fill the param_t struct, based on user inputs
@@ -276,15 +316,16 @@ scr_show_bad_bs( char * filename, param_t * p )
         if( count == cur || fdata[count] != '\n' )  /* we're okay */
             continue;
 
-        bad++;
-
         /* this line is bad, and we're looking at '\n' */
         if( !p->quiet )
         {
-            printf("bad line %4d of file %s: ", lnum, filename);
+            if( !bad ) printf("file '%s' has bad backslashes:\n", filename);
+            printf("   bad line %4d : ", lnum);
             for( cp = line_start; cp <= fdata+count; cp++)
                 putchar(*cp);
         }
+
+        bad++;
     }
 
     if(p->debug > 0) fprintf(stderr,"file %s: %d bad lines\n",filename,bad);
@@ -303,7 +344,7 @@ scr_show_bad_ch( char * filename, param_t * p )
     static char * fdata = NULL;
     static int    flen  = 0;
     char        * cp;
-    int           length, count, bad = 0;
+    int           length, count, bad = 0, bad_loc=-1;
 
     if( p->debug ) fprintf(stderr,"-- show_bad_chars: file %s ...\n",
                            filename);
@@ -319,14 +360,18 @@ scr_show_bad_ch( char * filename, param_t * p )
             if( p->debug ) {
                 if( !bad ) fprintf(stderr,"file '%s' has non-printable chars\n",
                                    filename);
-                fprintf(stderr,": %d (0x%0x)",cp[count],cp[count]);
+                if( !bad ) fputs("bad chars", stderr);
+                fprintf(stderr," : %d (0x%0x)",cp[count],0xff & cp[count]);
             }
+            if( bad_loc < 0 ) bad_loc = count;
             bad++;
         }
     }
 
     if( bad && p->debug ) putc('\n', stderr);
-    printf("%s has %d bad characters\n", filename, bad);
+    printf("%s has %d bad characters", filename, bad);
+    if ( bad ) printf(", starting at position %d\n", bad_loc);
+    else       putchar('\n');
     return 0;
 }
 
@@ -343,7 +388,7 @@ scr_show_file( char * filename, param_t * p )
     static char * fdata = NULL;
     static int    flen  = 0;
     char        * cp;
-    int           length, count, bin = 0;
+    int           length, count, bin = 0, bom;
 
     if( p->debug ) fprintf(stderr,"-- show_file_type: file %s ...\n",filename);
 
@@ -351,43 +396,51 @@ scr_show_file( char * filename, param_t * p )
 
     if( p->debug ) fprintf(stderr,"file length = %d\n", length);
 
+    bom = BOM_val_type(fdata, length);
+    if( bom )
+        fprintf(stderr,"file '%s' has BOM header: %s\n",filename,BOM_list[bom]);
+
     for( cp = fdata, count = 0; count < length; count++ )
     {
         if( !isprint(cp[count]) && !isspace(cp[count]) )
         {
-            if( bin == 0 ) {
+            if( ! bin ) {
                 fprintf(stderr,"file '%s' has non-printable chars\n", filename);
+                fprintf(stderr,"   bad chars start at position %d\n", count);
+                if( p->debug ) fputs("   bad chars", stderr);
                 bin = 1;
             }
-            if( p->debug ) fprintf(stderr,": %d (0x%0x)",cp[count],cp[count]);
+            if( p->debug )
+                fprintf(stderr," : %d (0x%0x)", cp[count], 0xff & cp[count]);
         }
 
         if( cp[count] != '\r' ) continue;
 
-        if( count <= length && cp[count+1] == '\n' )      /* dos type */
+        if( count < length-1 && cp[count+1] == '\n' )      /* dos type */
         {
-            if( p->debug )
+            if( p->debug ) {
+                if( bin ) fputc('\n', stderr);
                 fprintf(stderr,"found 0x0d0a char pair at offset %d\n",count);
-            if( bin )putchar('\n');
+            }
             printf("%s file type: DOS\n", filename);
             return 0;
         }
         else                                            /* mac type */
         {
-            if( p->debug )
+            if( p->debug ) {
+                if( bin ) fputc('\n', stderr);
                 fprintf(stderr,"found 0x0d char at offset %d\n",count);
-            if( bin )putchar('\n');
+            }
             printf("%s file type: MAC\n", filename);
             return 0;
         }
     }
 
-    if( bin )putchar('\n');
+    if( bin && p->debug )putchar('\n');
     printf("%s file type: UNIX\n", filename);
     return 0;
 }
 
-
 /*------------------------------------------------------------
  * process an ANALYZE file
  *
@@ -1138,7 +1191,8 @@ set_params( param_t * p, int argc, char * argv[] )
             CHECK_NEXT_OPT(ac, argc, "-prefix");
             p->prefix = argv[ac];
         }
-        else if ( ! strncmp(argv[ac], "-show_bad_all", 12 ) )
+        else if ( ! strncmp(argv[ac], "-show_bad_all", 12 ) ||
+                  ! strcmp (argv[ac], "-test") )
         {
             p->script = 0xff;   /* set all show bits */
         }
