@@ -115,6 +115,8 @@ typedef struct {
 #define NOPE 0
 #define YUP 1
 
+typedef enum { SUMA_SIDE_ERROR=-1, SUMA_NO_SIDE, SUMA_LR, SUMA_LEFT, SUMA_RIGHT } SUMA_SO_SIDE; 
+
 typedef enum { SUMA_NO_NUM_UNITS = 0, 
                SUMA_MM_UNITS,
                SUMA_P_VALUE_UNITS,
@@ -200,12 +202,23 @@ typedef struct {
    char *FileName;
    char *FileName_NoExt;
    char *FullName;
+   char *FullName_NoSel;
    char *Ext;
+   char *Prefix;
+   char *View;
+   char *TypeExt;
+   char *StorageModeName;
+   int   StorageMode;
    char *NodeSelect;
    char *ColSelect;
    char *RowSelect;
    char *RangeSelect;
    int only_index;
+   int OnDisk;
+   char *HeadName;
+   char *BrikName;
+   char *NameAsParsed;
+   char *cwdAsParsed;
 }SUMA_PARSED_NAME;
 
 
@@ -266,13 +279,15 @@ typedef enum {
    SUMA_XML_DSET,                  /* 16 */
    SUMA_XML_ASCII_DSET,            /* 17 */
    SUMA_XML_B64_DSET,              /* 18 */
-   SUMA_XML_B64GZ_DSET             /* 19 */
+   SUMA_XML_B64GZ_DSET,             /* 19 */
    
+   SUMA_N_DSET_FORMATS           /* leave at the end */
 } SUMA_DSET_FORMAT; /*!<  Format of data set
                           When you add a new element, modify functions
                           SUMA_Dset_Format_Name
                           SUMA_Dset_Format */ 
 #define SUMA_IS_DSET_1D_FORMAT(d) ( (d)==SUMA_1D || (d)==SUMA_1D_PURE || (d)==SUMA_1D_STDOUT || (d)==SUMA_1D_STDERR ) ? 1:0
+
 #define SUMA_IS_DSET_STDXXX_FORMAT(d) ( (d)>=SUMA_1D_STDOUT && (d)<= SUMA_1D_PURE_STDERR_TRANSPOSE) ? 1:0
 
 typedef enum {
@@ -300,10 +315,14 @@ typedef enum {
    SUMA_NODE_STRING,   /*!< Generic String */
    SUMA_NODE_SHORT,      /*!< Generic short */
    SUMA_NODE_DOUBLE,     /*!< Generic double */
+   SUMA_NODE_COMPLEX,     /*!< Generic complex */
    SUMA_NODE_XCORR,      /*!< Cross Correlation Coefficient */
    SUMA_NODE_ZSCORE,      /*!< Zscore */
    SUMA_NODE_VFR,       /* Visual Field Ration */
-   SUMA_NODE_PHASE     /* Phase of some sort */ 
+   SUMA_NODE_PHASE,     /* Phase of some sort */
+   SUMA_NODE_AREA,      /* Area associated with node*/
+   SUMA_NODE_VOLUME,    /* Volume associated with node */
+   SUMA_NODE_THICKNESS  /* Thickness (distance between isotopic nodes) */
 }  SUMA_COL_TYPE; /*!<  Column types.
                         When you add a new element, you need to modify
                         SUMA_AddColAttr
@@ -338,6 +357,7 @@ typedef struct {
                            see SUMA_ROI_DRAWING_TYPE*/
    char *idcode_str;
    char *Parent_idcode_str;
+   int Parent_side;
    char *Label;
    char *ColPlaneName;
    float FillColor[4];  /*!< RGB fill color */
@@ -672,7 +692,18 @@ typedef struct {
    #define SDSET_VECFILLED(dset) dset->dnel->vec_filled
    #define SDSET_NODEINDFILLED(dset) dset->inel->vec_filled
    #define SDSET_NODE_INDEX_COL(dset) ( (!dset || !dset->inel || !dset->inel->vec) ? NULL:(int*)(dset->inel->vec[0]) )
+   #define SDSET_COL(dset, icol) ( (!dset || !dset->dnel || !dset->dnel->vec) ? NULL:(dset->dnel->vec[icol]) )
 #endif
+
+#define DSET_MAX_NODE_INDEX(dset, MM) {\
+   double r[2]; int loc[2];   \
+   if (!SUMA_GetDsetNodeIndexColRange( dset, r, loc, 1)) {  \
+      MM = -1; \
+   } else { \
+      MM = (int)r[1];   \
+   }  \
+}
+
 /*!
    \brief Macros to access commonly used colorplane parameters
    DO NOT USE COLP_NODEDEF macro inside a loop where the returned
@@ -893,11 +924,32 @@ If you want to have some index before the entries, use SUMA_WRITE_IND_ARRAY_1D*/
    FILE * m_fp=NULL;\
    m_fp = (name) ? fopen((name),"w"): fopen("yougavemenoname","w");  \
    if (m_fp) { \
-      fprintf(m_fp,"# Output from %s, %d values (%d per line).", FuncName, Nel, m);  \
-      for (m_kkk=0; m_kkk<Nel; ++m_kkk) { if (!(m_kkk % m)) fprintf(m_fp,"\n"); fprintf(m_fp,"%d   ", (int)v[m_kkk]); }\
+      fprintf(m_fp,"# Output from %s, %d values (%d per line).", \
+                     FuncName, Nel, m);  \
+      for (m_kkk=0; m_kkk<Nel; ++m_kkk) { \
+         if (!(m_kkk % m)) fprintf(m_fp,"\n"); \
+         fprintf(m_fp,"%d   ", (int)v[m_kkk]); }\
       fclose(m_fp); \
    }  \
 }
+#define SUMA_WRITE_INT_ARRAY_AND_FLAG_1D(v,Nel,m,name,flg){  \
+   int m_kkk; \
+   FILE * m_fp=NULL;\
+   m_fp = (name) ? fopen((name),"w"): fopen("yougavemenoname","w");  \
+   if (m_fp) { \
+      fprintf(m_fp,"# Output from %s, %d values (%d per line).", \
+                     FuncName, Nel, m);  \
+      for (m_kkk=0; m_kkk<Nel; ++m_kkk) { \
+         if (!(m_kkk % m)) {  \
+            if (m_kkk) fprintf(m_fp,"%d   ", flg); \
+            fprintf(m_fp,"\n"); \
+         }  \
+         fprintf(m_fp,"%d   ", (int)v[m_kkk]); }\
+         fprintf(m_fp,"%d   ", flg); \
+      fclose(m_fp); \
+   }  \
+}
+
 /* Just like SUMA_WRITE_ARRAY_1D but ind contains indices
 to add at the beginning of each line.
 If ind is NULL, then the index will be the line number.
@@ -1163,7 +1215,16 @@ If ind is NULL, then the index will be the line number.
 }
 /*!
    replace a string positioned in column col, row row in NI_element * nel.
-   str is a copy of the pointer to that string and must not be freed
+   
+   This macro, when called from SUMA_AddColAtt_CompString ends up reporting
+   a NI_malloc post corruption, at least when starting the dot-product 
+   demo script "tcsh @bugy" under directory
+      /Volumes/Data/ziad/SUMA_test_dirs/suma/srf
+   I can't figure out how m_rc[(row)] is getting corrupted. For now, 
+   allocating for 10 chars more gets rid of the problem.
+   Set +10 to +1 if you want to reproduce the problem.
+   Search for comment tagged: TAG09102012 to find calling 
+   culprit. 
 */
 #define SUMA_NEL_REPLACE_STRING(nel, row, col, str) {\
    char **m_rc;   \
@@ -1171,7 +1232,7 @@ If ind is NULL, then the index will be the line number.
    if (m_rc[(row)]) NI_free(m_rc[(row)]); \
    m_rc[(row)] = NULL;\
    if (str) { \
-      m_rc[(row)] = (char*)NI_malloc(char, (strlen((str))+1)*sizeof(char));\
+      m_rc[(row)] = (char*)NI_malloc(char, (strlen((str))+10)*sizeof(char));\
       strcpy( m_rc[(row)], str );   \
    }  \
 }
@@ -1237,6 +1298,22 @@ If ind is NULL, then the index will be the line number.
    }  \
 }
 
+/*!
+   A typical check on the output status of a selected prefix for
+   surface datasets
+*/
+#define SUMA_CHECK_OUTPUT_SDSET_STATUS(Opref, InName, oform, pre, app, exists) {\
+   char *ooo = SUMA_OutputDsetFileStatus(Opref, InName, \
+                                          &(oform), pre, app, &(exists)); \
+   if (exists && !THD_ok_overwrite()) {   \
+      SUMA_S_Errv("Output file %s already exists.\n"  \
+                  "Pick another prefix or add -overwrite\n", ooo);   \
+      exit(1); \
+   }  \
+   if (ooo) SUMA_free(ooo); ooo=NULL;\
+}
+   
+
 #define SUMA_MAX_OPEN_DX_FIELD_COMPONENTS 500
 #define SUMA_MAX_OPEN_DX_FIELD_ATTRIBUTES 500
 #define SUMA_MAX_OPEN_DX_OBJECTS  500
@@ -1286,6 +1363,7 @@ NI_element *SUMA_FindDsetAttributeElement(SUMA_DSET *dset, char *attname);
 NI_element *SUMA_FindNgrAttributeElement(NI_group *ngr, char *attname);
 NI_element *SUMA_FindNgrDataElement(NI_group *ngr, char *nelname, char *typename);
 float SUMA_LatestVersionNumber(void);
+int SUMA_IcoNums(int depth, byte bin, char what);
 char * SUMA_Dset_Type_Name (SUMA_DSET_TYPE tp);
 SUMA_DSET_TYPE SUMA_Dset_Type (char *Name);
 char * SUMA_Col_Type_Name (SUMA_COL_TYPE tp);
@@ -1299,16 +1377,19 @@ int SUMA_ShowNel (void *nel);
 char *SUMA_NI_nel_Info (NI_element *nel, int detail);
 
 void SUMA_allow_nel_use(int al);
+int SUMA_AddDsetIndexCol(SUMA_DSET *dset, int *icolu);
 int SUMA_AddDsetNelCol ( SUMA_DSET *dset, char *col_label, 
                      SUMA_COL_TYPE ctp, void *col, 
                      void *col_attr, int stride);
-int SUMA_InsertDsetNelCol ( SUMA_DSET *dset, char *col_label, SUMA_COL_TYPE ctp, void *col, 
+int SUMA_InsertDsetNelCol ( SUMA_DSET *dset, char *col_label, 
+                     SUMA_COL_TYPE ctp, void *col, 
                      void *col_attr, int stride, int icol);
 int SUMA_AddNelCol ( NI_element *nel, char *col_label,
                      SUMA_COL_TYPE ctp, void *col, 
                      void *col_attr, int stride);
 int SUMA_AddDsetColAttr (SUMA_DSET *dset, char *col_label, 
-                     SUMA_COL_TYPE ctp, void *col_attr, int col_index, int insert_mode);
+                     SUMA_COL_TYPE ctp, void *col_attr, 
+                     int col_index, int insert_mode);
 int SUMA_AddDsetNodeIndexColAttr (SUMA_DSET *dset, char *col_label, 
                      SUMA_COL_TYPE ctp, void *col_attr );
 int SUMA_AddColAttr (NI_element *nel, char *col_label,
@@ -1337,6 +1418,7 @@ NI_element * SUMA_NewNel (SUMA_DSET_TYPE dtp, char* MeshParent_idcode,
                           char * geometry_parent_idcode, int N_el, 
                           char *name, char *thisidcode);
 SUMA_DSET_FORMAT SUMA_Dset_Format (char *Name);
+long SUMA_sdset_dnel_size(SUMA_DSET *dset);
 char * SUMA_Dset_Format_Name (SUMA_DSET_FORMAT fr);
 char *SUMA_HistString (char *CallingFunc, int N_arg, char **arg, char *sold);
 char * SUMA_GetNgrHist(NI_group *ngr);
@@ -1346,8 +1428,10 @@ int SUMA_RemoveDsetHist(SUMA_DSET *dset);
 int SUMA_AddNelHist(NI_element *nel, char *CallingFunc, int N_arg, char **arg);
 void SUMA_FreeDset(void *dset);
 SUMA_DSET * SUMA_FindDset_ns (char *idcode_str, DList *DsetList);
+SUMA_DSET * SUMA_FindDset2_ns (char *idcode_str, DList *DsetList, char *itype);
 DListElmt * SUMA_FindDsetEl_ns (char *idcode, DList *DsetList);
-SUMA_DSET * SUMA_FindDset_eng (char *idcode_str, DList *DsetList, DListElmt **elp);
+SUMA_DSET * SUMA_FindDset_eng (char *idcode_str, DList *DsetList, 
+                                 DListElmt **elp, char *itype);
 char *SUMA_DsetInfo (SUMA_DSET *dset, int detail);
 void SUMA_ShowDset (SUMA_DSET *dset, int detail, FILE *out);
 char *SUMA_ShowMeSome (void *dt, SUMA_VARTYPE tp, int N_dt, int mxshow, char *title);
@@ -1358,6 +1442,11 @@ SUMA_DSET * SUMA_CreateDsetPointer (
                               char *idcode_str,
                               char *domain_idcode_str,
                               int N_Alloc); 
+SUMA_DSET * SUMA_CreateFullDsetPointer (  
+                              char *filename, SUMA_DSET_TYPE tp,
+                              char *idcode,
+                              char *domain_idcode,
+                              int N_Alloc);
 int SUMA_InsertDsetPointer (SUMA_DSET **dset, DList *DsetList, int replace);
 int SUMA_DeleteDsetPointer (SUMA_DSET **dsetp, DList *DsetList);
 void * SUMA_GetCx(char *idcode_str, DList *DsetList, int ReturnDsetPointer) ;
@@ -1410,11 +1499,17 @@ SUMA_DSET *SUMA_Load1DDset_ns (char *Name, int verb);
 SUMA_DSET *SUMA_LoadDXDset_eng (char *Name, int verb);
 SUMA_DSET *SUMA_LoadDXDset_ns (char *Name, int verb);
 char *SUMA_RemoveDsetExtension_ns (char*Name, SUMA_DSET_FORMAT form);
-char *SUMA_RemoveDsetExtension_eng (char*Name, SUMA_DSET_FORMAT form);
-char * SUMA_WriteDset_ns (char *Name, SUMA_DSET *dset, SUMA_DSET_FORMAT form, int overwrite, int verb); 
-int SUMA_WriteDset_NameCheck_ns (char *Name, SUMA_DSET *dset, SUMA_DSET_FORMAT form, int verb, char **NameOutp); 
-int SUMA_WriteDset_NameCheck_eng (char *Name, SUMA_DSET *dset, SUMA_DSET_FORMAT form, int verb, char **NameOutp); 
-char * SUMA_WriteDset_eng (char *Name, SUMA_DSET *dset, SUMA_DSET_FORMAT form, int overwrite, int verb); 
+char *SUMA_RemoveDsetExtension_eng (char*Name, SUMA_DSET_FORMAT *form);
+char * SUMA_WriteDset_ns (char *Name, SUMA_DSET *dset, SUMA_DSET_FORMAT form, 
+                          int overwrite, int verb); 
+int SUMA_WriteDset_NameCheck_ns (char *Name, SUMA_DSET *dset, 
+                                 SUMA_DSET_FORMAT form, int verb, 
+                                 char **NameOutp); 
+int SUMA_WriteDset_NameCheck_eng (char *Name, SUMA_DSET *dset, 
+                                  SUMA_DSET_FORMAT form, int verb, 
+                                  char **NameOutp); 
+char * SUMA_WriteDset_eng (char *Name, SUMA_DSET *dset, SUMA_DSET_FORMAT form, 
+                           int overwrite, int verb, int rename_autoid); 
 SUMA_DSET * SUMA_far2dset_eng( char *FullName, char *dset_id, char *dom_id, 
                                  float **farp, int vec_len, int vec_num, 
                                  int ptr_cpy);
@@ -1430,6 +1525,9 @@ int SUMA_is_VFR_dset(SUMA_DSET *dset);
 NI_group *SUMA_NICmapToNICmap(NI_group *NIcmap);
 int * SUMA_UniqueValuesInLabelDset(SUMA_DSET *dset, int *N_unq);
 int SUMA_is_AllConsistentNumeric_dset(SUMA_DSET *dset, SUMA_VARTYPE *vtpp);
+int SUMA_GetConsistentColType_dset(SUMA_DSET *dset);
+int SUMA_is_AllConsistentColType_dset(SUMA_DSET *dset, SUMA_COL_TYPE ctpi); 
+int SUMA_is_AllConsistentCastType_dset(SUMA_DSET *dset, int typecast);
 int SUMA_is_AllNumeric_ngr(NI_group *ngr) ;
 int SUMA_is_AllNumeric_nel(NI_element *nel);
 int SUMA_is_TimeSeries_dset(SUMA_DSET *dset, double *TRp);
@@ -1438,10 +1536,14 @@ SUMA_Boolean SUMA_NewDsetID (SUMA_DSET *dset);
 SUMA_Boolean SUMA_NewDsetID2 (SUMA_DSET *dset, char *str);
 char *SUMA_DsetColStringAttrCopy(SUMA_DSET *dset, int i, 
                                  int addcolnum, char *attrname);
+char *SUMA_DsetColLabel(SUMA_DSET *dset, int i);
 char *SUMA_DsetColLabelCopy(SUMA_DSET *dset, int i, int addcolnum);
+int SUMA_FindDsetColLabeled(SUMA_DSET *dset, char *label);
 char **SUMA_AllDsetColLabels(SUMA_DSET *dset);
 char **SUMA_FreeAllDsetColLabels(char **);
 char *SUMA_ColLabelCopy(NI_element *nel, int i, int addcolnum);
+int SUMA_FloatScanDset ( SUMA_DSET *odset, int doNan, int doInf, 
+                         int zeroout, int fixrange);
 SUMA_DSET * SUMA_PaddedCopyofDset ( SUMA_DSET *odset, int MaxNodeIndex );
 SUMA_DSET * SUMA_MaskedCopyofDset(SUMA_DSET *odset, byte *rowmask, byte *colmask, int masked_only, int keep_node_index);
 SUMA_DSET * SUMA_MaskedByOrderedNodeIndexCopyofDset(
@@ -1490,7 +1592,7 @@ int *SUMA_CreateNodeIndexToRowIndexMap(SUMA_DSET *dset, int maxind,
                                        double *range);
 SUMA_DSET * SUMA_ngr_2_dset(NI_group *nini, int warn);
 SUMA_Boolean SUMA_LabelDset(SUMA_DSET *dset, char *lbl);
-SUMA_Boolean SUMA_RenameDset(SUMA_DSET *dset, char *filename);
+SUMA_Boolean SUMA_RenameDset(SUMA_DSET *dset, char *filename, int autoid);
 byte *SUMA_load_1D_n_mask(char *name, int N_Node, byte *omask, const char *oper, int *N_inmask);
 byte * SUMA_indexlist_2_bytemask(int *ind_list, int N_ind_list, int N_mask, int *N_inmask);  
 byte * SUMA_Meshbmask_2_IndexListbmask(
@@ -1538,21 +1640,56 @@ int NoSumaRcFound (void);
 void SUMA_ParseInput_basics_ns (char *argv[], int argc); 
 int SUMA_ParseInput_basics_eng (char *argv[], int argc); 
 void WorkErrLog_ns(void);
+
+#define SUMA_DSET_NAME_CHECK(prefix) { \
+   char *NameOut=NULL; SUMA_DSET_FORMAT form=SUMA_NO_DSET_FORMAT; \
+   if (!THD_ok_overwrite()) { \
+      form = SUMA_GuessFormatFromExtension(prefix, "jeveux.niml.dset" );  \
+      if (SUMA_WriteDset_NameCheck_s (prefix, NULL, form,  \
+                                  0, &NameOut)) {   \
+         SUMA_S_Errv("Dset %s already exists\n", NameOut); \
+         SUMA_free(NameOut); \
+         exit(1);  \
+      }  \
+   } if (NameOut) SUMA_free(NameOut); \
+}
+
 SUMA_FileName SUMA_StripPath (char *FileName);
+SUMA_DSET_FORMAT SUMA_FormatFromFormString(char *arg);
 SUMA_PARSED_NAME * SUMA_ParseFname (char *FileName, char *cwd);
+SUMA_PARSED_NAME * SUMA_DuplicateParsedName(SUMA_PARSED_NAME *pn);
+SUMA_PARSED_NAME * SUMA_ModifyParsedName (SUMA_PARSED_NAME *pn, 
+                                          char *what, char *val);
+char * SUMA_ModifyName(char *name, char *what, char *val, char *cwd);
+SUMA_PARSED_NAME * SUMA_ParseModifyName(char *name, 
+                                        char *what, char *val, char *cwd);
 char *SUMA_Extension(char *filename, char *ext, SUMA_Boolean Remove);
 SUMA_DSET_FORMAT SUMA_GuessFormatFromExtension(char *Name, char *fallbackname);
 const char *SUMA_ExtensionOfDsetFormat (SUMA_DSET_FORMAT form);
 SUMA_Boolean SUMA_isExtension(char *filename, char *ext);
 char * SUMA_CropExtension(char *filename, char *ext);
 void *SUMA_Free_Parsed_Name(SUMA_PARSED_NAME *Test);
+char *SUMA_OutputDsetFileStatus(char *prefix, char *inname, 
+                            SUMA_DSET_FORMAT *oform, 
+                            char *pre, char *app, int *exists);
 char *SUMA_FnameGet(char *Fname, char *sel, char *cwd);
 int SUMA_NumStringUnits (char *s, int marktip); 
+int SUMA_strtod(char *n, double *valp);
 int SUMA_StringToNum (char *s, void *vv, int N, int p);
+int SUMA_StringToNumSide (char *s, void *vv, int N, int p, int *sd);
 int SUMA_isNumString (char *s, void *p);
 int SUMA_CleanNumString (char *s, void *p);
+int SUMA_CleanNumStringSide (char *s, void *p);
 char *SUMA_copy_string(char *buf);
+char *SUMA_copy_quoted( char *s, char *eop, 
+                        char q1, char q2, int deblank,
+                        int withquotes, int *is_closed );
+char *args_in_quotes(char **argv, int *kar, int N_argv, char *opq, 
+                     char *cloq, int clearused);
+char *args_in_niml_quotes(char **argv, int *kar, int N_argv, int clearused);
+char *args_in_simple_quotes(char **argv, int *kar, int N_argv, int clearused);
 char * SUMA_append_string(char *s1, char *s2);
+char * SUMA_append_extension(char *s1, char *s2);
 char * SUMA_append_replace_string(  char *s1, char *s2, 
                                     char *Spc, int whichTofree);
 char * SUMA_append_replace_num(char *s1, char *form, double num, SUMA_VARTYPE tp, int whichTofree);
@@ -1590,9 +1727,12 @@ int SUMA_GetNodeIndex_FromNodeRow_eng(SUMA_DSET *dset, int row, int N_Node);
 double SUMA_GetDsetNodeValInCol2(SUMA_DSET *dset, int ind, 
                                  int node, int N_Node);
 NI_str_array *SUMA_free_NI_str_array(NI_str_array *nisa);
+NI_str_array *SUMA_NI_str_array(NI_str_array *clss, char *what, char *action); 
+int SUMA_NI_str_array_find( char *targ , NI_str_array *sar , 
+                            int partial, int ci);
 NI_str_array *SUMA_comp_str_2_NI_str_ar(char *s, char *sep);
+void SUMA_Show_NI_str_ar(NI_str_array *nisa, FILE *out);
 char *SUMA_NI_str_ar_2_comp_str (NI_str_array *nisa, char *sep);
-NI_str_array *SUMA_free_NI_str_array(NI_str_array *nisa);
 char *SUMA_Get_Sub_String(char *cs, char *sep, int ii);
 int SUMA_AddColAtt_CompString(NI_element *nel, int col, char *lbl, 
                               char *sep, int insert_mode);
@@ -1620,6 +1760,7 @@ char *SUMA_isEnv(char *env, char *sval);
 float SUMA_floatEnv(char *env, float defval);
 ENV_SPEC SUMA_envlistelement(int i);
 char * SUMA_EnvVal(char *env);
+int SUMA_EnvEquals(char *env, char *sval, byte ci, char *sep);
  
 /*********************** END Miscellaneous support functions **************************** */
 
@@ -1634,6 +1775,8 @@ int afni_write_gifti_surf( NI_group *aSO, char * fname,
 
 
 /******** END functions for surface structure  ******************** */
-
-
+int SUMA_init_GISET_setup(NI_stream nsg , NI_element *nel, GICOR_setup *giset,
+                          int bmode);   /* 17 Aug 2012 [rickr] */
+int SUMA_PopulateDsetsFromGICORnel(NI_element *nel, GICOR_setup *giset, 
+                                   SUMA_DSET **sdsetv);
 #endif
