@@ -69,8 +69,33 @@ ENTRY("THD_open_dataset") ;
      RETURN(dset) ;
    }
 
-   /*-- 04 Mar 2003: allow input of .1D files   --*/
-   /*--              which deals with [] itself --*/
+   /*-- allow filelist:FILENAME to work like tcat       23 Jul 2012 [rickr] */
+   if( ! strncmp(pathname,"filelist:",9) ) {
+     dset = THD_open_tcat( pathname ) ;
+     if( ISVALID_DSET(dset) &&
+        !ISVALID_MAT44(dset->daxes->ijk_to_dicom) )  /* 15 Dec 2005 */
+       THD_daxes_to_mat44(dset->daxes) ;
+     THD_patch_brickim(dset); RETURN(dset) ;
+   }
+
+   /*-- 04 Aug 2004: allow input of a list of datasets, separated by spaces --*/
+   /*  unless a count command is used inside the brackets 9 May 2007 drg*/
+   /* allow use of spaces with AFNI_PATH_SPACES_OK        2 May 2012 [rickr]  */
+   if( ! AFNI_yesenv("AFNI_PATH_SPACES_OK") && 
+         (strchr(pathname,' ') != NULL )    && 
+         (strstr(pathname,"[count ")==NULL) && 
+         (strstr(pathname,"[1dcat ")==NULL) ) {
+     dset = THD_open_tcat( pathname ) ;
+     if( ISVALID_DSET(dset) &&
+        !ISVALID_MAT44(dset->daxes->ijk_to_dicom) )  /* 15 Dec 2005 */
+       THD_daxes_to_mat44(dset->daxes) ;
+     THD_patch_brickim(dset); RETURN(dset) ;
+   }
+
+   /*-- 04 Mar 2003: allow input of .1D files     --*/
+   /*--              which deals with [] itself   --*/
+   /*-- 19 May 2012: moved after check for spaces 
+    *        [rickr] (to allow space cat of 1D)   --*/
 
    if( strstr(pathname,".1D") != NULL || strncmp(pathname,"1D:",3) == 0 ){
      dset = THD_open_1D( pathname ) ;
@@ -80,17 +105,6 @@ ENTRY("THD_open_dataset") ;
      if( dset != NULL ){ THD_patch_brickim(dset); RETURN(dset); }
    }
 
-   /*-- 04 Aug 2004: allow input of a list of dataset, separated by spaces --*/
-   /*  unless a count command is used inside the brackets 9 May 2007 drg*/
-   if((strchr(pathname,' ') != NULL ) && 
-      (strstr(pathname,"[count ")==NULL)){
-     dset = THD_open_tcat( pathname ) ;
-     if( ISVALID_DSET(dset) &&
-        !ISVALID_MAT44(dset->daxes->ijk_to_dicom) )  /* 15 Dec 2005 */
-       THD_daxes_to_mat44(dset->daxes) ;
-     THD_patch_brickim(dset); RETURN(dset) ;
-   }
-
    /*-- find the opening "[" and/or "<" --*/
 
    cpt = strstr(pathname,"[") ;
@@ -159,29 +173,12 @@ ENTRY("THD_open_dataset") ;
       dset->dblk->master_top = 0.0 ;
    }
 
+   /* moved to new function that goes after labels   17 Apr 2012 [rickr] */
    if( bpt != NULL ){
-      char *dpt = strstr(bpt,"..") ;
-#if 0
-fprintf(stderr,"bpt=%s\n",bpt) ;
-#endif
-      if( dpt != NULL ){
-#if 0
-fprintf(stderr,"dpt=%s\n",dpt) ;
-#endif
-         kk  = sscanf( bpt+1 , "%f" , &bot ) ;
-         kk += sscanf( dpt+2 , "%f" , &top ) ;
-         if( kk == 2 && bot <= top ){
-            dset->dblk->master_bot = bot ;
-            dset->dblk->master_top = top ;
-         } else {
-            dset->dblk->master_bot = 1.0 ;
-            dset->dblk->master_top = 0.0 ;
-         }
-      } else {/* ZSS: Why not allow for <val> ? */
-         kk  = sscanf( bpt+1 , "%f" , &bot ) ;
-         dset->dblk->master_bot = bot ;
-         dset->dblk->master_top = bot ;
-      }
+      bot = 1.0 ; top = 0.0 ; /* in case of odd failure */
+      MCW_get_angle_range(dset, bpt, &bot, &top) ;
+      dset->dblk->master_bot = bot ;
+      dset->dblk->master_top = top ;
    }
 
    /* modify the dataset according to the selector string */
@@ -668,6 +665,7 @@ ENTRY("THD_copy_dset_subs");
    like "name[1]", "name[2]", "name[3]".  The reason for this is for older
    programs like 3dttest that operate on a list of single brick datasets.
    [19 Jul 2007 - RWCox]
+   Modified to allow use of labelized lists (per ZSS) -- 09 Dec 2011.
 --------------------------------------------------------------------------*/
 
 THD_string_array * THD_multiplex_dataset( char *pathname )
@@ -675,13 +673,24 @@ THD_string_array * THD_multiplex_dataset( char *pathname )
    char *cpt , *pname , *bname ;
    int  *ivlist=NULL , ii ;
    THD_string_array *sar ;
+   THD_3dim_dataset *qset ;
 
 ENTRY("THD_multiplex_dataset") ;
    if( pathname == NULL ) RETURN(NULL) ;
    cpt = strstr(pathname,"[") ; if( cpt == NULL ) RETURN(NULL) ;
 
    bname = strdup(pathname) ; cpt = strstr(bname,"[") ; *cpt = '\0' ;
-   ivlist = MCW_get_intlist( 999999 , cpt+1 ) ;
+   if( STRING_HAS_SUFFIX(bname,".mnc")    ||
+       STRING_HAS_SUFFIX(bname,".mri")    ||
+       STRING_HAS_SUFFIX(bname,".svl")      ){
+     ERROR_message("Can't use sub-brick selectors '[...]' on dataset %s",bname) ;
+     free(bname) ; RETURN(NULL) ;
+   }
+   qset = THD_open_one_dataset(bname) ;  /* 09 Dec 2011 */
+   if( !ISVALID_DSET(qset) ){
+     ERROR_message("Can't open dataset %s",bname) ; free(bname) ; RETURN(NULL) ;
+   }
+   ivlist = MCW_get_thd_intlist(qset,cpt+1) ; DSET_delete(qset) ;
    if( ivlist == NULL || ivlist[0] == 0 ){ free(bname); RETURN(NULL); }
 
    INIT_SARR(sar) ;
